/**
 * 
 */
package gov.nist.p25.issi.issiconfig;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;

import gov.nist.p25.issi.issiconfig.GlobalTopologyParser.MyResolver;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfiguration;

import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author M. Ranganathan
 * 
 */
public class SystemTopologyParser extends DefaultHandler implements
		XMLTagsAndAttributes {
	TopologyConfig topologyConfig;

	int wacnId;

	int systemId;

	private String configFile;

	private XMLReader saxParser;

	private ISSITesterConfiguration issiTesterConfiguration;

	private static Logger logger = Logger.getLogger(SystemTopologyParser.class);
	
	
	// /////////////////////////////////////////////////////////////////////////////////////////
	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/systemtopology.dtd")) {
				FileReader fileReader = new FileReader(new File(
						"dtd/systemtopology.dtd"));
				InputSource inputSource = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}

	public SystemTopologyParser(ISSITesterConfiguration config) throws Exception {
		
		if ( config == null )throw new NullPointerException("Null tester config arg!");
		
		topologyConfig = new TopologyConfig();

		try {
			
			this.issiTesterConfiguration = config;

			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
			this.saxParser.setEntityResolver(new MyResolver());

			saxParser.setContentHandler(this);

			saxParser
					.setFeature("http://xml.org/sax/features/validation", true);
			

		} catch (Exception pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();
			throw pce;
		}

	}

	@Override
	public void startElement(String namespaceURI, String local, String name,
			Attributes attrs) throws SAXException {
		try {
			if (name.compareToIgnoreCase("wacn-config") == 0) {
				if (attrs.getValue(WACN_NAME) == null)
					throw new SAXException("Missing a required attribute");
				if (attrs.getValue(WACN_ID) == null)
					throw new SAXException("Missing a required attribute : "
							+ WACN_ID);
				this.wacnId = Integer.parseInt(attrs.getValue(WACN_ID), 16);
				if (topologyConfig.wacnNameTable.containsKey(attrs
						.getValue(WACN_NAME)))
					throw new SAXException("Duplicate WACN Name");
				String wacnName = attrs.getValue(WACN_NAME);
				WacnConfig wacnConfig = new WacnConfig(wacnName,wacnId);
				topologyConfig.wacnNameTable.put(attrs.getValue(WACN_NAME),
						wacnConfig);

			} else if (name.compareToIgnoreCase("systemconfig") == 0) {
				String sName = attrs.getValue(SYSTEM_NAME);
				if (sName == null)
					throw new SAXException("Missing a required attribute "
							+ SYSTEM_NAME);
				if (attrs.getValue(SYSTEM_ID) == null)
					throw new SAXException("MIssing a required attribute "
							+ SYSTEM_ID);
				int systemId = Integer.parseInt(attrs.getValue(SYSTEM_ID), 16);
				this.systemId = systemId;
				
				String wacnName = attrs.getValue(WACN_NAME);
				if (wacnName == null && topologyConfig.wacnNameTable.size() > 1)
					throw new SAXException("Missing a required attribute"
							+ WACN_NAME);
				
				WacnConfig wacnConfig = topologyConfig.getWacnConfig(wacnName);
				SystemConfig sysConfig = new SystemConfig(this.topologyConfig,
						sName, wacnConfig, systemId);
				String systemName = sysConfig.getSystemName();

				logger.debug("putting  " + systemName + " into systemTable ");
				
				topologyConfig.addSystemConfiguration(sysConfig);
				

			} else if (name.compareToIgnoreCase("rfssconfig") == 0) {
				if (attrs.getValue(RFSS_ID) == null) {
					throw new SAXException("Missing a required attribute "
							+ RFSS_ID);
				}
				int rfssId = Integer.parseInt(attrs.getValue(RFSS_ID), 16);
				if (attrs.getValue(RFSS_NAME) == null) {
					throw new SAXException("Missing a required attribute "
							+ RFSS_NAME);
				}
				String rfssName = attrs.getValue(RFSS_NAME);
				boolean isEmulated = attrs.getValue("emulated") == null ? true
						: attrs.getValue("emulated").equals("true");

				String ipAddress = attrs.getValue(IP_ADDRESS);
				

				String tag = RfssConfig.generateTag(rfssName);
				
				if ( isEmulated  )	 {
					ipAddress = this.issiTesterConfiguration.getIpAddressByName(tag);
				}
				
					
				
				if ( ipAddress == null ) {
					ipAddress = "X.X.X.X";
					logger.error("Could not find an emulator node definition for RFSS " + rfssName + 
							"\n missing reference " + ipAddress + 
							"\n check the systemtopology and the RFSS tester configuration files. Assigning dummy address.");
				}
				
		
				
				int port = Integer.parseInt(attrs.getValue(PORT));

				if (attrs.getValue(SYSTEM_NAME) == null) {
					throw new SAXException("Missing a required attribute : "
							+ SYSTEM_NAME);
				}

				SystemConfig sysconfig = topologyConfig.getSystemConfigByName(attrs
							.getValue(SYSTEM_NAME));
				wacnId = sysconfig.getWacnId();
			
				String color = attrs.getValue(COLOR) == null? "BLACK" : attrs.getValue(COLOR);
				
						

				// The default values.
				RfssConfig rfssConfig = new RfssConfig(this.topologyConfig, 
						sysconfig, rfssId, ipAddress, port, rfssName,isEmulated);
				
				rfssConfig.setColor(color);
				
				
				rfssConfig.tag = tag;
				
				RfssConfig duplicate = this.topologyConfig.getRfssConfig(ipAddress, port);
				if ( duplicate != null) {
					throw new Exception("Duplicate Rfss found in the table");
				}
					
				topologyConfig.addRfssConfig(rfssConfig);
				
				sysconfig.addRfss(rfssConfig);
			}
		} catch (Exception ex) {
			logger.error("Unexpected excepton ", ex);
			throw new SAXException("Exception parsing " + configFile, ex);
		}
	}
	
	

	public TopologyConfig parse(String fileName) throws IOException,
			SAXException {
		logger.debug("Parsing " + fileName);
		this.configFile = fileName;
		this.saxParser.parse(new File(fileName).toURI().toURL().toString());
		return this.topologyConfig;
	}
	
	

}
