/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body;

import gov.nist.p25.issi.issiconfig.ISSIConstants;
import gov.nist.p25.issi.p25body.serviceprofile.group.GroupServiceProfile;
import gov.nist.p25.issi.rfss.ProtocolObjects;

import java.text.ParseException;

import javax.sip.header.ContentTypeHeader;
import javax.sip.header.HeaderFactory;

/**
 * @author M. Ranganathan
 * 
 */
public class GroupServiceProfileContent extends Content {
	private GroupServiceProfile groupServiceProfile;

	private static HeaderFactory headerFactory = ProtocolObjects.headerFactory;

	GroupServiceProfileContent(ContentTypeHeader contentType,
			String groupServiceProfileContent) throws ParseException {
		super(contentType, groupServiceProfileContent);
		this.groupServiceProfile = GroupServiceProfile
				.createGroupServiceProfile(groupServiceProfileContent);
		super.setContent(groupServiceProfileContent);
	}

	public static GroupServiceProfileContent createGroupServiceProfileContent(
			String groupServiceProfileContent) throws ParseException {

		return new GroupServiceProfileContent(p25ContentTypeHeader,
				groupServiceProfileContent);
	}

	/**
	 * @return Returns the groupServiceProfile.
	 */
	public GroupServiceProfile getGroupServiceProfile() {
		return groupServiceProfile;
	}

	@Override
	public String toString() {

		if (super.boundary == null)
			return this.groupServiceProfile.toString();
		else {
			return new StringBuffer().append(
					super.boundary + "\r\n" + getContentTypeHeader() + "\r\n"
							+ this.groupServiceProfile.toString()).toString();

		}
	}

	@Override
	public String toRtfString() {
		if (super.boundary == null)
			return this.groupServiceProfile.toRtfString();
		else {
			return new StringBuffer().append(
					HEADER_START +
					super.boundary + HEADER_END + "\r\n" + HEADER_START + getContentTypeHeader() + HEADER_END+ "\r\n"
							+ this.groupServiceProfile.toRtfString()).toString();

		}
	}
	
	@Override
	public boolean isDefault() {
		return false;
	}

	@Override
	public boolean match(Content template) {
			return true;
	}
}
