/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body;

import gov.nist.p25.issi.p25body.serviceprofile.user.UserServiceProfile;

import java.text.ParseException;

import javax.sip.header.ContentTypeHeader;

/**
 * @author M. Ranganathan
 * 
 */
public class UserServiceProfileContent extends Content {
	private UserServiceProfile userServiceProfile;

	UserServiceProfileContent(ContentTypeHeader contentType,
			String userServiceProfileContent) throws ParseException {
		super(contentType, userServiceProfileContent);
		this.userServiceProfile = UserServiceProfile
				.createUserServiceProfile(userServiceProfileContent);
		super.setContent(userServiceProfileContent);
	}

	/**
	 * @return Returns the userServiceProfile.
	 */
	public UserServiceProfile getUserServiceProfile() {
		return userServiceProfile;
	}

	/**
	 * create an instance of this object.
	 */
	public UserServiceProfileContent createUserServiceProfileContent(
			String uspcontent) throws ParseException {
		return new UserServiceProfileContent(p25ContentTypeHeader, uspcontent);
	}

	@Override
	public String toString() {

		if (super.boundary == null)
			return this.userServiceProfile.toString();
		else {
			return new StringBuffer().append(
					super.boundary + "\r\n" + getContentTypeHeader() + "\r\n"
							+ this.userServiceProfile.toString()).toString();

		}
	}

	@Override
	public String toRtfString() {
		
		if (super.boundary == null)
			return this.userServiceProfile.toRtfString();
		else {
			return new StringBuffer().append(
					HEADER_START +
					super.boundary + 
					HEADER_END + 
					"\r\n" + 
					HEADER_START+ getContentTypeHeader() + HEADER_END + LINE_FEED + 
					"\r\n"
							+ this.userServiceProfile.toRtfString()).toString();

		}
	}
	
	@Override
	public boolean match(Content other) {
		if ( !(other instanceof UserServiceProfileContent)) return false;
		UserServiceProfileContent that = (UserServiceProfileContent) other;
		return this.userServiceProfile.match(that.userServiceProfile);
		
	}

	@Override
	public boolean isDefault() {
		
		return false;
	}
}
