/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile;

/**
 * The Authentication Policy attribute specifies the authentication policy to be
 * applied to the unit.
 * 
 * @author M. Ranganathan
 * 
 */
public enum AuthenticationPolicyType {
	NONE(0, "No policy is required"), REQUIRED(1,
			"authentication is required when the home is reachable"), REJECT_IF_NOT_CURRENT(
			2,
			"authentication is always required and authentication SHOULD be rejected "
					+ "if the serving RFSS does not have the current parameters of the SU.");

	private String description;

	public int intValue;

	AuthenticationPolicyType(int dup, String desc) {
		this.intValue = dup;
		this.description = desc;

	}

	public int getIntValue() {
		return intValue;
	}

	@Override
	public String toString() {
		return description;
	}

	public static AuthenticationPolicyType getInstance(int intval) {
		for (AuthenticationPolicyType d : AuthenticationPolicyType.values())
			if (d.intValue == intval)
				return d;
		throw new IllegalArgumentException("Value out of range");
	}

}
