/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile;

public enum InterruptModeType {
	//(NeverAllowed|PriorityBasedAllowed|AlwaysAllowd)
	AUDIO_INTERRRUPT_NEVER_ALLOWED(0, "0: Audio Interrupt Never Allowed","NeverAllowed"),
	AUDIO_INTERRRUPT_BASED_ON_PRIORITY(
			1, "1: Audio Interrupt Based On Priority","PriorityInterruptAllowed"), AUDIO_INTERRRUPT_ALWAYS_ALLOWED(
			2, " 2: Audio Interrupt Always Allowed","AlwaysAllowed");
	int intValue;

	private String description;
	
	private String shortName;

	public int getIntValue() {
		return this.intValue;
	}

	public static InterruptModeType getModeValues(int modeValue) {
		for (InterruptModeType mv : InterruptModeType.values()) {
			if (modeValue == mv.intValue)
				return mv;
		}
		throw new IllegalArgumentException("Illegal arg : " + modeValue);
	}

	private InterruptModeType(int modeValue, String description, String shortName) {
		this.intValue = modeValue;
		this.description = description;
		this.shortName = shortName;
	}

	@Override
	public String toString() {
		return this.description;

	}

	/**
	 * Find mode corresponding to xml attribute.
	 * 
	 * @param modeValue
	 * @return
	 */
	public static InterruptModeType getValue(String modeValue) {
		
		for (InterruptModeType mv : InterruptModeType.values()) {
			if (modeValue.equals(mv.shortName))
				return mv;
		}
		throw new IllegalArgumentException("Illegal arg : " + modeValue);
		
	}
}