package gov.nist.p25.issi.p25body.serviceprofile;

import gov.nist.p25.issi.issiconfig.XMLTagsAndAttributes;
import gov.nist.p25.issi.p25body.serviceprofile.group.GroupServiceProfile;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.util.Hashtable;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;

public abstract class ServiceProfile {
	private static Logger logger = Logger.getLogger(GroupServiceProfile.class);

	protected Hashtable<String, ServiceProfileLine> profileLines;

	/**
	 * This table is used by the factory to create a structure of the
	 * appropriate type when parsing a line.
	 */
	protected static Hashtable<String, Class<? extends ServiceProfileLine>> profileLineClasses = new Hashtable<String, Class<? extends ServiceProfileLine>>();

	protected static Class<? extends ServiceProfileLine> getProfileLineClass(
			String fieldName) {
		return profileLineClasses.get(fieldName.toLowerCase());
	}

	/**
	 * Get a profile line of a specified name.
	 * 
	 * @param profileLineName --
	 *            name of the profile line.
	 */
	protected ServiceProfileLine getByName(String name) {
		return this.profileLines.get(name.toLowerCase());
	}

	/**
	 * Add a group service profile line.
	 * 
	 * @param serviceProfileLine --
	 *            a profile line to add.
	 */
	protected void addProfileLine(ServiceProfileLine serviceProfileLine) {

		profileLines.put(serviceProfileLine.getName().toLowerCase(), serviceProfileLine);
	}

	public String toXmlString() {

		StringBuffer sb = new StringBuffer().append("<").append(getXMLTag())
				.append(">").append("\n");
		sb.append("<![CDATA[\n");
		sb.append(this.toString());
		sb.append("]]>");
		sb.append("\n</").append(getXMLTag()).append(">\n");
		return sb.toString();
	}

	protected static void parseProfileLines(ServiceProfile retval,
			String profile) throws ParseException {
		// Be generous in what you accept. Unixen use CR for line termination.
		// When stored in
		// a file and edited, it could easily replace CRLF with CR and hence we
		// accept both
		// CRLF and CR as terminators.
		String buffer = profile.replaceAll("\r\n", "\n");
		String[] profileLines = buffer.split("\n");

		for (String line : profileLines) {
			String trimmedLine = line.trim();
			if (!trimmedLine.equals("")) {
				ServiceProfileLine profileLine = retval
						.createServiceProfileLine(line.trim());
				retval.addProfileLine(profileLine);
			}
		}
	}

	/*
	 * Absract methods.
	 */

	/**
	 * Get the profile as an xml tag.
	 */
	protected abstract String getXMLTag();
	
	/**
	 * Get the profile as an RTF String.
	 * 
	 * @return the profile as an RTF String.
	 * 
	 */
	public abstract String toRtfString();

	/**
	 * match the service profile line with a template.
	 * 
	 * @param template -- the template to match against.
	 * 
	 * @return true if the match succeeds.
	 */
	public abstract boolean match(Object template);
	
	protected abstract ServiceProfileLine createServiceProfileLine(
			String serviceProfileLine) throws ParseException;

}
