/**
 *This software was developed by employees of the National Institute of
 Standards and Technology (NIST), an agency of the Federal Government.
 Pursuant to title 15 Untied States Code Section 105, works of NIST
 employees are not subject to copyright protection in the United States
 and are considered to be in the public domain.  As a result, a formal
 license is not needed to use the software.


 The ISSI Emulator software is provided by NIST as a service and is expressly
 provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 AND DATA ACCURACY.  NIST does not warrant or make any representations
 regarding the use of the software or the results thereof, including but
 not limited to the correctness, accuracy, reliability or usefulness of
 the software.


 Permission to use this software is contingent upon your acceptance
 of the terms of this agreement
 .

 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * 
 * The Group Access Permission attribute defines whether or not the group is
 * allowed to make non-emergency calls at the serving RFSS. It has the following
 * format in ABNF notation: <br/> access-permission = g-access: / g-as: boolean
 * CRLF <br/> where 0 is used to indicate that non-emergency calls on the group
 * MAY NOT be placed from the serving RFSS, and 1 is used to indicate that
 * non-emergency calls MAY be placed. If the Access Permission attribute is not
 * present, non-emergency calls MAY be placed from the serving RFSS.
 * 
 * @author M. Ranganathan
 * 
 */
public class AccessPermission extends ServiceProfileLine {

	public static final String NAME = "g-access";

	public static final String SHORTNAME = "g-as";

	public static final boolean DEFAULTVALUE = true;

	private boolean isNonEmergencyCallingAllowed;

	/**
	 * Default constructor
	 * 
	 */
	AccessPermission() {
		super(NAME, DEFAULTVALUE);
		isNonEmergencyCallingAllowed = DEFAULTVALUE;
	}

	AccessPermission(String permission) throws ParseException {

		super(NAME, permission);

		if (permission.equals("1"))
			isNonEmergencyCallingAllowed = true;
		else if (permission.equals("0"))
			isNonEmergencyCallingAllowed = false;
		else
			throw new ParseException("Permission is invalid ", 0);

	}

	AccessPermission(boolean accessPermission) {
		super(NAME, accessPermission);
		this.isNonEmergencyCallingAllowed = accessPermission;

	}

	/**
	 * @param isNonEmergencyCallingAllowed
	 *            The isNonEmergencyCallingAllowed to set.
	 */
	public void setNonEmergencyCallingAllowed(
			boolean isNonEmergencyCallingAllowed) {
		this.isNonEmergencyCallingAllowed = isNonEmergencyCallingAllowed;
		super.setValue(isNonEmergencyCallingAllowed);
		;
	}

	/**
	 * @return Returns the isNonEmergencyCallingAllowed.
	 */
	public boolean isNonEmergencyCallingAllowed() {
		return isNonEmergencyCallingAllowed;
	}
	
	public static AccessPermission createFromXmlAttribute(String accessPermission) {
		//accessPermission (EmergencyOnly|NonEmergencyAllowed) 
		AccessPermission retval = new AccessPermission();
		if ( accessPermission.equals("EmergencyOnly")) {
			retval.setNonEmergencyCallingAllowed(false);
		} else {
			retval.setNonEmergencyCallingAllowed(true);
		}
		return retval;
		
	}

}
