/*
 *This software was developed by employees of the National Institute of
 Standards and Technology (NIST), an agency of the Federal Government.
 Pursuant to title 15 Untied States Code Section 105, works of NIST
 employees are not subject to copyright protection in the United States
 and are considered to be in the public domain.  As a result, a formal
 license is not needed to use the software.


 The ISSI Emulator software is provided by NIST as a service and is expressly
 provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 AND DATA ACCURACY.  NIST does not warrant or make any representations
 regarding the use of the software or the results thereof, including but
 not limited to the correctness, accuracy, reliability or usefulness of
 the software.


 Permission to use this software is contingent upon your acceptance
 of the terms of this agreement
 .

 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * 
 * The Announcement Group attribute defines the announcement group for the
 * group. It has the following format in ABNF notation: <br/> announcement-group =
 * g-agroup: / g-ag: 12HEX-DIGIT CRLF <br/> where the 12HEX-DIGIT represent the
 * fully-qualified hexadecimal form of the AGID. If no announcement group
 * attribute is included, then the group has no announcement group.
 * 
 * @author M. Ranganathan
 * 
 */
public class AnnouncementGroup extends ServiceProfileLine {

	public static final String NAME = "g-agroup";

	public static final String SHORTNAME = "g-ag";

	private long groupId;

	/**
	 * Constructor with supplied group Id as an int.
	 */
	AnnouncementGroup(long groupId) {
		super(NAME, groupId);
		this.groupId = groupId;
	}

	/**
	 * Constructor
	 * 
	 * @param groupId --
	 *            group id for the announcement group.
	 * @throws ParseException --
	 *             if not a properly formatted hex string.
	 */
	AnnouncementGroup(String groupId) throws ParseException {
		super(NAME, groupId);
		try {
			this.groupId = Long.parseLong(groupId, 16);
		} catch (NumberFormatException ex) {
			throw new ParseException("Invalid announcement group id ["
					+ groupId + "]", 0);
		}
	}

	/**
	 * get the group ID.
	 */
	public long getGroupId() {
		return groupId;
	}

	/**
	 * Set the group Id.
	 * 
	 * @param groupId
	 */
	public void setGroupId(long groupId) {
		this.groupId = groupId;
		super.setValue(groupId);
	}

}
