/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * @author M. Ranganathan
 * 
 */
public class ConfirmedCallSetupTime extends ServiceProfileLine {
	public static final String NAME = "g-ccsetupT";

	public static final String SHORTNAME = "g-c";

	public static final int DEFAULTVALUE = 0;

	private int setupTime = DEFAULTVALUE;

	ConfirmedCallSetupTime(String hangtime) throws ParseException {
		super(NAME, hangtime);
		try {
			setupTime = new Integer(hangtime).intValue();
			if (setupTime > 32766 || setupTime < 0)
				throw new ParseException("Value out of range [" + setupTime
						+ "]", 0);
		} catch (NumberFormatException ex) {
			throw new ParseException("Invalid integer specified [" + hangtime
					+ "]", 0);
		}
	}

	ConfirmedCallSetupTime(int csetupTime) {
		super(NAME, csetupTime);
		if (csetupTime > 32766 || csetupTime < 0)
			throw new IllegalArgumentException("out of range");
		this.setupTime = csetupTime;
	}

	public ConfirmedCallSetupTime() {
		super(NAME, DEFAULTVALUE);
		this.setupTime = DEFAULTVALUE;
	}

	/**
	 * @return Returns the confirmed call setup time.
	 */
	public int getSetupTime() {
		return setupTime;
	}

	/**
	 * Set the call setup time.
	 */
	public void setSetupTime(int setupTime) {
		if (setupTime > 32766 || setupTime < 0)
			throw new IllegalArgumentException("Value out of range ["
					+ setupTime + "]");
		super.setValue(setupTime);
		this.setupTime = setupTime;

	}

	/**
	 * return true if setup time is infinite.
	 */
	public boolean isInfiniteSetupTime() {
		return this.setupTime == 32766;
	}

	/**
	 * return true if group calls do not need to be confirmed.
	 */
	public boolean isUnconfirmed() {
		return this.setupTime == 0;
	}

	

}
