/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * Emergency Capable (g-ecap). The Emergency Capable attribute defines whether
 * or not emergency calls can be made on the group. If the emergency capable
 * attribute is not provided, emergency calls are allowed from the serving RFSS.
 * 
 * @author M. Ranganathan
 * 
 * 
 * 
 */
public class EmergencyCapable extends ServiceProfileLine {

	private boolean emergencyCapable = DEFAULTVALUE;

	public static final String NAME = "g-ecap";

	public static final String SHORTNAME = "g-ec";

	public static final boolean DEFAULTVALUE = true;

	EmergencyCapable() {
		super(NAME, "1");
		emergencyCapable = DEFAULTVALUE;
	}

	EmergencyCapable(String ecapString) throws ParseException {
		super(NAME, ecapString);
		if (ecapString == null)
			throw new NullPointerException("Null arg");
		this.emergencyCapable = super.getBoolFromString(ecapString);

	}
	
	public static EmergencyCapable createFromXmlAttribute (String  value) {
		EmergencyCapable retval = new EmergencyCapable();
		if ( value.equals("EmergencyCallsAllowed")) {
			retval.setEmergencyCapable(true);
		} else {
			retval.setEmergencyCapable(false);
		}
		return retval;
	}

	/**
	 * @param emergencyCapable
	 *            The emergencyCapable to set.
	 */
	public void setEmergencyCapable(boolean emergencyCapable) {
		this.emergencyCapable = emergencyCapable;
		super.setValue(emergencyCapable);
	}

	/**
	 * @return Returns the emergencyCapable.
	 */
	public boolean isEmergencyCapable() {
		return emergencyCapable;
	}

}
