/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * @author M. Ranganathan
 * 
 */
public class EmergencyPreemption extends ServiceProfileLine {
	public static final String NAME = "g-eprempt";

	public static final String SHORTNAME = "g-ep";

	private boolean ruthlessOnEmergency = DEFAULTVALUE;

	public static final boolean DEFAULTVALUE = false;

	EmergencyPreemption() {
		super(NAME, DEFAULTVALUE);
		this.ruthlessOnEmergency = DEFAULTVALUE;
	}

	EmergencyPreemption(String ruthlessFlag) throws ParseException {
		super(NAME, ruthlessFlag);
		if (ruthlessFlag.equals("1"))
			ruthlessOnEmergency = true;
		else if (ruthlessFlag.equals("0"))
			ruthlessOnEmergency = false;
		else
			throw new ParseException("Invalid flag " + ruthlessFlag, 0);
	}

	/**
	 * @param ruthlessOnEmergency
	 *            The ruthlessPreemptionAppliedOnEmergencyCalls to set.
	 */
	public void setRuthlessOnEmergency(boolean ruthlessOnEmergency) {
		this.ruthlessOnEmergency = ruthlessOnEmergency;
		super.setValue(ruthlessOnEmergency);
	}

	/**
	 * @return Returns the ruthlessPreemptionAppliedOnEmergencyCalls flag.
	 */
	public boolean isRuthlessOnEmergency() {
		return ruthlessOnEmergency;
	}

	public static EmergencyPreemption createFromXmlAttribute(String attr) {
		EmergencyPreemption retval = new EmergencyPreemption();
		if ( attr.equals("NonRuthless")) {
			retval.setRuthlessOnEmergency(false);
		} else {
			retval.setRuthlessOnEmergency(true);
		}
		return retval;
	}

}
