/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.SecurityLevelType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * @author M. Ranganathan
 * 
 */
public class GroupSecurityLevel extends ServiceProfileLine {
	public static final String NAME = "g-sec";

	public static final String SHORTNAME = "g-s";

	public static final SecurityLevelType DEFAULTVALUE = SecurityLevelType.SECURE_AND_CLEAR_CALLS;

	private SecurityLevelType level = DEFAULTVALUE;

	/**
	 * Default constructor -- assigns the default group security level.
	 * 
	 */
	GroupSecurityLevel() {
		super(NAME, DEFAULTVALUE.getIntValue());
		this.level = DEFAULTVALUE;

	}

	/**
	 * Constructor taking the level String as an argument
	 * 
	 * @param level --
	 *            level 1,2,3
	 * @throws ParseException
	 */
	GroupSecurityLevel(String level) throws ParseException {
		super(NAME, level);
		try {
			int lval = Integer.parseInt(level);
			if (lval < 1 || lval > 3)
				throw new ParseException("Value out of range", 0);
			this.level = SecurityLevelType.getInstance(lval);
		} catch (NumberFormatException ex) {
			throw new ParseException("Illegal value for level ", 0);
		}

	}

	GroupSecurityLevel(SecurityLevelType securityLevel) {
		super(NAME, new Integer(securityLevel.getIntValue()).toString());
		this.level = securityLevel;
	}

	public SecurityLevelType getLevel() {
		return level;
	}

	public void setLevel(SecurityLevelType level) {
		this.level = level;
		super.setValue(level.getIntValue());
	}

	/**
	 * Create (parse) from xml attributes.
	 * 
	 * @param level
	 * @return
	 */
	public static ServiceProfileLine createFromXmlAttributes(String level) {
		return new GroupSecurityLevel(SecurityLevelType.getInstance(level));
	}

}
