/*
 *This software was developed by employees of the National Institute of
 Standards and Technology (NIST), an agency of the Federal Government.
 Pursuant to title 15 Untied States Code Section 105, works of NIST
 employees are not subject to copyright protection in the United States
 and are considered to be in the public domain.  As a result, a formal
 license is not needed to use the software.


 The ISSI Emulator software is provided by NIST as a service and is expressly
 provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 AND DATA ACCURACY.  NIST does not warrant or make any representations
 regarding the use of the software or the results thereof, including but
 not limited to the correctness, accuracy, reliability or usefulness of
 the software.


 Permission to use this software is contingent upon your acceptance
 of the terms of this agreement
 .

 */

package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * The Interconnect Security parameter indicates whether PSTN interconnect calls
 * are to start in the clear or in a secure mode. 0 indicates that interconnect
 * calls are to start in the clear, and 1 indicates that they are to start
 * secure. If the interconnect security attribute is not provided, calls are to
 * be started in the clear.
 * 
 * @author M. Ranganathan
 * 
 */

public class InterconnectSecurity extends ServiceProfileLine {
	public static final String NAME = "g-icsecstart";

	public static final String SHORTNAME = "g-ics";

	public static final boolean DEFAULTVALUE = false;

	private boolean interconnectCallsSecure = DEFAULTVALUE;

	/**
	 * Default constructor.
	 * 
	 */
	InterconnectSecurity() {
		super(NAME, "0");
		this.interconnectCallsSecure = DEFAULTVALUE;
	}

	/**
	 * Constructor called by the parser.
	 * 
	 * @param flag --
	 *            "0" or "1" are the valid args.
	 * @throws ParseException --
	 *             if the supplied arg is not good.
	 */
	InterconnectSecurity(String flag) throws ParseException {
		super(NAME, flag);
		this.interconnectCallsSecure = super.getBoolFromString(flag);

	}

	/**
	 * @param interconnectCallsSecure
	 *            The interconnectCallsSecure to set.
	 */
	public void setInterconnectCallsSecure(boolean interconnectCallsSecure) {
		this.interconnectCallsSecure = interconnectCallsSecure;
	}

	/**
	 * @return Returns the interconnectCallsSecure.
	 */
	public boolean isInterconnectCallsSecure() {
		return interconnectCallsSecure;
	}

}
