/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.group;

import gov.nist.p25.issi.p25body.serviceprofile.InterruptModeType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

import java.text.ParseException;

/**
 * Interrupt Mode (g-intmode) The Interrupt Mode attribute specifies the audio
 * interrupt mode of the voice group. imode = 0 indicates that audio
 * interruption by SUs within a talk group call is never allowed, 1 indicates
 * that it is allowed based on SU priority, and 2 indicates that it is always
 * allowed. If the Interrupt Mode is not specified, the Interrupt Mode is 0.
 * 
 * @author M. Ranganathan
 * 
 */
public class InterruptMode extends ServiceProfileLine {
	public static final String NAME = "g-intmode";

	public static final String SHORTNAME = "g-in";

	public static final InterruptModeType DEFAULTVALUE = InterruptModeType.AUDIO_INTERRRUPT_NEVER_ALLOWED;

	private InterruptModeType mode = DEFAULTVALUE;

	/**
	 * Constructor called by parser.
	 * 
	 * @param value --
	 *            "0", "1" or "2". Anything else results in parse exception.
	 * @throws ParseException
	 */
	InterruptMode(String value) throws ParseException {
		super(NAME, value);
		try {
			int interruptMode = Integer.parseInt(value);
			if (interruptMode < 0 || interruptMode > 2)
				throw new ParseException(value
						+ ": Value out of range 0,1,2 only allowed", 0);
			this.setMode(InterruptModeType.getModeValues(interruptMode));

		} catch (NumberFormatException ex) {
			throw new ParseException("Bad number format [" + value + "]", 0);
		}
	}

	/**
	 * Default constructor.
	 * 
	 * @return
	 */
	InterruptMode() {
		super(NAME, DEFAULTVALUE.getIntValue());
	}

	/**
	 * @param modeValue
	 *            The modeValue to set.
	 */
	public void setMode(InterruptModeType modeValue) {
		this.mode = modeValue;
	}

	/**
	 * @return Returns the modeValue.
	 */
	public InterruptModeType getMode() {
		return mode;
	}

	public static InterruptMode createFromXmlAttributes(String interruptMode) {
			
		InterruptMode im = new InterruptMode();
		im.setMode(InterruptModeType.getValue(interruptMode));
		return im;
	}

}
