/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.AuthenticationType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * The Authentication Type attribute specifies the type of authentication to be
 * applied to the unit. If no authentication type is specified, the unit does
 * not require authentication.
 * 
 * 
 * @author M. Ranganathan
 * 
 */
public class AuthenticationCapability extends ServiceProfileLine {
	public static final String NAME = "u-authtype";

	public static final String SHORTNAME = "u-at";

	public static final AuthenticationType DEFAULTVALUE = AuthenticationType.NONE;

	private AuthenticationType authenticationType = DEFAULTVALUE;

	AuthenticationCapability(String capability) throws ParseException {
		super(NAME, capability);
		int ctype = super.getIntFromString(capability);
		if (ctype != 0 && ctype != 1)
			throw new ParseException("Bad value [" + capability + "]", 0);
		this.authenticationType = AuthenticationType.getInstance(ctype);

	}

	AuthenticationCapability() {
		super(NAME, DEFAULTVALUE.intValue());
	}

	/**
	 * @param authenticationType
	 *            The authenticationType to set.
	 */
	public void setAuthenticationType(AuthenticationType authenticationType) {
		this.authenticationType = authenticationType;
	}

	/**
	 * @return Returns the authenticationType.
	 */
	public AuthenticationType getAuthenticationType() {
		return authenticationType;
	}

}
