/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.AuthenticationPolicyType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * The Authentication Policy attribute specifies the authentication policy to be
 * applied to the unit.
 * 
 * @author M. Ranganathan
 * 
 */
public class AuthenticationPolicy extends ServiceProfileLine {
	public static final String NAME = "u-authpol";

	public static final String SHORTNAME = "u-apo";

	public static final AuthenticationPolicyType DEFAULTVALUE = AuthenticationPolicyType.NONE;

	private AuthenticationPolicyType authenticationType = DEFAULTVALUE;

	AuthenticationPolicy(String capability) throws ParseException {
		super(NAME, capability);
		int ctype = super.getIntFromString(capability);
		if (ctype < 0 && ctype > 2)
			throw new ParseException("Bad value [" + capability + "]", 0);
		this.authenticationType = AuthenticationPolicyType.getInstance(ctype);

	}

	AuthenticationPolicy() {
		super(NAME, DEFAULTVALUE.getIntValue());
	}

	/**
	 * @param authenticationType
	 *            The authenticationType to set.
	 */
	void setAuthenticationPolicyType(AuthenticationPolicyType authenticationType) {
		this.authenticationType = authenticationType;
	}

	/**
	 * @return Returns the authenticationType.
	 */
	public AuthenticationPolicyType getAuthenticationPolicyType() {
		return authenticationType;
	}

}
