/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.AvailabilityCheckType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * The Availability Check attribute specifies the ability of the SU to support
 * an Availability Check, a direct call or both.
 * 
 * @author M. Ranganathan
 * 
 */
public class AvailabilityCheck extends ServiceProfileLine {

	public static final String NAME = "u-availcheck";

	public static final String SHORTNAME = "u-ac";

	public static final AvailabilityCheckType DEFAULTVALUE = AvailabilityCheckType.AVAIL_CHECK_AND_DIRECT_CALL;

	private AvailabilityCheckType availabilityCheckType;

	AvailabilityCheck(String val) throws ParseException {
		super(NAME, val);
		int intval = super.getIntFromString(val);
		if (intval < 1 || intval > 3)
			throw new ParseException("Illegal argument [" + val + "]", 0);

		this.availabilityCheckType = AvailabilityCheckType.getInstance(intval);

	}

	AvailabilityCheck() {
		super(NAME, DEFAULTVALUE.getIntValue());
		this.availabilityCheckType = DEFAULTVALUE;

	}

	/**
	 * @param availabilityCheckType
	 *            The availabilityCheckType to set.
	 */
	public void setAvailabilityCheckType(
			AvailabilityCheckType availabilityCheckType) {
		this.availabilityCheckType = availabilityCheckType;
	}

	/**
	 * @return Returns the availabilityCheckType.
	 */
	public AvailabilityCheckType getAvailabilityCheckType() {
		return availabilityCheckType;
	}

}
