/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.CallSetupPreferenceType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * @author M. Ranganathan
 * 
 */
public class CallSetupPreference extends ServiceProfileLine {

	public static final String NAME = "u-prefsetup";

	public static final String SHORTNAME = "u-prf";

	public static final CallSetupPreferenceType DEFAULTVALUE = CallSetupPreferenceType.CALLER_PREFERS_AVAILABILITY_CHECK;

	private CallSetupPreferenceType callSetupPreferenceType;

	CallSetupPreference(String val) throws ParseException {
		super(NAME, val);
		int intval = super.getIntFromString(val);
		try {
			this.callSetupPreferenceType = CallSetupPreferenceType
					.getInstance(intval);
		} catch (IllegalArgumentException ex) {
			throw new ParseException("Illegal argument [" + val + "]", 0);
		}

	}

	CallSetupPreference() {
		super(NAME, DEFAULTVALUE.getIntValue());
		this.callSetupPreferenceType = DEFAULTVALUE;
	}

	/**
	 * @param callSetupPreferenceType
	 *            The callSetupPreferenceType to set.
	 */
	public void setCallSetupPreferenceType(
			CallSetupPreferenceType callSetupPreferenceType) {
		this.callSetupPreferenceType = callSetupPreferenceType;
	}

	/**
	 * @return Returns the callSetupPreferenceType.
	 */
	public CallSetupPreferenceType getCallSetupPreferenceType() {
		return callSetupPreferenceType;
	}

}
