/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.DuplexityType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * @author M. Ranganathan
 * 
 */
public class Duplexity extends ServiceProfileLine {

	public static final String NAME = "u-dup";

	public static final String SHORTNAME = "u-d";

	public static final DuplexityType DEFAULTVALUE = DuplexityType.HALF;

	private DuplexityType duplex;

	Duplexity(String duplexity) throws ParseException {
		super(NAME, duplexity);
		int d = super.getIntFromString(duplexity);
		this.duplex = DuplexityType.getInstance(d);
	}

	Duplexity() {
		super(NAME, DEFAULTVALUE.getIntValue());
		this.duplex = DEFAULTVALUE;
	}

	/**
	 * @param duplex
	 *            The duplex to set.
	 */
	public void setDuplex(DuplexityType duplex) {
		this.duplex = duplex;
		super.setValue(duplex.getIntValue());
	}

	/**
	 * @return Returns the duplex.
	 */
	public DuplexityType getDuplex() {
		return duplex;
	}

}
