/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.SecurityLevelType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * The Interconnect Security attribute specifies the acceptable security modes
 * of interconnect calls. If no interconnect security attribute is specified,
 * both secure and clear calls are acceptable.
 * 
 * @author M. Ranganathan
 * 
 */
public class InterconnectSecurity extends ServiceProfileLine {

	public static final String NAME = "u-icsec";

	public static final String SHORTNAME = "u-ics";

	public static final SecurityLevelType DEFAULTVALUE = SecurityLevelType.SECURE_AND_CLEAR_CALLS;

	private SecurityLevelType securityLevel = DEFAULTVALUE;

	InterconnectSecurity(String levelStr) throws ParseException {
		super(NAME, levelStr);
		int lev = super.getIntFromString(levelStr);
		try {
			this.securityLevel = SecurityLevelType.getInstance(lev);
		} catch (IllegalArgumentException ex) {
			throw new ParseException("Value out of range " + levelStr, 0);
		}
	}

	InterconnectSecurity() {
		super(NAME, DEFAULTVALUE.getIntValue());
	}

	/**
	 * @param securityLevel
	 *            The securityLevel to set.
	 */
	public void setSecurityLevel(SecurityLevelType securityLevel) {
		this.securityLevel = securityLevel;
	}

	/**
	 * @return Returns the securityLevel.
	 */
	public SecurityLevelType getSecurityLevel() {
		return securityLevel;
	}

}
