/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.CallPermissionType;
import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * The Unit-to-unit Call Permissions (u-ucall) indicates the types of
 * unit-to-unit calls in which the SU MAY participate. If no unit-to-unit call
 * permission is provided, the unit has no permission to participate in
 * unit-to-unit calls.
 * 
 * @author M. Ranganathan
 * 
 */
public class UnitToUnitCallPermission extends ServiceProfileLine {
	public static final String NAME = "u-ucall";

	public static final String SHORTNAME = "u-uc";

	public static final CallPermissionType DEFAULTVALUE = CallPermissionType.NONE;

	private CallPermissionType permission = DEFAULTVALUE;

	UnitToUnitCallPermission(String perm) throws ParseException {
		super(NAME, perm);
		int p = super.getIntFromString(perm);
		try {
			this.permission = CallPermissionType.getInstance(p);
		} catch (IllegalArgumentException ex) {
			throw new ParseException("Value is out of range [" + perm + "]", 0);
		}
	}

	UnitToUnitCallPermission() {
		super(NAME, DEFAULTVALUE.getIntValue());
	}

	public CallPermissionType getPermission() {
		return permission;
	}

	public void setPermission(CallPermissionType permission) {
		this.permission = permission;
	}

}
