/*
 *This software was developed by employees of the National Institute of
 *Standards and Technology (NIST), an agency of the Federal Government.
 *Pursuant to title 15 Untied States Code Section 105, works of NIST
 *employees are not subject to copyright protection in the United States
 *and are considered to be in the public domain.  As a result, a formal
 *license is not needed to use the software.
 *
 *
 *The ISSI Emulator software is provided by NIST as a service and is expressly
 *provided "AS IS."  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 *OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 *MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 *AND DATA ACCURACY.  NIST does not warrant or make any representations
 *regarding the use of the software or the results thereof, including but
 *not limited to the correctness, accuracy, reliability or usefulness of
 *the software.
 *
 *
 *Permission to use this software is contingent upon your acceptance
 *of the terms of this agreement
 *.
 *
 */
package gov.nist.p25.issi.p25body.serviceprofile.user;

import java.text.ParseException;

import gov.nist.p25.issi.p25body.serviceprofile.ServiceProfileLine;

/**
 * @author M. Ranganathan
 * 
 */
public class UnitToUnitCallPriority extends ServiceProfileLine {

	private int priority;

	public static final String NAME = "u-upri";

	public static final String SHORTNAME = "u-p";

	private static int defaultValue = 3; // Arbitrarily chosen can be set by
											// RFSS

	/**
	 * This can be set.
	 * 
	 * @param defaultValue
	 */
	public static void setDefaultValue(int defaultVal) {
		if (defaultValue < 1 || defaultValue > 10)
			throw new IllegalArgumentException("Bad default value ["
					+ defaultValue + "]");
		defaultValue = defaultVal;
	}

	/**
	 * Get the default value.
	 */
	public static int getDefaultValue() {
		return defaultValue;
	}

	/**
	 * Default constructor. Note that a priority value of 3 is arbitrarily
	 * assigned
	 * 
	 */
	UnitToUnitCallPriority() {
		super(NAME, new Integer(defaultValue).toString());
		this.priority = defaultValue;
	}

	/**
	 * Constructor -- supply the arg as a string
	 * 
	 * @param priorityVal --
	 *            value of priority as a string.
	 * @throws ParseException
	 */
	UnitToUnitCallPriority(String priorityVal) throws ParseException {
		super(NAME, priorityVal);
		try {
			this.priority = Integer.parseInt(priorityVal);
		} catch (NumberFormatException ex) {
			throw new ParseException("Illegal Priority Value [" + priorityVal
					+ "]", 0);

		}
		if (this.priority > 10 || this.priority < 1)
			throw new ParseException("Illegal Priority Value [" + priorityVal
					+ "]", 0);

	}

	/**
	 * Constructor -- supply arg as an int.
	 * 
	 * @param priorityVal --
	 *            priority value supplied as a valid int in the range 1 -- 10
	 * @throws IllegalArgumentException
	 */
	public UnitToUnitCallPriority(int priorityVal)
			throws IllegalArgumentException {
		super(NAME, new Integer(priorityVal).toString());
		if (priorityVal > 10 || priorityVal < 1)
			throw new IllegalArgumentException("Illegal priority value ["
					+ priorityVal + "]");
		this.priority = priorityVal;
	}

	/**
	 * @param priority
	 *            The priority to set.
	 */
	public void setPriority(int priority) {
		this.priority = priority;
		if (this.priority > 10 || this.priority < 1)
			throw new IllegalArgumentException("Illegal Priority Value ["
					+ priority + "]");
		super.setValue(priority);

	}

	/**
	 * @return Returns the priority.
	 */
	public int getPriority() {
		return priority;
	}

}
