package gov.nist.p25.issi.packetmonitor;

import gov.nist.p25.issi.issiconfig.RfssConfig;
import gov.nist.p25.issi.packetmonitor.PacketMonitor.EndPoint;

import java.util.LinkedList;

/**
 * 
 * Represents a Media session. Media sessions
 * are inferred from the SDP information of the
 * SIP request and matching response pair. A media session
 * is an RTP session between two EndPoints. The owningRfss
 * field represents the RFSS that created the media session.
 *
 */
class MediaSession {
	RfssConfig owningRfss;

	EndPoint source;

	EndPoint destination;

	LinkedList<CapturedPttMessage> capturedPttMessages;

	RfssConfig remoteRfss;

	
	private MediaSession() {
		this.capturedPttMessages = new LinkedList<CapturedPttMessage>();
	}

	public MediaSession( EndPoint destination) {

		this();
		if (destination == null)
			throw new NullPointerException("Null Arg!");
		this.destination = destination;

	}

	public void setSource(EndPoint source) {
		if (source == null)
			throw new NullPointerException("Null arg!");
		this.source = source;
	}

	public EndPoint getSource() {
		return source;
	}

	public EndPoint getDestination() {
		return destination;
	}

	public CapturedPttMessage addPttMessage(byte[] message, long time) {
		try {
			CapturedPttMessage newMessage = new CapturedPttMessage( message,
					time, this);
			if (this.capturedPttMessages.size() == 0)
				this.capturedPttMessages.add(newMessage);
			else {
				CapturedPttMessage previousMessage = this.capturedPttMessages.peekLast();
				if (previousMessage.match(newMessage)) {
					PacketMonitor.logger
							.debug("Previous Ptt message matches current message -- dropping message");
				} else {
					this.capturedPttMessages.add(newMessage);
				}
			}
			return newMessage;
		} catch (Exception ex) {
			PacketMonitor.logger.error("Could not log packet", ex);
			return null;
		}

	}

	public MediaSession reverse() {
		MediaSession retval = new MediaSession();
		retval.source = this.destination;
		retval.destination = this.source;
		retval.owningRfss = this.remoteRfss;
		retval.remoteRfss = this.owningRfss;

		return retval;
	}

	public String toString() {

		return new StringBuffer()
				.append("<ptt-session\n")

				.append("rfssId = \"" + this.remoteRfss.getRfssId())
				.append("\"\n")
				.append("myIpAddress = \"" + this.destination.host + "\"\n")
				.append(
						"myRtpRecvPort = \"" + this.destination.port
								+ "\"\n").append(
						"remoteIpAddress = \"" + this.source.host + "\"\n")
				.append(
						"remoteRtpRecvPort = \"" + this.source.port
								+ "\"\n").append("/>\n").toString();
	}

}