package gov.nist.p25.issi.packetmonitor;

import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;
import gov.nist.p25.issi.rfss.ISSITesterConstants;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

/**
 * Servlet for the control of the packet monitor.
 * 
 * @author M. Ranganathan
 * 
 */
public class MonitorController extends HttpServlet {

	private static PacketMonitor packetMonitor;
	
	private static Logger logger = Logger.getLogger(MonitorController.class);

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		logger.debug("packetMonitor : command = " + request.getParameter(ISSITesterConstants.COMMAND));
		try {
			String command = request.getParameter(ISSITesterConstants.COMMAND);
			if (command.equals(ISSITesterConstants.START_PACKET_MONITOR)) {
				String topologyName = request
						.getParameter(ISSITesterConstants.SYSTEM_TOPOLOGY);

				TopologyConfig topologyConfig = new SystemTopologyParser(PacketMonitor.getConfigurations())
						.parse(topologyName);
				logger.debug("starting packet monitor " +  topologyConfig);
				
				packetMonitor = PacketMonitor
						.startPacketMonitor(topologyConfig);
			} else if ( command.equals(ISSITesterConstants.GET_ERROR_FLAG)) {
				if ( packetMonitor == null ) {
					response.getWriter().print(new Boolean(true).toString());
				} else {
					response.getWriter().print(new Boolean(packetMonitor.getErrorStatus()).toString());
				}
				
			} else if ( command.equals(ISSITesterConstants.GET_ERROR_LOG)) {
				if ( packetMonitor == null) {
					response.getWriter().print("Monitor not started");
				} else {
					response.getWriter().print(packetMonitor.getErrorMessage());
				}
			}
		} catch (Exception ex) {
			logger.error (
					"Unexpected error processing request " , ex);
			response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
		}

	}

}
