package gov.nist.p25.issi.packetmonitor.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.log4j.Logger;

/**
 * Panel where the control buttons go. This class also talks to the "back end".
 * 
 * @author mranga@nist.gov
 * @author niepin@nist.gov
 * 
 */

public class ControlPanel extends JPanel implements ActionListener {

	private static Logger logger = Logger.getLogger(ControlPanel.class);

	private PacketMonitorGUI packetMonitor;

	private PacketMonitorController packetMonitorController;
	
	private MeasurementTable measurementTable = null;

	JButton startCaptureButton;

	JButton fetchTraceButton;
	
	JButton evaluateButton;

	public ControlPanel(PacketMonitorGUI packetMonitor, PacketMonitorController packetMonitorController) {

		startCaptureButton = new JButton("Start Packet Capture");
		fetchTraceButton = new JButton("Fetch Trace");
		evaluateButton = new JButton("Evaluate Trace");
		this.packetMonitor = packetMonitor;
		startCaptureButton.addActionListener(this);
		fetchTraceButton.addActionListener(this);
		evaluateButton.addActionListener(this);
		super.add(startCaptureButton);
		super.add(fetchTraceButton);
		super.add(evaluateButton);
		this.packetMonitorController = packetMonitorController;
		//this.measurementTable = new MeasurementTable(this.packetMonitor);
	}

	public void actionPerformed(ActionEvent ae) {
		try {
			if (ae.getSource() == this.startCaptureButton) {
				this.packetMonitorController.startCapture();
			} else if (ae.getSource() == this.evaluateButton) {
				// check the traces and evaluate if they are available
				if(this.measurementTable == null)
					this.measurementTable = new MeasurementTable(packetMonitor);
				measurementTable.showTable();
			} else if (ae.getSource() == this.fetchTraceButton) {
				boolean errorFlag = this.packetMonitorController
						.fetchErrorFlag();
				if (errorFlag) {
					String errorCause = this.packetMonitorController
							.fetchErrorString();
					JOptionPane.showConfirmDialog(null, errorCause,
							"Capture Error", JOptionPane.ERROR_MESSAGE);

				} else {
					String sipTraces = this.packetMonitorController
							.fetchSipTraces();
					String pttTraces = this.packetMonitorController
							.fetchPttTraces();
					String result = this.packetMonitorController
							.fetchResult();
					this.packetMonitor.getMessageInformationPanel()
							.renderSipPttTraces(sipTraces, pttTraces, null);
					if(this.measurementTable == null)
						this.measurementTable = new MeasurementTable(packetMonitor);
					measurementTable.setData(result);
				}
			}
		} catch (Exception ex) {
			logger.error("Error communicating with the monitor", ex);
			JOptionPane.showMessageDialog(packetMonitor,
					"Error communicating with the monitor", "Unexpected Error",
					JOptionPane.ERROR_MESSAGE);
		}

	}

}
