package gov.nist.p25.issi.packetmonitor.gui;

import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;

import gov.nist.p25.issi.traceviewer.MessageData;
import gov.nist.p25.issi.traceviewer.PttSessionInfo;
import gov.nist.p25.issi.traceviewer.PttTraceLoader;
import gov.nist.p25.issi.traceviewer.RfssData;
import gov.nist.p25.issi.traceviewer.SipTraceLoader;
import gov.nist.p25.issi.traceviewer.TraceFileFilter;
import gov.nist.p25.issi.traceviewer.TracePanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.TreeSet;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import org.apache.log4j.Logger;

public class MessageInformationPanel extends JPanel {
	private static Logger logger = Logger.getLogger( MessageInformationPanel.class);
	
	private PacketMonitorGUI gui;
	
	private JTextPane pttTraceLogPane;
	private JTextPane sipTraceLogPane;
	
	private String sipRtfTrace;

	private String pttRtfTrace;

	private TracePanel sipTracePanel;

	private TracePanel pttTracePanel;

	private String systemTopologyFileName;
	
	public static SimpleAttributeSet ERROR_MESSAGE_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet STACK_TRACE_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet INFO_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet FATAL_MESSAGE_TEXT = new SimpleAttributeSet();

	static {

		StyleConstants.setForeground(ERROR_MESSAGE_TEXT, Color.red);
		StyleConstants.setForeground(STACK_TRACE_TEXT, Color.blue);
		StyleConstants.setForeground(INFO_TEXT, Color.black);
		StyleConstants.setForeground(FATAL_MESSAGE_TEXT, Color.red);
		StyleConstants.setUnderline(FATAL_MESSAGE_TEXT, true);
		StyleConstants.setFontSize(FATAL_MESSAGE_TEXT, 14);

	}
	
	public MessageInformationPanel( PacketMonitorGUI gui, String systemTopology) {
		this.gui = gui;
		this.setLayout(new BorderLayout());
		this.setBorder(new TitledBorder("Message Data"));

		this.systemTopologyFileName = systemTopology;
		this.sipTraceLogPane = new JTextPane();
		sipTraceLogPane.setEditable(false);
		JScrollPane sipLogScrollPane = new JScrollPane(sipTraceLogPane);
		sipTraceLogPane.setAutoscrolls(true);

		sipTraceLogPane.setToolTipText("Right Click to Save ");

		sipTraceLogPane.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent me) {
			
				if (me.getButton() == 3) {

					JFileChooser fileChooser = new JFileChooser();
					fileChooser.setDialogTitle("Save As");
					TraceFileFilter traceFilter = new TraceFileFilter();
					traceFilter.addExtension("txt");
					traceFilter.addExtension("rtf");
					traceFilter.setDescription("Text or RTF Files");
					fileChooser.setFileFilter(traceFilter);

					int returnVal = fileChooser
							.showSaveDialog(MessageInformationPanel.this.gui);

					File file = null;
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						logger.debug("Saving: " + file.getName());
					} else {
						logger.debug("Save command cancelled by user.");
						return;
					}
					logger.debug("path for save: " + file.getAbsolutePath());
					String message = (traceFilter.getExtension(file)
							.endsWith("txt")) ? sipTraceLogPane.getText()
							: sipRtfTrace;

					if (message == null || message.trim().equals("")) {
						JOptionPane.showMessageDialog(null, "No data to save",
								"Save Trace Info",
								JOptionPane.INFORMATION_MESSAGE);
						return;
					}

					try {

						OutputStream out = new FileOutputStream(new File(file
								.getAbsolutePath()));
						out.write(message.getBytes());
						out.close();
					} catch (Exception e) {
						logger
								.error(
										"An exception occured while trying to save file",
										e);
						JOptionPane.showMessageDialog(null,
								"An Error Occured while saving the file",
								"Save File Status", JOptionPane.ERROR_MESSAGE);
					}

				}
			}

			public void mouseEntered(MouseEvent e) {

			}

			public void mouseExited(MouseEvent e) {

			}

			public void mousePressed(MouseEvent e) {

			}

			public void mouseReleased(MouseEvent e) {

			}
		});

		this.pttTraceLogPane = new JTextPane();
		pttTraceLogPane.setEditable(false);
		JScrollPane pttLogScrollPane = new JScrollPane(pttTraceLogPane);
		pttTraceLogPane.setAutoscrolls(true);
		pttTraceLogPane.setToolTipText("Right Click to Save ");

		pttTraceLogPane.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent me) {
				// TODO Auto-generated method stub

				if (me.getButton() == 3) {

					JFileChooser fileChooser = new JFileChooser();
					fileChooser.setDialogTitle("Save As");
					TraceFileFilter traceFilter = new TraceFileFilter();
					traceFilter.addExtension("txt");
					traceFilter.addExtension("rtf");
					traceFilter.setDescription("Text or RTF Files");
					fileChooser.setFileFilter(traceFilter);
					File file = null;
					int returnVal = fileChooser
							.showSaveDialog(MessageInformationPanel.this.gui);

					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						logger.debug("Saving: " + file.getName());
					} else {
						logger.debug("Save command cancelled by user.");
						return;
					}
					logger.debug("path for save: " + file.getAbsolutePath());
					String message = traceFilter.getExtension(file).endsWith(
							"txt") ? pttTraceLogPane.getText() : pttRtfTrace;

					if (message == null || message.trim().equals("")) {
						JOptionPane.showMessageDialog(null, "No data to save",
								"Save Trace Info",
								JOptionPane.INFORMATION_MESSAGE);
						return;
					}

					try {

						OutputStream out = new FileOutputStream(new File(file
								.getAbsolutePath()
								+ ".txt"));
						out.write(message.getBytes());
						out.close();
					} catch (Exception e) {
						logger
								.error(
										"An exception occured while trying to save file",
										e);
						JOptionPane.showMessageDialog(null,
								"An Error Occured while saving the file",
								"Save File Status", JOptionPane.ERROR_MESSAGE);
					}

				}
			}

			public void mouseEntered(MouseEvent e) {

			}

			public void mouseExited(MouseEvent e) {
				

			}

			public void mousePressed(MouseEvent e) {
				

			}

			public void mouseReleased(MouseEvent e) {
			

			}
		});
		
		JTabbedPane jtabbedPane = new JTabbedPane();
		
		jtabbedPane.add("SIP Traces",sipLogScrollPane);
		jtabbedPane.add("PTT Traces",pttLogScrollPane);
		this.add(jtabbedPane);
		

	}
	
	
	public void renderSipPttTraces(String sipTraces, String pttTraces,
			Hashtable<String, HashSet<PttSessionInfo>> rfssSessionData) {

		boolean remoteRetrievalSucceeded = true;

	
		// Now get SIP traces
		Collection<RfssData> rfssList = null;

		try {

			Hashtable<String, Color> colorMap = new Hashtable<String, Color>();

			byte[] stringAsBytes = sipTraces.getBytes();
			ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
					stringAsBytes);
			TopologyConfig systemTopology = new SystemTopologyParser(PacketMonitorGUI.getIssiTesterConfiguration())
					.parse(systemTopologyFileName);
		
			SipTraceLoader traceLoader = new SipTraceLoader(
					byteArrayInputStream, rfssSessionData, colorMap,
					systemTopology);
			
			rfssList = traceLoader.getSortedRfssList();

			ArrayList<MessageData> records = traceLoader.getRecords();

			int currentLength = sipTraceLogPane.getDocument().getLength();
			// Clear up the existing stuff.
			sipTraceLogPane.getDocument().remove(0, currentLength);

			StringBuffer sbuf = new StringBuffer();
			for (MessageData messageData : records) {
				sbuf.append("F" + messageData.getId() + ":\n");
				sbuf.append(messageData.getData().trim() + "\n\n");
			}

			// Now generate the RTF.
			// TODO -- make this into a resource

			/*
			 * Generate the RTF traces for SIP.
			 */

			File rtfHeaderFile = new File("doc/rtf-header.txt");
			int length = (int) rtfHeaderFile.length();
			char[] headerbytes = new char[length];
			FileReader fileReader = new FileReader(rtfHeaderFile);
			fileReader.read(headerbytes);
			fileReader.close();
			StringBuffer rtfBuffer = new StringBuffer().append(new String(
					headerbytes));
			for (MessageData messageData : records) {
				String rtf = messageData.getRtfText();
				rtfBuffer.append(rtf);
			}
			rtfBuffer.append("}");
			sipTraceLogPane.getDocument().insertString(
					sipTraceLogPane.getDocument().getLength(), sbuf.toString(),
					INFO_TEXT);
			this.sipRtfTrace = rtfBuffer.toString();

			// Create a new tabbed panel
			this.sipTracePanel = new TracePanel(gui,gui.tabbedPane, "", rfssList,
					records, colorMap);

			// close existing tabs.
			gui.tabbedPane.removeAll();
			gui.tabbedPane.add("SIP traces " , sipTracePanel);
			gui.tabbedPane.setSelectedComponent(sipTracePanel);
			
		} catch (Exception ex) {

			ex.printStackTrace();
			remoteRetrievalSucceeded = false;
				
			try {
				pttTraceLogPane.getDocument().insertString(
						pttTraceLogPane.getDocument().getLength(),
						"Error Fetching SIP Trace From Packet Monitor \n"
								+ ex.getMessage() + "\n", ERROR_MESSAGE_TEXT);

			} catch (Exception e) {

				e.printStackTrace();

			}

		}

		// If remote retrieval of SIP traces failed, return here
		if (!remoteRetrievalSucceeded)
			return;

		try {

			if (pttTraces == null) {

				return;

			}

			Hashtable<String, Color> colorMap = new Hashtable<String, Color>();

			if ( logger.isDebugEnabled())
				logger.debug("ptt traces = " + pttTraces);
			byte[] stringAsBytes = pttTraces.getBytes();
			ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
					stringAsBytes);

			PttTraceLoader traceLoader = new PttTraceLoader(
					byteArrayInputStream, 
					//rfssList, 
					colorMap);
			StringBuffer sbuf = new StringBuffer();

			for (MessageData mdata : traceLoader.getRecords()) {
				sbuf.append("F" + mdata.getId() + "\n");
				sbuf.append(mdata.getData());
				sbuf.append("\n\n");

			}

			pttTraceLogPane.getDocument().insertString(
					pttTraceLogPane.getDocument().getLength(), sbuf.toString(),
					INFO_TEXT);

			// Create a new tabbed panel
			this.pttTracePanel = new TracePanel(gui,gui.tabbedPane, "", rfssList,
					traceLoader.getRecords(), colorMap);

			gui.tabbedPane.add("PTT ", pttTracePanel);
			gui.tabbedPane.setSelectedComponent(pttTracePanel);
			
			/*
			 * The PTT rtf buffer.
			 */
			File rtfHeaderFile = new File("doc/rtf-header.txt");
			int length = (int) rtfHeaderFile.length();
			char[] headerbytes = new char[length];
			FileReader fileReader = new FileReader(rtfHeaderFile);
			fileReader.read(headerbytes);
			fileReader.close();
			StringBuffer rtfBuffer = new StringBuffer().append(new String(
					headerbytes));

			for (MessageData messageData : traceLoader.getRecords()) {
				String rtf = messageData.getRtfText();
				rtfBuffer.append(rtf);
			}
			rtfBuffer.append("}");
			this.pttRtfTrace = rtfBuffer.toString();

		} catch (Exception ex) {

			try {
				pttTraceLogPane.getDocument().insertString(
						pttTraceLogPane.getDocument().getLength(),
						"Error Fetching PTT Trace From Packet Monitor \n"
								+ ex.getMessage() + "\n", ERROR_MESSAGE_TEXT);

			} catch (Exception e) {

				e.printStackTrace();

			}

		}

	}
}
