package gov.nist.p25.issi.packetmonitor.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.Properties;
import java.util.TimerTask;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;

import gov.nist.p25.issi.ISSITimer;
import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.rfss.ISSITesterConstants;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfiguration;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfigurationParser;
import gov.nist.p25.issi.startup.DietsConfigProperties;
import gov.nist.p25.issi.testlauncher.AboutDialog;
import gov.nist.p25.issi.testlauncher.DietsGUI;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * A GUI front end for the packet monitor.
 * 
 * @author mranga
 * 
 */
public class PacketMonitorGUI extends JFrame implements ActionListener,
		MouseListener {

	private static Logger logger = Logger.getLogger(PacketMonitorGUI.class
			.getPackage().getName());

	static {
		try {

			PropertyConfigurator.configure("log4j.properties");
			logger.addAppender(new FileAppender(new SimpleLayout(),
					"logs/itatguidebug.txt"));

		} catch (Exception ex) {

		}
	}

	// private static final long serialVersionUID = 0;
	private static final String VERSION =  ISSITesterConstants.VERSION;

	private static final String BUILD = "102";

	private static String systemTopologyFileName;

	private static String testerConfigurationFile;

	private static int PREFERRED_WIDTH = 1050;

	private static int PREFERRED_HEIGHT = 850;

	private static ISSITesterConfiguration issiTesterConfiguration;

	private JMenuItem aboutMenuItem;

	private ControlPanel controlPanel;

	public JTabbedPane tabbedPane = null;

	private JLabel statusLabel;

	private MessageInformationPanel messageInformationPanel;

	public PacketMonitorGUI() throws Exception {
		super();
		try {

			this.initGUI();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void initGUI() throws Exception {
		logger.info("GUI system initializing!");
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		this.messageInformationPanel = new MessageInformationPanel(this,
				systemTopologyFileName);
		this.getMessageInformationPanel().setMinimumSize(
				new Dimension(PREFERRED_WIDTH, 200));
		this.getMessageInformationPanel().setPreferredSize(
				new Dimension(PREFERRED_WIDTH, 225));
		this.getMessageInformationPanel().setMaximumSize(
				new Dimension(PREFERRED_WIDTH, 325));

		getContentPane().setLayout(new BorderLayout());
		this.controlPanel = new ControlPanel(this, new RemotePacketMonitorController(
				this, testerConfigurationFile, systemTopologyFileName));

		JPanel tabbedPanel = new JPanel();
		tabbedPanel.setLayout(new BorderLayout());
		tabbedPanel.setBorder(new TitledBorder("Test Traces"));

		tabbedPane = new JTabbedPane();
		tabbedPanel.add(tabbedPane, BorderLayout.CENTER);

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, this
				.getMessageInformationPanel(), tabbedPanel);

		this.getContentPane().add(splitPane, BorderLayout.CENTER);
		this.getContentPane().add(controlPanel, BorderLayout.SOUTH);

		
		setTitle("DIETS PACKET MONITOR AND TRACE ANALYSIS TOOL " + " v" + VERSION);
		setIconImage(new ImageIcon("gfx/nist.gif").getImage());
		JPanel statusPanel = new JPanel();
		statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
		setStatusLabel(new JLabel("Ready"));
		getStatusLabel().setMinimumSize(new Dimension(300, 16));
		getStatusLabel().setPreferredSize(new Dimension(700, 16));
		getStatusLabel().setMaximumSize(new Dimension(900, 16));

		setSize(PREFERRED_WIDTH, PREFERRED_HEIGHT);

		this.setVisible(true);
		this.setAlwaysOnTop(true);
		
		ISSITimer.getTimer().schedule(new TimerTask() {

			@Override
			public void run() {
				PacketMonitorGUI.this.setAlwaysOnTop(false);	
			}},1000);
		
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				PacketMonitorGUI.this.dispose();
			}
		});

	}

	private void setStatusLabel(JLabel label) {
		this.statusLabel = label;

	}

	public JLabel getStatusLabel() {
		return this.statusLabel;
	}

	public MessageInformationPanel getMessageInformationPanel() {
		return messageInformationPanel;
	}

	public void actionPerformed(ActionEvent ae) {

		if (ae.getSource() == this.aboutMenuItem) {

			AboutDialog about_dialog = new AboutDialog(this, VERSION, BUILD);
			// about_dialog.setSize(350, 175);
			about_dialog.setLocationRelativeTo(this);
			about_dialog.setVisible(true);

		}
	}
	
	/**
	 * Use a PrintWriter wrapped around a BufferedWriter, which in turn
	 * is wrapped around a FileWriter, to write the string data to the
	 * given file.
	**/
	  public boolean writeFile (File file, String data) {
	    try {
	       PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
	       out.print (data);
	       out.flush ();
	       out.close ();
	    }
	    catch (IOException e) {
	       return false;
	    }
	    return true;
	  }

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public static ISSITesterConfiguration getIssiTesterConfiguration() {

		return issiTesterConfiguration;
	}

	public static void initProperties(Properties props) {
		systemTopologyFileName = props.getProperty(DietsConfigProperties.SYSTEM_TOPOLOGY_PROPERTY,
				systemTopologyFileName);
		testerConfigurationFile = props.getProperty(DietsConfigProperties.DAEMON_CONFIG_PROPERTY,
				testerConfigurationFile);

		PacketMonitorGUI.PREFERRED_WIDTH = props.getProperty(DietsConfigProperties.DIETSGUI_WIDTH) != null ? Integer
				.parseInt(props.getProperty(DietsConfigProperties.DIETSGUI_WIDTH))
				: PacketMonitorGUI.PREFERRED_WIDTH;
		PacketMonitorGUI.PREFERRED_HEIGHT = props.getProperty(DietsConfigProperties.DIETSGUI_HEIGHT) != null ? Integer
				.parseInt(props.getProperty(DietsConfigProperties.DIETSGUI_HEIGHT))
				: PacketMonitorGUI.PREFERRED_HEIGHT;

		if (testerConfigurationFile == null || systemTopologyFileName == null) {

			JOptionPane.showMessageDialog(null,
					"Bad or missing startup properties", "Startup Error",
					JOptionPane.ERROR_MESSAGE);
			return;

		}

		try {
			issiTesterConfiguration = new ISSITesterConfigurationParser(
					testerConfigurationFile).parse();

			new SystemTopologyParser(issiTesterConfiguration)
					.parse(systemTopologyFileName);
		} catch (Exception ex) {
			logger.error("Bad or missing config file", ex);
			JOptionPane.showMessageDialog(null,
					"Bad or missing configuration file", "Startup Error",
					JOptionPane.ERROR_MESSAGE);
			return;

		}

	}
	
	public static void main(String[] args) throws Exception {

		Properties props = new Properties();
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-startup")) {
				String fileName = args[++i];
				InputStream inStream = new FileInputStream(new File(fileName));

				props.load(inStream);
			} else {
				JOptionPane.showMessageDialog(null,
						"Bad startup flag " + args[i], "Startup Error",
						JOptionPane.ERROR_MESSAGE);
			}
		}
		
		initProperties(props);
		
		// Schedule a job for the event-dispatching thread
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					PacketMonitorGUI gui =new PacketMonitorGUI();
					gui.addWindowListener(new WindowAdapter() {

						public void windowClosing(WindowEvent e) {

							//System.exit(0);

						}
					});
				} catch (Exception ex) {
					logger.error("Exception starting the GUI", ex);
				}
			}
		});

	}

}
