/*******************************************************************************
 * Product of NIST/ITL Advanced Networking Technologies Division (ANTD)         *
 *******************************************************************************/
package gov.nist.p25.issi.rfss;

import gov.nist.p25.issi.issiconfig.ISSIConstants;

import java.util.*;
import javax.sip.header.*;
import javax.sip.address.*;

import org.apache.log4j.Logger;

/**
 * Class for removing the bindings after expires
 */
public class ExpiresTask extends TimerTask {

	private String key;

	private ContactHeader contactHeader;

	private RegistrationsTable registrationsTable;

	static Logger logger = Logger.getLogger(ExpiresTask.class);

	/**
	 * Constructor
	 */
	public ExpiresTask(String key, ContactHeader contactHeader,
			RegistrationsTable registrationsTable) {
		this.registrationsTable = registrationsTable;
		this.key = key;
		this.contactHeader = contactHeader;
	}

	public void run() {
		Address address = contactHeader.getAddress();
		javax.sip.address.URI cleanedUri = UnitToUnitMobilityManager
				.getCleanUri(address.getURI());
		String contactURI = cleanedUri.toString();
		registrationsTable.removeContact(key, contactHeader);
		registrationsTable.expiresTaskTable.remove(contactURI);
		if (logger.isDebugEnabled())
			logger.debug("ExpiresTask, run(), we  removed the contact: "
					+ contactURI + " for the user: " + key);

		synchronized (registrationsTable.expiresTaskTable) {
			registrationsTable.expiresTaskTable.remove(key);
		}

	}

}
