package gov.nist.p25.issi.rfss;

import javax.sip.address.SipURI;
import javax.sip.header.ContactHeader;

import gov.nist.p25.issi.issiconfig.SuConfig;

/**
 * This is the service access point for mobility management. This is essentially
 * a provider for the MobiliytManager.
 * 
 * @author M. Ranganathan
 * 
 */

public class MobilityManagementSAP {
	private RFSS rfss;

	private UnitToUnitMobilityManager mobilityManager;

	private SuConfig suConfig;

	//private SU su;

	MobilityManagementSAP(RFSS rfss, SuConfig suConfig) {
		this.rfss = rfss;
		this.mobilityManager = rfss.getUnitToUnitMobilityManager();
		this.suConfig = suConfig;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.nist.p25.issi.rfss.MobilityManagementSAP#mmRegistrationRequest()
	 */
	public void mmRegistrationRequest() throws Exception {
		this.mobilityManager.sendRegisterRegister(suConfig);
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.nist.p25.issi.rfss.MobilityManagementSAP#mmRoamedRequest(gov.nist.p25.issi.rfss.CallSegment)
	 */

	public void mmRoamedRequest(UnitToUnitCall unitToUnitCall,
			ContactHeader newContact, ContactHeader oldContact)
			throws Exception {

		this.mobilityManager.sendMMRoamed(this.suConfig, newContact, oldContact,
				(unitToUnitCall != null ? unitToUnitCall.getCallID(): null));

	}

	/*
	 * Get the current registration.
	 */
	public ContactHeader getCurrentRegistration() {
		Registration registration = this.mobilityManager
				.getRegistration(this.suConfig.getRadicalName());
		return registration.getContact();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.nist.p25.issi.rfss.MobilityManagementSAP#mmSuDeparted()
	 */
	public void mmSuDeparted() {
		this.rfss.removeServedSubscriberUnit(suConfig);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gov.nist.p25.issi.rfss.MobilityManagementSAP#mmSuArrived()
	 */
	public void mmSuArrived() {
		this.rfss.addServedSubscriberUnit(suConfig);

	}

}
