package gov.nist.p25.issi.rfss.tester;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;

import gov.nist.p25.issi.issiconfig.DaemonWebServerAddress;
import gov.nist.p25.issi.issiconfig.PacketMonitorWebServerAddress;
import gov.nist.p25.issi.rfss.ISSITesterConstants;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

public class ISSITesterConfigurationParser extends DefaultHandler {

	private ISSITesterConfiguration retval;

	private String fileName;

	private XMLReader saxParser;

	private static Logger logger = Logger
			.getLogger(ISSITesterConfigurationParser.class);

	private String ipAddress;

	private int port;

	private DaemonWebServerAddress daemonWebServerAddress;
	

	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/issi-tester-config.dtd")) {
				FileReader fileReader = new FileReader(new File(
						"dtd/issi-tester-config.dtd"));
				InputSource inputSource = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}

	public ISSITesterConfiguration parse() throws Exception {
		this.saxParser.parse(fileName);
		return retval;
	}

	public ISSITesterConfigurationParser(String fileName) throws SAXException,
			ParserConfigurationException {
		this.fileName = fileName;
		logger.info("Parsing " + fileName);

		this.retval = new ISSITesterConfiguration();

		SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
		this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
		this.saxParser.setEntityResolver(new MyResolver());

		saxParser.setContentHandler(this);

		saxParser.setFeature("http://xml.org/sax/features/validation", false);

	}

	public void startElement(String uri, String localName, String name,
			Attributes attrs) throws SAXException {
		if (name.equals("diets-daemon")) {
			this.ipAddress = attrs.getValue("ipAddress");
			if ( ipAddress == null) ipAddress = "127.0.0.1";
			String nodeName = attrs.getValue("name");
			if (attrs.getValue("httpPort") != null) {
				this.port = Integer.parseInt(attrs.getValue("httpPort"));
			} else {
				this.port =  ISSITesterConstants.DEFAULT_DIETS_HTTP_PORT;
				
			}
			boolean isTester = Boolean.parseBoolean(attrs.getValue("isConformanceTester"));
			boolean isPacketMonitor = Boolean.parseBoolean(attrs.getValue("isPacketMonitor"));
			
			this.daemonWebServerAddress = new DaemonWebServerAddress(nodeName,ipAddress,
					port, isTester, isPacketMonitor);
			// note that there may not be a name associated with this address.
			this.retval.addDaemonWebServerAddress(daemonWebServerAddress);
			
			
		} else if (name.equalsIgnoreCase("refid")) {
			String id = attrs.getValue("id");
			this.retval.addReference(id,daemonWebServerAddress);
		} 
		
	}

}
