package gov.nist.p25.issi.rfss.tester;

import java.util.HashSet;

import javax.sip.message.Message;
import javax.sip.message.Request;
import javax.sip.message.Response;

import org.apache.log4j.Logger;

public class SipMessageTrigger extends Trigger {
	private boolean isRequest;
	private int responseCode;
	
	HashSet<String > firedTable = new HashSet<String>();
	
	private static Logger logger = Logger.getLogger(SipMessageTrigger.class);

	public SipMessageTrigger(String messageType) {
		super(messageType);
		if ( messageType.equals(Request.INVITE) || messageType.equals(Request.ACK) || 
				messageType.equals(Request.BYE) || messageType.equals(Request.CANCEL)) {
			isRequest = true;
			
		} else {
			responseCode = Integer.parseInt(messageType);
		}
		
	}
	@Override
	public boolean isReady(String rfssDomainName, Object matchObject) {
		if ( isRequest && matchObject instanceof Request) {
			Request request = (Request) matchObject;
			String type  = request.getMethod();
			boolean fired = firedTable.contains(rfssDomainName.toLowerCase());
			logger.debug("SipMessageTrigger: type : "  +  type + " fired " + fired + " oneShot " + oneShot);
			
			if (fired && oneShot) return false;
			fired = type.equals(super.matchValue);
			
			logger.debug("Returning " + fired);
			if ( fired) firedTable.add(rfssDomainName.toLowerCase());
			return fired;
			
		} else if ( (!isRequest) && matchObject instanceof Response)  {
			Response response = (Response) matchObject;
			int statusCode = response.getStatusCode();
			boolean fired = firedTable.contains(rfssDomainName.toLowerCase());
			
			if ( fired && oneShot) return false;
			fired =  this.responseCode == statusCode;
			if ( fired ) firedTable.add(rfssDomainName.toLowerCase());
			return fired;
		} else return false;
	}

}
