package gov.nist.p25.issi.rfss.tester;

import gov.nist.p25.issi.rfss.ProtocolObjects;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.sip.message.Request;
import javax.sip.message.Response;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A parser for parsing the stored trace messages. These messages are compared
 * against the messages that are captured during test execution time. Each of
 * the messages in the stored trace file should match the trace messages
 * captured at test execution time.
 * 
 * @author M. Ranganathan
 * 
 */
public class TestMessagesParser extends DefaultHandler {
	private StringBuffer cdatabuffer;
	private XMLReader saxParser;

	private TestMessages testMessages;
	private String testMessagesFile;

	private static Logger logger = Logger.getLogger(TestMessagesParser.class);
	//	 /////////////////////////////////////////////////////////////////////////////////////////
	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/expected-messages.dtd")) {
				FileReader fileReader = new FileReader( new File("dtd/expected-messages.dtd"));
				InputSource inputSource  = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}
	@Override
	public void characters(char[] ch, int start, int length) { 
		this.cdatabuffer.append(ch, start, length);
	}

	@Override
	public void endElement(String namespaceURI, String local, String name) {
		try {
			if ("siprequest".equals(name)) {
				String requestString = this.cdatabuffer.toString();
				Request request = ProtocolObjects.messageFactory
						.createRequest(requestString);
				testMessages.addRequest(request);
				this.cdatabuffer = new StringBuffer();
			} else if ("sipresponse".equals(name)) {
				String responseString = this.cdatabuffer.toString();
				Response response = ProtocolObjects.messageFactory
						.createResponse(responseString);
				testMessages.addResponse(response);
				this.cdatabuffer = new StringBuffer();

			}
		} catch (Exception ex) {
			logger.fatal("Unexpected exception ", ex);
			//System.exit(0);
		}

	}
	@Override
	public void startElement(String namespaceURI, String local, String name,
			Attributes attrs) throws SAXException {

	}
	public TestMessagesParser(String testMessagesFile) {
		this.cdatabuffer = new StringBuffer();
		try {
			this.testMessagesFile = testMessagesFile;
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
			this.saxParser.setEntityResolver ( new MyResolver() );

			saxParser.setContentHandler(this);

			saxParser
					.setFeature("http://xml.org/sax/features/validation", true);
			testMessages = new TestMessages();
		} catch (Exception ex) {
			logger.fatal("Unexpected exception", ex);
			ex.printStackTrace();
			System.exit(0);
		}

	}

	public TestMessages parse() {
		try {
			this.saxParser.parse(testMessagesFile);
			return testMessages;
		} catch (Exception ex) {
			logger.fatal("Unexpected exception", ex);
			ex.printStackTrace();
			System.exit(0);
		}
		return null;
	}

}
