package gov.nist.p25.issi.rfss.tester;

import gov.nist.p25.issi.issiconfig.GlobalTopologyParser;
import gov.nist.p25.issi.issiconfig.GroupConfig;
import gov.nist.p25.issi.issiconfig.RfssConfig;
import gov.nist.p25.issi.issiconfig.SuConfig;
import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;
import gov.nist.p25.issi.issiconfig.TopologyConfigParser;
import gov.nist.p25.issi.issiconfig.XMLTagsAndAttributes;
import gov.nist.p25.issi.p25payload.PacketType;
import gov.nist.p25.issi.rfss.ISSITesterConstants;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;

import javax.sip.message.Request;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parser for the XML test script.
 * 
 * @author M. Ranganathan
 *
 */
public class TestScriptParser extends DefaultHandler implements
		XMLTagsAndAttributes {
	private static Logger logger = Logger.getLogger(TestScriptParser.class);

	private TestScript retval;

	private XMLReader saxParser;

	private String testScriptName;

	private boolean inPostCondition;

	private StringBuffer cdatabuffer;

	private TopologyConfig topologyConfig;

	private String location;

	private String assertion;

	
	private SuScript testSuEventScenario;
	
	private AbstractScenario currentScenario;

	private boolean inTestSuEvent;

	private boolean inDescription;

	private boolean inTestRfssEvent;

	private RfssScript testRfssEventScenario;

	private String dirName;

	private Trigger currentTrigger;

	private boolean interactive;

	private String topologyFileName;

	private String globalTopology;

	private String systemTopology;

	// /////////////////////////////////////////////////////////////////////////////////////////
	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			logger.info("public id = " + publicId);
			logger.info("system id = " + systemId);
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/testscript.dtd")) {
				FileReader fileReader = new FileReader(new File(
						"dtd/testscript.dtd"));
				InputSource inputSource = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}

	public TestScript parse() throws Exception {
		
		saxParser.parse(testScriptName);
		return this.retval;
	}

	@Override
	public void startElement(String namespaceURI, String local, String name,
			Attributes attrs) throws SAXException {

		try {
			if (name.equals(DESCRIPTION)) {
				this.inDescription = true;
				this.cdatabuffer = new StringBuffer();
			} else if (name.equals(TEST_SCRIPT)) {
				ISSITesterConfiguration config = TestRFSS.getTesterConfiguration();
				TopologyConfig systemTopology = new SystemTopologyParser(config).parse(this.systemTopology);
			
				TopologyConfig globalTopology =   new GlobalTopologyParser(this.interactive)
						.parse( systemTopology, this.globalTopology);
				this.topologyConfig = new TopologyConfigParser().parse(
						globalTopology, this.topologyFileName);
				retval.setTopologyConfig(topologyConfig);
				String testName = attrs.getValue(TEST_NAME);
				retval.setTestName(testName);
				retval.setTestDirectory(this.dirName);
				
				int testCompletionDelay = 
					attrs.getValue(TEST_COMPLETION_DELAY) == null ? 0 :
						Integer.parseInt(attrs.getValue(TEST_COMPLETION_DELAY));
				
				
				
				retval.testCompletionDelay = testCompletionDelay;
				
				int traceGenerationTime = 
						attrs.getValue(TRACE_GENERATION_TIME) == null ? 25 :
								Integer.parseInt(attrs.getValue(TRACE_GENERATION_TIME));
				ISSITesterConstants.TEST_RUNS_FOR = traceGenerationTime*1000;
				
				retval.traceGenerationTime = traceGenerationTime*1000;
				
			
				if (null != attrs.getValue(XMLTagsAndAttributes.RTP_DEBUG_LOG)) {
					TestScript.rtpDebuglog = attrs
							.getValue(XMLTagsAndAttributes.RTP_DEBUG_LOG);

				}

				if (null != attrs
						.getValue(XMLTagsAndAttributes.RTP_MESSAGE_LOG)) {
					TestScript.rtpMessagelog = attrs.getValue(RTP_MESSAGE_LOG);
				}

				TestScript.generateSipTrace = attrs.getValue(
						XMLTagsAndAttributes.GENERATE_SIP_TRACE).equals("no") ? false
						: true;

				TestScript.generatePttTrace = attrs.getValue(
						XMLTagsAndAttributes.GENERATE_SIP_TRACE).equals("yes") ? true
						: false;

			} else if (name.equals(ROAMING)) {
				

				if (attrs.getValue(SU_NAME) == null) {
					throw new SAXException("Missing required attribute "
							+ SU_NAME);
				}
				SuConfig suConfig = topologyConfig.getSuConfigByName(attrs
						.getValue(SU_NAME));

				if (attrs.getValue(DESTINATION_RFSS_NAME) == null)
					throw new SAXException("Missing a required attribute "
							+ DESTINATION_RFSS_NAME + " tag = " + name);

				String initiatedBy = attrs.getValue(INITIATED_BY);

				boolean flag = initiatedBy == null ? false : initiatedBy
						.equals("home");

				RfssConfig rfssConfig = topologyConfig
						.getRfssConfigByName(attrs
								.getValue(DESTINATION_RFSS_NAME));
				RoamingScenario roamingScenario = new RoamingScenario(
						rfssConfig, suConfig, flag);
				this.currentScenario = roamingScenario;
				String testId = attrs.getValue(ID);
				roamingScenario.setId(testId);
				if (retval.scenarios.containsKey(testId)) {
					throw new Exception("Duplicate ID for test action");
				}
				retval.scenarios.put(testId, roamingScenario);
			} else if (name.equals(TRIGGER)) {
				String type = attrs.getValue(TRIGGER_TYPE);
				String value = attrs.getValue(TRIGGER_VALUE);
				String dependentScenarioName = attrs.getValue(REF_ID);
				logger.debug("refId for trigger = " + dependentScenarioName);
				AbstractScenario precondition = null;
				if ( dependentScenarioName != null && this.retval.getScenario(dependentScenarioName) == null) {
					throw new SAXException ("Cannot find dependent scenario");
				
				} else if ( dependentScenarioName != null && !type.equals(TIME)) {
					throw new SAXException("ONLY time triggers currently supported with dependencies");
				} else if ( dependentScenarioName != null){
					precondition = this.retval.getScenario(dependentScenarioName);
					if ( precondition == null) 
						throw new SAXException("Cannot find precondition! " + dependentScenarioName);
				}
				if ( type == null || value == null) throw new SAXException ("Missing a required attribute ");
				if ( type.equals(TIME) ) {
					this.currentTrigger = new TimeTrigger(Integer.parseInt(value));
					
				} else if ( type.equalsIgnoreCase("PTT_MESSAGE")) {
					if (value.equalsIgnoreCase("PTT_PROGRESS"))
							this.currentTrigger = new PttMessageTrigger( PacketType.PTT_TRANSMIT_PROGRESS );
					else if (value.equalsIgnoreCase("PTT_START"))
							this.currentTrigger = new PttMessageTrigger(PacketType.PTT_TRANSMIT_START);
					else if (value.equalsIgnoreCase("PTT_MUTE")) 
							this.currentTrigger = new PttMessageTrigger(PacketType.PTT_TRANSMIT_MUTE);
					else if (value.equalsIgnoreCase("PTT_UNMUTE")) 
						this.currentTrigger = new PttMessageTrigger(PacketType.PTT_TRANSMIT_UNMUTE);
					else if (value.equalsIgnoreCase("PTT_DENY"))
							this.currentTrigger = new PttMessageTrigger(PacketType.PTT_TRANSMIT_DENY);
					else if (value.equalsIgnoreCase("PTT_WAIT"))
							this.currentTrigger = new PttMessageTrigger(PacketType.PTT_TRANSMIT_WAIT);
					else if (value.equalsIgnoreCase("PTT_HEARTBEAT_QUERY"))
							this.currentTrigger = new PttMessageTrigger(PacketType.HEARTBEAT_QUERY);
					else if (value.equalsIgnoreCase("PTT_HEARTBEAT"))
						this.currentTrigger = new PttMessageTrigger(PacketType.HEARTBEAT);
					else throw new SAXException("Unknown PTT message trigger " + value);
				} else if ( type.equalsIgnoreCase("SIP_REQUEST") || type.equalsIgnoreCase("SIP_RESPONSE")) {
					this.currentTrigger = new SipMessageTrigger(value);
				} else {
					throw new SAXException ("Unknown tritter type " + type);
				}
				this.currentScenario.setTrigger(currentTrigger);
				if ( precondition != null ) {
					currentTrigger.addPrecondition(precondition);
				}
				this.currentTrigger.setScenario( this.currentScenario);
			} else if (name.equals(SU_TO_SU_CALL)) {

				
				if (attrs.getValue(CALLING_SU_NAME) == null)
					throw new SAXException("Missing required attribute "
							+ CALLING_SU_NAME);

				if (attrs.getValue(ID) == null)
					throw new SAXException("Missing required attribute" + ID);

				String id = attrs.getValue(ID);
				SuConfig callingSu = topologyConfig.getSuConfigByName(attrs
						.getValue(CALLING_SU_NAME));
				SuConfig calledSu = topologyConfig.getSuConfigByName(attrs
						.getValue(CALLED_SU_NAME));
				
				boolean isEmergency = attrs.getValue(IS_EMERGENCY) == null ? false
						: "true".equals(attrs.getValue(IS_EMERGENCY));

				
				
				int cancelAfter = attrs.getValue(CANCEL_AFTER) == null ||
					attrs.getValue(CANCEL_AFTER).equalsIgnoreCase(UNBOUNDED)? -1 : 
						Integer.parseInt(attrs.getValue(CANCEL_AFTER));
				int terminateAfter = attrs.getValue(TERMINATE_AFTER) == null
				|| attrs.getValue(TERMINATE_AFTER).equalsIgnoreCase(
						UNBOUNDED) ? -1 : Integer.parseInt(attrs
				.getValue(TERMINATE_AFTER));
	
				boolean isTerminatedByCalledParty  = attrs.getValue(IS_TERMINATED_BY_CALLED_SERVING) == null ? false :
					attrs.getValue(IS_TERMINATED_BY_CALLED_SERVING).equals("true");
				
				String talkSpurtSenderName = attrs.getValue(TALK_SPURT_SENDER) == null ? null : 
						attrs.getValue(TALK_SPURT_SENDER);
				
				SuConfig talkSpurtSender = null;
				if (talkSpurtSenderName == null ) {
					talkSpurtSender = null;
				} else if ( talkSpurtSenderName.equals(attrs
						.getValue(CALLING_SU_NAME))) {
					talkSpurtSender = callingSu;
					
				} else if ( talkSpurtSenderName.equals(attrs
						.getValue(CALLED_SU_NAME))) {
					talkSpurtSender = calledSu;
				}
				
				boolean forceFlag = attrs.getValue(IS_TALK_SPURT_FORCED) == null ? false :
					"true".equals(attrs.getValue(IS_TALK_SPURT_FORCED));
				
				
				
				boolean protectionFlag = attrs.getValue(IS_PROTECTED_CALL) == null ? false  : 
					"true".equals( attrs.getValue(IS_PROTECTED_CALL));
				
				SuToSuCallSetupScenario suToSuCallSetupScenario = new SuToSuCallSetupScenario(
						callingSu, calledSu,  isEmergency,
						  cancelAfter,terminateAfter,isTerminatedByCalledParty, talkSpurtSender, forceFlag,
						  protectionFlag);
				if ( talkSpurtSender == callingSu) {
					if ( talkSpurtSender.getHomeRfss().isEmulated() )
						talkSpurtSender.getHomeRfss().setCallingSuInitialTransmitter(true);
				} else if ( talkSpurtSender == calledSu) {
					if ( talkSpurtSender.getHomeRfss().isEmulated())
						talkSpurtSender.getHomeRfss().setCallingSuInitialTransmitter(false);
				}
				suToSuCallSetupScenario.setTrigger( currentTrigger);
				suToSuCallSetupScenario.setId(id);

				if (retval.scenarios.containsKey(id)) {
					throw new Exception("Duplicate key for test action " + id);
				}
				retval.scenarios.put(id, suToSuCallSetupScenario);
				this.currentScenario = suToSuCallSetupScenario;
				
				if (terminateAfter != -1) {
					SuConfig terminatingSu = isTerminatedByCalledParty ? calledSu : callingSu;
					CallTerminator callTerminator = new CallTerminator(terminatingSu);
					String scenarioName = id+"-call-terminator";
					callTerminator.setId(scenarioName);
					TimeTrigger timeTrigger = new TimeTrigger(terminateAfter);
					timeTrigger.addPrecondition(suToSuCallSetupScenario);
					callTerminator.setTrigger(timeTrigger);
					timeTrigger.setScenario(callTerminator);
					callTerminator.setDescription("Hang up the SU to SU call from SU %x" +
							Integer.toHexString( terminatingSu.getSuId()) + " (isEmulated = " + terminatingSu.isEmulated() + ")");
					retval.scenarios.put ( scenarioName, callTerminator);
				}

			} else if (name.equals(GROUP_CALL)) {
				

				String id = attrs.getValue(ID);
			
				GroupConfig calledGroup = topologyConfig
						.getGroupConfigByName(attrs.getValue(CALLED_GROUP_NAME));
				SuConfig callingSu = topologyConfig.getSuConfigByName(attrs
						.getValue(CALLING_SU_NAME));
				String priorityString = attrs.getValue(PRIORITY);
				int priority = priorityString == null ? 1 : Integer
						.parseInt(priorityString);
				boolean isEmergency = attrs.getValue(IS_EMERGENCY) == null ? false
						: "true".equals(attrs.getValue(IS_EMERGENCY));

				boolean isConfirmed = attrs.getValue(IS_CONFIRMED) == null ? false
						: "true".equals(attrs.getValue(IS_CONFIRMED));

				int cancelAfter = attrs.getValue(CANCEL_AFTER) == null
						|| attrs.getValue(CANCEL_AFTER).equalsIgnoreCase(
								UNBOUNDED) ? -1 : Integer.parseInt(attrs
						.getValue(CANCEL_AFTER));
				int terminateAfter = attrs.getValue(TERMINATE_AFTER) == null
				|| attrs.getValue(TERMINATE_AFTER).equalsIgnoreCase(
						UNBOUNDED) ? -1 : Integer.parseInt(attrs
				.getValue(TERMINATE_AFTER));
	
				boolean talkSpurtForced = attrs.getValue(IS_TALK_SPURT_FORCED) == null ? false : 
					Boolean.parseBoolean(attrs.getValue(IS_TALK_SPURT_FORCED));
				
				boolean isTalkSpurtSentAfterCallSetup = attrs.getValue(IS_TALK_SPURT_SENT_AFTER_CALL_SETUP) == null ? false : 
					Boolean.parseBoolean(attrs.getValue(IS_TALK_SPURT_SENT_AFTER_CALL_SETUP));
				
				boolean isProtected = attrs.getValue(IS_PROTECTED_CALL) == null ? false :
					Boolean.parseBoolean(attrs.getValue(IS_PROTECTED_CALL));
				GroupCallSetupScenario scenario = new GroupCallSetupScenario(
						callingSu, calledGroup,  priority, cancelAfter,
						terminateAfter,
						isEmergency, isConfirmed,isTalkSpurtSentAfterCallSetup, talkSpurtForced,isProtected);
					
				scenario.setTrigger(currentTrigger);
				scenario.setId(id);
				if (retval.scenarios.containsKey(id)) {
					throw new Exception("Duplicate id for test action " + id);
				}
				retval.scenarios.put(id, scenario);
				this.currentScenario = scenario;
			} else if ("post-condition".equalsIgnoreCase(name)) {
				this.inPostCondition = true;
				this.location = attrs.getValue("locationSelector");
				this.assertion = attrs.getValue("assertion");
				this.cdatabuffer = new StringBuffer();

			} else if (SU_SCRIPT.equalsIgnoreCase(name)) {
				String id = attrs.getValue(ID);
				if (id == null) {
					throw new Exception("Missing a required attribute : id ");
				}
				if (retval.scenarios.containsKey(id)) {
					throw new Exception("Duplicate key for test action " + id);
				}
				

				if (attrs.getValue(SU_NAME) == null) {
					throw new Exception("Missing required attribute : "
							+ SU_NAME);
				}
				String method = attrs.getValue(METHOD);

				if (method == null) {
					throw new Exception("Missing a required attribute : "
							+ METHOD);
				}
				
				this.testSuEventScenario = new SuScript(retval);
				this.inTestSuEvent = true;
				this.cdatabuffer = new StringBuffer();

				String description = attrs.getValue(DESCRIPTION);

				if (attrs.getValue(SU_NAME) != null) {
					SuConfig suConfig = this.topologyConfig
							.getSuConfigByName(attrs.getValue(SU_NAME));
					this.testSuEventScenario.setId(id);
					this.testSuEventScenario.suConfig = suConfig;
					this.testSuEventScenario.method = method;
					this.testSuEventScenario.setTrigger(currentTrigger);
					if (description != null)
						this.testSuEventScenario.setDescription(description);
				} else {
					throw new Exception("Missing a required attribute: suId");
				}
				this.currentScenario = testSuEventScenario;
				
			} else if (RFSS_SCRIPT.equalsIgnoreCase(name)) {
				String id = attrs.getValue(ID);
				String description = attrs.getValue(DESCRIPTION);

				if (id == null) {
					throw new Exception("Missing a required attribute : id ");
				}
				if (retval.scenarios.containsKey(id)) {
					throw new Exception("Duplicate key for test action " + id);
				}
				
				if (attrs.getValue(RFSS_NAME) == null) {
					throw new Exception("Missing a required attribute : "
							+ RFSS_NAME);
				}
				String rfssname = attrs.getValue(RFSS_NAME);
				if (rfssname.indexOf(":") != -1) {
				
					String[] parts = rfssname.split(":");

					String gname = parts[0];
					String operation = parts[1];
					if (operation.equals("groupHome")) {
						GroupConfig groupConfig = topologyConfig
								.getGroupConfigByName(gname);
						if (groupConfig == null)
							throw new Exception("Group not found " + gname);
						rfssname = groupConfig.getHomeRfss().getRfssName();
					} else if (operation.equals("homeRfss")) {
						SuConfig suConfig = topologyConfig
								.getSuConfigByName(gname);
						if (suConfig == null)
							throw new Exception("SU not found " + gname);
						rfssname = suConfig.getHomeRfss().getRfssName();
					} else if (operation.equals("initialServingRfss")) {
						SuConfig suConfig = topologyConfig
								.getSuConfigByName(gname);
						if (suConfig == null)
							throw new Exception("SU not found " + gname);
						rfssname = suConfig.getInitialServingRfss().getRfssName();
					}
				}
				String method = attrs.getValue(METHOD);

				if (method == null) {
					throw new Exception("Missing a required attribute : "
							+ METHOD);
				}
				
				this.testRfssEventScenario = new RfssScript(retval);
				this.inTestRfssEvent = true;
				this.cdatabuffer = new StringBuffer();
				RfssConfig rfssConfig = this.topologyConfig
						.getRfssConfigByName(rfssname);
				if (rfssConfig == null) {
					throw new Exception("Rfss not found " + rfssname);
				}
				this.testRfssEventScenario.setId(id);
				this.testRfssEventScenario.rfssConfig = rfssConfig;
				this.testRfssEventScenario.method = method;
				this.testRfssEventScenario.setTrigger(currentTrigger);
				this.testRfssEventScenario.setDescription(description);
				this.currentScenario = testRfssEventScenario;

			} else if ( "script".equalsIgnoreCase(name)) {
				logger.debug("script");
			}
		} catch (Exception ex) {
			logger.fatal("Error Parsing Test Script", ex);
			throw new SAXException(
					"Missing required information in confiuration file ", ex);
		}

	}

	@Override
	public void characters(char[] ch, int start, int length) {
		if (this.inPostCondition || this.inTestSuEvent || this.inTestRfssEvent
				|| this.inDescription)
			this.cdatabuffer.append(ch, start, length);

	}

	@Override
	public void endElement(String namespaceURI, String local, String name)
			throws SAXException {
		 if ("post-condition".equalsIgnoreCase(name)) {
			this.inPostCondition = false;
			try {
				if (this.cdatabuffer.length() > 0) {
					PostCondition pc = new PostCondition(
							this.assertion, this.cdatabuffer.toString(),
							this.location);
					retval.addPostCondition(pc);
				}
			} catch (Exception ex) {
				logger.fatal("Error parsing test script", ex);
				throw new SAXException(
						"Could not instantiate scripting engine for predicate",
						ex);
			}
		} else if (SU_SCRIPT.equalsIgnoreCase(name)) {
			this.testSuEventScenario.setTrigger(currentTrigger);
			
			this.inTestSuEvent = false;
			if (this.cdatabuffer.length() > 0) {
				this.testSuEventScenario.script = this.cdatabuffer.toString();
				try {
					this.testSuEventScenario.init();

					retval.scenarios.put(this.testSuEventScenario.getId(),
							this.testSuEventScenario);
				} catch (Exception e) {
					e.printStackTrace();
					throw new SAXException(
							"Could not instantiate the su script ", e);
				}
			}
		} else if (RFSS_SCRIPT.equalsIgnoreCase(name)) {
			this.inTestRfssEvent = false;
			this.testRfssEventScenario.setTrigger(this.currentTrigger);
			if (this.cdatabuffer.length() > 0) {
				try {
					this.testRfssEventScenario.script = this.cdatabuffer
							.toString();
					this.testRfssEventScenario.init();
					retval.scenarios.put(this.testRfssEventScenario.getId(),
							this.testRfssEventScenario);
				} catch (Exception e) {
					e.printStackTrace();
					throw new SAXException(
							"Could not instantiate the rfss script ", e);
				}
			}

		}
	}

	public TestScriptParser(String testScriptFileName, String topologyFileName , String globalTopology, String systemTopology, boolean interactive) throws Exception {
		try {

			this.retval = new TestScript();
			this.interactive = interactive;
			this.retval.setTraceCaptureEnabled(!interactive);
			this.globalTopology = globalTopology;
			this.systemTopology = systemTopology;
			this.testScriptName = testScriptFileName;
			logger.debug("Parsing " + testScriptFileName + "\nglobalTopology " + this.globalTopology);
			logger.debug("Topology = " + topologyFileName);
			logger.debug("systemTopology = " + systemTopology);
			this.topologyFileName = topologyFileName;
			
			

			File file = new File(testScriptFileName);
			this.dirName = file.getParent();

			
			if (! new File(globalTopology).exists() || ! new File(globalTopology).isFile()) {
				logger.error("File not found " + globalTopology);
				throw new IOException ("File not found " + globalTopology);
			}
			
			if ( ! new File(systemTopology).exists() || ! new File(systemTopology).isFile()) {
				logger.error("File not found " + systemTopology);
				throw new IOException ("File not found " + systemTopology);
			}
			
			if ( !new File(testScriptFileName).exists()|| !new File(testScriptFileName).isFile()) {
				logger.error("File not found " + testScriptFileName);
				throw new IOException ("File not found " + testScriptFileName);
			}
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
			this.saxParser.setEntityResolver(new MyResolver());

			saxParser.setContentHandler(this);

			saxParser
					.setFeature("http://xml.org/sax/features/validation", true);
			
		} catch (Exception pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();
			throw pce;
		}

	}
}
