/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */
package gov.nist.p25.issi.testlauncher;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class implements the about dialog.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/04/22 00:59:13 $
 * @since 1.5
 */
public class AboutDialog extends JDialog implements ActionListener
{ 
	/***************************************************************************
	 * Constants
	 **************************************************************************/
	
	private static final long serialVersionUID = 0;
	
	
	/***************************************************************************
	 * Constructors
	 **************************************************************************/
	
   /**
    * Constructor.
    * @param frame The main GUI frame.
    */
   public AboutDialog(Frame frame, String version, String build) 
   {
      super(frame, true);
      setTitle("About");
      
      JLabel iconLabel = new JLabel(new ImageIcon("gfx/diets.jpg"));
      Dimension iconDimension = new Dimension(432, 180);
      iconLabel.setMinimumSize(iconDimension);
      iconLabel.setPreferredSize(iconDimension);
      
      String disclaimer = 
    	  	"THIS SOFTWARE is provided \"AS IS\" WITHOUT ANY \n" +
    	  	"WARRANTY OF ANY TYPE AS TO ANY MATTER WHATSOEVER,\n" +
      		"EXPRESSED OR IMPLIED, INCLUDING NO IMPLIED \n" +
      		"WARRANTIES OF MERCHANTABILITY AND NO WARRANTIES \n" +
      		"OF FITNESS FOR A PARTICULAR PURPOSE. \n\n" +
      		"Users acknowledge that the National Institute of\n" +
      		"Standards and Technology (NIST) has no obligation\n" +
      		"to correct any defect or problem with this \n" +
      		"software or documentation. NIST is under no \n" +
      		"obligation to provide future software updates, \n" +
      		"and is under no obligation to provide assistance,\n" +
      		"service, help or advice to User. NIST is under no\n" +
      		"obligation to make or provide any modifications\n" +
      		"to the Software. This software is not intended to\n" +
      		"substitute for acquiring and application of \n" +
      		"independent engineering, communications or other\n" +
      		"expertise. Users of the software are solely\n" +
      		"responsible for assuring the safety, reliability \n" +
      		"and operability of any product developed out of\n" +
      		"the use of the software.\n\n" +
      		"NIST disclaims any liability whatsoever related\n" +
      		"to the use or distribution of this software.\n\n" +
      		"For more information, visit:\n" +
      		"http://www-x.antd.nist.gov/proj25";

      JTextArea textArea = new JTextArea("DIETS Version " + version + 
    		  ", Build " + build + "\n\n" + disclaimer);

      textArea.setLineWrap(true);
      JScrollPane scrollPane = new JScrollPane(textArea);
      Dimension scrollPaneDimension = new Dimension(432, 200);
      scrollPane.setMinimumSize(scrollPaneDimension);
      scrollPane.setPreferredSize(scrollPaneDimension);
      
      JPanel mainPanel = new JPanel();
      mainPanel.setLayout(new BorderLayout());
      mainPanel.add(iconLabel, BorderLayout.CENTER);
      
      JButton okButton = new JButton("OK");
      okButton.setActionCommand("Ok");
      okButton.addActionListener(this);
      
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new BorderLayout());
      buttonPanel.add(scrollPane, BorderLayout.CENTER);
      buttonPanel.add(okButton, BorderLayout.SOUTH);
      

      mainPanel.add(buttonPanel, BorderLayout.SOUTH);
      setContentPane(mainPanel);

      this.pack();
      
      this.addWindowListener(new WindowAdapter() {

    	  public void windowClosing(WindowEvent e) {

    		  setVisible(false);

    	  }
      });
	}   
  
   
	/***************************************************************************
	 * Methods
	 **************************************************************************/
   
   /**
    * This method traps GUI events.
    * @param ae the trapped event.
    */
   public void actionPerformed(ActionEvent ae)
   {      
      if (ae.getActionCommand().equals("Ok"))
      {
         setVisible(false);
      }
   }
}
