/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.testlauncher;

import javax.swing.*;

import org.apache.log4j.Logger;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Configuration dialog.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/08/06 00:03:27 $
 * @since 1.5
 */
public class ConfigDialog extends JDialog implements ActionListener {

	private JButton saveButton = null;

	private JButton cancelButton = null;

	private boolean saved = false;

	private JTextArea textArea;

	private File file;

	private static Logger logger = Logger.getLogger(ConfigDialog.class);
	
	private DietsGUI gui;

	/** Creates the reusable dialog. */
	public ConfigDialog(DietsGUI aFrame, File configFile, String title) {
		super(aFrame, true);

		this.setMinimumSize(new Dimension(500, 800));
		this.setPreferredSize(new Dimension(500, 800));
		this.setMaximumSize(new Dimension(500, 800));
		this.setSize(500, 800);
		this.file = configFile;
		this.gui = aFrame;

		try {

			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

			this.textArea = new JTextArea();

			this.textArea.setEditable(true);

			FileReader fr = new FileReader(configFile);

			char[] contents = new char[(int) configFile.length()];
			fr.read(contents);
			fr.close();

			String str = new String(contents);

			this.textArea.setText(str);

			JScrollPane scrollPane = new JScrollPane(textArea,
					JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
					JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

			setTitle(title);

			JPanel panel = new JPanel();
			panel.setLayout(new BorderLayout());

			saveButton = new JButton("Save");
			saveButton.addActionListener(this);
			saveButton.setEnabled(true); // TODO - this will be enabled when we are ready.

			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(this);

			JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout(new GridLayout(1, 2));
			buttonPanel.add(saveButton);
			buttonPanel.add(cancelButton);

			panel.add(scrollPane, BorderLayout.CENTER);
			panel.add(buttonPanel, BorderLayout.SOUTH);

			setContentPane(panel);

			// Handle window closing correctly.
			setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

			// addWindowListener(new WindowAdapter() {
			// public void windowClosing(WindowEvent we) {
			// /*
			// * Instead of directly closing the window,
			// * we're going to change the JOptionPane's
			// * value property.
			// */
			// optionPane.setValue(new Integer(
			// JOptionPane.CLOSED_OPTION));
			// }
			// });
		} catch (Exception e) {

			logger.error("Unexpected exception ", e);
			return;

		}

	}

	/** This method handles events for the text field. */
	public void actionPerformed(ActionEvent ae) {

		Object source = ae.getSource();

		if (source == cancelButton) {

			setSaved(false);
			setVisible(false);

		} else if (source == saveButton) {
			try {
				/*
				 * Note that we do not check to see if the file is dirty, we
				 * just simply rewrite the file even if no changes were made.
				 */
				String textToSave = this.textArea.getText();
				FileWriter fw = new FileWriter(this.file);
				fw.write(textToSave);
				fw.close();
			
			} catch (Exception ex) {
				logger.error("Unexpected exception occured", ex);
				return;
			}

			setSaved(true);
			setVisible(false);
		}

	}

	/**
	 * @param saved the saved to set
	 */
	private void setSaved(boolean saved) {
		this.saved = saved;
	}

	/**
	 * @return the saved
	 */
	public boolean isSaved() {
		return saved;
	}

}
