/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.testlauncher;

import gov.nist.p25.issi.ISSITimer;
import gov.nist.p25.issi.issiconfig.GlobalTopologyParser;
import gov.nist.p25.issi.issiconfig.RfssConfig;
import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;
import gov.nist.p25.issi.packetmonitor.PacketMonitor;
import gov.nist.p25.issi.packetmonitor.gui.PacketMonitorGUI;
import gov.nist.p25.issi.rfss.ISSITesterConstants;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfiguration;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfigurationParser;
import gov.nist.p25.issi.rfss.tester.RfssController;
import gov.nist.p25.issi.rfss.tester.TestRFSS;
import gov.nist.p25.issi.startup.DietsConfigProperties;
import gov.nist.p25.issi.traceverifier.PttTraceVerifier;
import gov.nist.p25.issi.traceverifier.SipTraceVerifier;
import gov.nist.p25.issi.traceverifier.StatusFlag;
import gov.nist.p25.issi.traceverifier.VerificationRecord;
import gov.nist.p25.issi.utils.Zipper;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.Reader;
import java.util.Collection;
import java.util.HashSet;
import java.util.Properties;
import java.util.TimerTask;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class implements the front end test controller and trace viewer GUI for
 * DIETS.
 * 
 * @author mranga@nist.gov
 * @author steveq@nist.gov
 * @version $Revision: 1.79 $, $Date: 2007/12/07 02:36:29 $
 * @since 1.5
 */
public class DietsGUI extends JFrame implements ActionListener, MouseListener {

	static {
		PropertyConfigurator.configure("log4j.properties");
	}

	private static final long serialVersionUID = 1L;

	private static Logger logger = Logger.getLogger(DietsGUI.class);

	/***************************************************************************
	 * Constants
	 **************************************************************************/

	// private static final long serialVersionUID = 0;
	private static final String VERSION = ISSITesterConstants.VERSION;

	private static final String BUILD = ISSITesterConstants.BUILD;

	public static final int TEST_TIMEOUT = ISSITesterConstants.TEST_RUNS_FOR + 5 * 1000; // in

	// miliseconds

	// private static String ipAddress = null;

	private static String testerConfigurationFile = null;

	private static boolean standalone = false;

	public static String testRegistry = ISSITesterConstants.getScenarioDir()
			+ "/" + ISSITesterConstants.TEST_REGISTRY_XML;

	public static boolean evaluatePassFail = true;

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	// private static String testTopologyFileName = null;
	private TestControllerInterface remoteController = null;

	// public JMenuItem openGlobalTopologyItem = null;

	public JTabbedPane tabbedPane = null;

	private JFileChooser fileChooser = null;

	private JMenuItem closeMenuItem = null;

	private JMenuItem closeAllMenuItem = null;

	private TestExecutionPanel testExecutionPanel = null;

	public static final String splashScreenMessage = "Loading...";

	private static String globalTopologyFileName;

	private static int PREFERRED_WIDTH = 1050;

	private static int PREFERRED_HEIGHT = 850;

	private static String systemTopologyFileName = ISSITesterConstants.TESTSUITES_DIR
			+ "/"
			+ ISSITesterConstants.getTestSuite()
			+ "/"
			+ ISSITesterConstants.SYSTEM_TOPOLOGY_XML;

	private static String testSuite = ISSITesterConstants.getTestSuite();

	private boolean interactive;

	private JLabel statusLabel = null;

	private JProgressBar statusProgressBar = null;

	private boolean isTraceViewer;

	private JMenuItem exitMenuItem;

	private JMenuItem openMenuItem;

	private JMenuItem aboutMenuItem;

	private JMenuItem saveMenuItem;
	
	private JMenuItem createTestrunZip;

	private JMenuItem verifyTraceMenuItem;

	private JMenuItem systemTopologyMenuItem;

	private JMenuItem startupPropertiesMenuItem;

	private JMenuItem loadPropertiesMenuItem;

	private static ISSITesterConfiguration issiTesterConfiguration;

	private static Collection<String> ipAddresses;

	private static String traceSource;

	private static String startupPropertiesFileName = ISSITesterConstants.STARTUP_PROPERTIES_FILENAME;

	private static DietsGUI theGui;

	private static Vector<TestCaseDescriptor> tests;

	// private String webConfigurationFileName;

	/***************************************************************************
	 * get the static flag that indicates whether we are a standalone client.
	 **************************************************************************/

	/**
	 * @return the standalone
	 */
	public static boolean isStandalone() {
		return standalone;
	}

	/**
	 * Get the available test cases.
	 * 
	 */
	private static Vector<TestCaseDescriptor> getAvailableTestCases()
			throws Exception {
		TestRegistryParser parser = new TestRegistryParser(testRegistry);
		parser.parse();

		return parser.getTestCaseList();

	}

	/***************************************************************************
	 * Private methods.
	 **************************************************************************/

	/**
	 * @param statusLabel
	 *            the statusLabel to set
	 */
	private void setStatusLabel(JLabel statusLabel) {
		this.statusLabel = statusLabel;
	}

	private Point centerComponent(Component c) {

		Rectangle rc = new Rectangle();
		rc = c.getBounds(rc);

		Rectangle rs = new Rectangle(Toolkit.getDefaultToolkit()
				.getScreenSize());

		return new Point((int) ((rs.getWidth() / 2) - (rc.getWidth() / 2)),
				(int) ((rs.getHeight() / 2) - (rc.getHeight() / 2)));

	}

	/**
	 * @param statusProgressBar
	 *            the statusProgressBar to set
	 */
	private void setStatusProgressBar(JProgressBar statusProgressBar) {
		this.statusProgressBar = statusProgressBar;
	}

	public void updateGlobalTopologyPane() {
		try {
			StringBuffer stringBuffer = new StringBuffer();
			BufferedReader freader = new BufferedReader(new FileReader(
					new File(globalTopologyFileName)));
			String line;
			while ((line = freader.readLine()) != null) {
				stringBuffer.append(line + "\n");
			}
			int len = this.testExecutionPanel.getGlobalTopologyPane()
					.getDocument().getLength();
			this.testExecutionPanel.getGlobalTopologyPane().getDocument()
					.remove(0, len);
			this.testExecutionPanel.getGlobalTopologyPane().getDocument()
					.insertString(0, stringBuffer.toString(),
							TestExecutionPanel.INFO_TEXT);
			freader.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public void updateSystemTopologyPane() {
		try {
			StringBuffer stringBuffer = new StringBuffer();
			BufferedReader freader = new BufferedReader(new FileReader(
					new File(systemTopologyFileName)));
			String line;
			while ((line = freader.readLine()) != null) {
				stringBuffer.append(line + "\n");
			}
			int len = this.testExecutionPanel.getSystemTopologyPane()
					.getDocument().getLength();
			this.testExecutionPanel.getSystemTopologyPane().getDocument()
					.remove(0, len);

			this.testExecutionPanel.getSystemTopologyPane().getDocument()
					.insertString(0, stringBuffer.toString(),
							TestExecutionPanel.INFO_TEXT);
			freader.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	public DietsGUI() throws Exception {

		super();

		initGui();

		try {

			if (standalone) {
				this.remoteController = new RfssController(ipAddresses);
			} else {
				this.remoteController = new RemoteTestController(this);
			}

		} catch (Exception e) {

			e.printStackTrace();

		}

	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	public void initGui() throws Exception {

		logger.info("GUI system initializing!");
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

		// File chooser used to open the trace file.
		fileChooser = new JFileChooser(System.getProperty("user.dir"));
		fileChooser.setDialogTitle("Open trace directory");
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		FileFilter traceFilter = new FileFilter() {

			@Override
			public boolean accept(File pathname) {
				if (pathname.isDirectory())
					return true;
				else
					return false;
			}

			@Override
			public String getDescription() {

				return "tracedir";
			}

		};
		fileChooser.setFileFilter(traceFilter);

		JMenuBar menuBar = new JMenuBar();

		// ---------------------- File ----------------------------

		JMenu fileMenu = new JMenu("File");

		this.openMenuItem = new JMenuItem("Open trace dir");
		openMenuItem.setActionCommand("Open");
		openMenuItem.addActionListener(this);
		this.loadPropertiesMenuItem = new JMenuItem("Load startup properties");
		this.loadPropertiesMenuItem.setActionCommand("LoadProperties");
		this.loadPropertiesMenuItem.addActionListener(this);
		openMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				ActionEvent.CTRL_MASK));

		this.saveMenuItem = new JMenuItem("Save trace for this test");
		saveMenuItem.setActionCommand("Save");
		saveMenuItem.addActionListener(this);
		saveMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				ActionEvent.CTRL_MASK));
		
		this.createTestrunZip = new JMenuItem("Create Testrun Zip");
		createTestrunZip.setActionCommand("Zip");
		createTestrunZip.addActionListener(this);
		createTestrunZip.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z,
				ActionEvent.CTRL_MASK));

		this.verifyTraceMenuItem = new JMenuItem("Verify saved traces");
		verifyTraceMenuItem.setActionCommand("Verify Saved Trace");
		verifyTraceMenuItem.addActionListener(this);
		verifyTraceMenuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_V, ActionEvent.CTRL_MASK));

		closeMenuItem = new JMenuItem("Close");
		closeMenuItem.setEnabled(false);
		closeMenuItem.setActionCommand("Close");
		closeMenuItem.addActionListener(this);
		closeMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
				ActionEvent.CTRL_MASK));

		closeAllMenuItem = new JMenuItem("Close All");
		closeAllMenuItem.setEnabled(false);
		closeAllMenuItem.setActionCommand("CloseAll");
		closeAllMenuItem.addActionListener(this);
		closeAllMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
				ActionEvent.CTRL_MASK));

		this.exitMenuItem = new JMenuItem("Exit");
		exitMenuItem.setActionCommand("Exit");
		exitMenuItem.addActionListener(this);
		exitMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
				ActionEvent.CTRL_MASK));

		// fileMenu.add(openGlobalTopologyItem);
		// fileMenu.addSeparator();

		fileMenu.add(openMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(this.loadPropertiesMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(saveMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(this.verifyTraceMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(this.createTestrunZip);
		fileMenu.addSeparator();

		// ---------------------- Options ----------------------------

		JMenu preferencesMenu = new JMenu("Edit");
		preferencesMenu.setEnabled(true);
		this.startupPropertiesMenuItem = new JMenuItem("Startup Properties");
		this.systemTopologyMenuItem = new JMenuItem("System Topology");
		systemTopologyMenuItem.setActionCommand("Options");
		systemTopologyMenuItem.addActionListener(this);
		startupPropertiesMenuItem.addActionListener(this);
		preferencesMenu.add(startupPropertiesMenuItem);
		preferencesMenu.add(systemTopologyMenuItem);
		fileMenu.add(preferencesMenu);

		fileMenu.addSeparator();

		fileMenu.add(exitMenuItem);

		JMenu helpMenu = new JMenu("Help");
		helpMenu.setEnabled(true);

		this.aboutMenuItem = new JMenuItem("About DIETS");
		aboutMenuItem.setActionCommand("About");
		aboutMenuItem.addActionListener(this);

		helpMenu.add(aboutMenuItem);

		menuBar.add(fileMenu);
		menuBar.add(helpMenu);

		setJMenuBar(menuBar);

		this.testExecutionPanel = new TestExecutionPanel(tests, this);
		StringBuffer stringBuffer = new StringBuffer();
		updateGlobalTopologyPane();
		updateSystemTopologyPane();

		this.testExecutionPanel.setMinimumSize(new Dimension(
				DietsGUI.PREFERRED_WIDTH, 200));
		this.testExecutionPanel.setPreferredSize(new Dimension(
				DietsGUI.PREFERRED_WIDTH, 225));
		this.testExecutionPanel.setMaximumSize(new Dimension(
				DietsGUI.PREFERRED_WIDTH, 325));

		JPanel tabbedPanel = new JPanel();
		tabbedPanel.setLayout(new BorderLayout());
		tabbedPanel.setBorder(new TitledBorder("Test Traces"));

		tabbedPane = new JTabbedPane();
		tabbedPanel.add(tabbedPane, BorderLayout.CENTER);

		setIconImage(new ImageIcon("gfx/nist.gif").getImage());

		setTitle("DIETS v" + VERSION);

		getContentPane().setLayout(new BorderLayout());

		getContentPane().add(testExecutionPanel, BorderLayout.NORTH);

		JPanel statusPanel = new JPanel();
		statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
		setStatusLabel(new JLabel("Ready"));
		getStatusLabel().setMinimumSize(new Dimension(300, 16));
		getStatusLabel().setPreferredSize(new Dimension(700, 16));
		getStatusLabel().setMaximumSize(new Dimension(900, 16));
		setStatusProgressBar(new JProgressBar());
		getStatusProgressBar().setMinimumSize(new Dimension(100, 16));
		getStatusProgressBar().setPreferredSize(new Dimension(150, 16));
		getStatusProgressBar().setMaximumSize(new Dimension(200, 16));
		getStatusProgressBar().setIndeterminate(false);

		statusPanel.add(getStatusLabel());
		statusPanel.add(getStatusProgressBar());

		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				testExecutionPanel, tabbedPanel);
		getContentPane().add(splitPane, BorderLayout.CENTER);

		getContentPane().add(statusPanel, BorderLayout.SOUTH);

		setSize(PREFERRED_WIDTH, PREFERRED_HEIGHT);
		Dimension d = tabbedPane.getSize();
		Point upperLeftPoint = centerComponent(this);
		setLocation(upperLeftPoint);
		this.setAlwaysOnTop(true);

		ISSITimer.getTimer().schedule(new TimerTask() {

			@Override
			public void run() {

				setAlwaysOnTop(false);
			}

		}, 1000);

		this.setVisible(true);
		Dimension z = getStatusProgressBar().getSize();
		double height = z.getHeight();
		double width = z.getWidth();
		logger.info("height: " + height + ", width: " + width);

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				DietsGUI.this.dispose();
			}
		});
	}

	public static String getGlobalTopologyName() {

		return globalTopologyFileName;

	}

	public JMenuItem getCloseMenuItem() {
		return closeMenuItem;
	}

	/**
	 * @return the statusProgressBar
	 */
	public JProgressBar getStatusProgressBar() {
		return statusProgressBar;
	}

	/**
	 * @return the statusLabel
	 */
	public JLabel getStatusLabel() {
		return statusLabel;
	}

	public TestExecutionPanel getTestExecutionPanel() {
		return testExecutionPanel;
	}

	public JMenuItem getCloseAllMenuItem() {
		return closeAllMenuItem;
	}

	public boolean isTraceViewer() {
		return this.isTraceViewer;
	}

	public void resetRemoteController() {
		this.remoteController.reset();

	}

	/**
	 * @return the remoteController
	 */
	public TestControllerInterface getRemoteController() {
		return remoteController;
	}

	/**
	 * The file name for the file that indicates where the testers are running
	 * 
	 * @return -- the web configuration file name.
	 */

	public String getWebConfigurationFileName() {
		return testerConfigurationFile;
	}

	public boolean isInteractive() {
		return interactive;
	}

	public void actionPerformed(ActionEvent ae) {

		// Using the command is not a good idea - if you change the command in
		// one place, your code will break.
		if (ae.getSource() == this.exitMenuItem) {
			DietsGUI.this.dispose();
			// System.exit(0);
		} else if (ae.getSource() == this.openMenuItem) {
			try {

				openFile();
			} catch (Exception ex) {
				logger.error("Exception caught", ex);
				this.showError("Error opening trace directory",
						"Load trace error");
			}

		} else if (ae.getSource() == this.loadPropertiesMenuItem) {
			try {
				loadProperties();
			} catch (Exception ex) {
				logger.error("Exception caught", ex);

			}
		} else if (ae.getSource() == this.saveMenuItem) {
			try {
				saveFiles();
			} catch (Exception ex) {
				logger.error("Exception caught", ex);
				this.showError("Error saving traces  " + ex.getMessage(), "Save trace error");
			}

		} else if ( ae.getSource() == this.createTestrunZip) {
			try {
				this.zipFiles();
			} catch (Exception ex) {
				logger.error("Exception caught", ex);
				this.showError("Error zipping testrun " + ex.getMessage(),
				"Zip Error");
			}
		} else if (ae.getSource() == this.verifyTraceMenuItem) {
			try {
				TestCaseDescriptor testCaseDescriptor = this.testExecutionPanel
						.getCurrentTest();
				String testSuiteName = testSuite;
				String topologyFileName = testCaseDescriptor
						.getTopologyFileName();
				ISSITesterConfiguration config = new ISSITesterConfigurationParser(
						testerConfigurationFile).parse();
				String testName = testCaseDescriptor.getDirectoryName();
				StringBuffer failureReason = new StringBuffer();
				boolean passed = true;

				SipTraceVerifier.testCapturedMessages(config, "./",
						testSuiteName, testName, topologyFileName);

				PttTraceVerifier.testCapturedMessages(config, "./",
						testSuiteName, testName, topologyFileName);
			} catch (Exception ex) {
				logger.error("An unepected exception occured", ex);
			}

		} else if (ae.getSource().equals("Config")) {

			this.showWarning("Configuration is not yet enabled.",
					"Implementation ToDo");

		} else if (ae.getSource() == this.closeMenuItem) {

			int showing = tabbedPane.getSelectedIndex();
			System.out.println("tab #" + showing);
			tabbedPane.remove(showing);

			if (tabbedPane.getComponentCount() == 0) {
				getCloseMenuItem().setEnabled(false);
				getCloseAllMenuItem().setEnabled(false);
			}

		} else if (ae.getSource() == this.aboutMenuItem) {

			AboutDialog about_dialog = new AboutDialog(this, VERSION, BUILD);
			// about_dialog.setSize(350, 175);
			about_dialog.setLocationRelativeTo(this);
			about_dialog.setVisible(true);

		} else if (ae.getSource() == this.systemTopologyMenuItem) {
			try {

				this.testExecutionPanel.disableAllButtons();

				String topologyFileName = DietsGUI.getSystemTopologyName();
				File file = new File(topologyFileName);

				ConfigDialog customDialog = new ConfigDialog(this, file,
						"System Topology ");
				customDialog.setLocationRelativeTo(this);
				customDialog.setVisible(true);

				/* Dialog returns here. */
				if (customDialog.isSaved()) {

					FileReader reader = new FileReader(DietsGUI
							.getSystemTopologyName());

					File topologyFile = new File(topologyFileName);
					int length = (int) topologyFile.length();
					char[] rbuf = new char[length];
					reader.read(rbuf);
					String ftext = new String(rbuf);
					this.testExecutionPanel.getConfigurePane().setText(ftext);

				}

				this.updateSystemTopologyPane();

			} catch (Exception ex) {
				ex.printStackTrace();
				this.showWarning("Make sure the file is writable!",
						"Edit error");

			} finally {
				this.testExecutionPanel.enableAllButtons();

			}
		} else if (ae.getSource() == this.startupPropertiesMenuItem) {
			char[] rbuf = null;
			try {
				FileReader reader = new FileReader(DietsGUI

				.startupPropertiesFileName);

				File topologyFile = new File(startupPropertiesFileName);
				int length = (int) topologyFile.length();
				rbuf = new char[length];
				reader.read(rbuf);
				reader.close();
			} catch (Exception ex) {
				this.showError("could not read startup props file",
						"File read error");
				return;
			}

			try {

				this.testExecutionPanel.disableAllButtons();

				String fileName = DietsGUI.startupPropertiesFileName;
				File file = new File(fileName);

				ConfigDialog customDialog = new ConfigDialog(this, file,
						"Startup Properties ");
				customDialog.setLocationRelativeTo(this);
				customDialog.setVisible(true);

				/* Dialog returns here. */
				if (customDialog.isSaved()) {

					InputStream fstream = new FileInputStream(new File(
							startupPropertiesFileName));
					Properties properties = new Properties();
					properties.load(fstream);
					fstream.close();

					DietsGUI.doStaticReIniitialization(properties);
					this.dispose();

					javax.swing.SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							try {
								new DietsGUI();
							} catch (Exception ex) {
								logger.error("Exception starting the GUI", ex);
							}
						}
					});

				}

			} catch (Exception ex) {

				try {
					FileWriter fwriter = new FileWriter(new File(
							DietsGUI.startupPropertiesFileName));
					fwriter.write(rbuf);
					fwriter.close();

					InputStream fstream = new FileInputStream(new File(
							startupPropertiesFileName));
					Properties properties = new Properties();
					properties.load(fstream);
					fstream.close();

					DietsGUI.doStaticReIniitialization(properties);
					this.testExecutionPanel.enableAllButtons();

				} catch (Exception ex1) {
					this.showError("Could not restore to old state",
							"Catastrophic error");

					ex1.printStackTrace();
					// System.exit(0);

				}

			} finally {

			}
		}

	}

	/**
	 * zip the current traces into a directory for shipment to the ISSI remote
	 * verifier.
	 */

	public void zipFiles() throws Exception {
		// These are the files to include in the ZIP file
		String[] filenames = new String[] { "testrun",
				testerConfigurationFile , 
				startupPropertiesFileName};

		Zipper.zipAll(filenames,"testrun.zip");
		
		
		
		JOptionPane.showMessageDialog(null, "Zip file of test run completed. Visit http://antd.nist.gov/submit and say your prayers");

	}

	/**
	 * Save the current trace so you can come back and examine it later if you
	 * want.
	 * 
	 * 
	 */
	public static void saveFiles() throws Exception {

		try {
			logger.debug("saveFiles()");
			String directoryName = ISSITesterConstants.DEFAULT_LOGS_DIR;
			File logDir = new File(directoryName);

			String currentTestCaseFile =  new File(logDir.getCanonicalPath()
			+ "/" + ISSITesterConstants.CURRENT_TEST_CASE_XML).toURI().toURL().toString();
			logger.debug("currentTestCaseFileName " + currentTestCaseFile);
			TestRunInfo testRun = new CurrentTestCaseParser().parse(currentTestCaseFile);

			new File(testRun.directory);
			String testCase = testRun.directory;
			boolean interactive = testRun.isInteractive;
			String testSuiteName = new File(testRun.testDirectory)
					.getParentFile().getName();
			String targetDirName = (interactive ? ISSITesterConstants.DEFAULT_TEST_RUN_DIRECTORY
					: ISSITesterConstants.DEFAULT_TRACES_DIR)
					+ "/"
					+ testSuiteName
					+ "/"
					+ testRun.topology.substring(0, testRun.topology
							.indexOf(".xml")) + ".logs";

			logger.debug("Saving files to directory " + targetDirName);
			File targetDir = new File(targetDirName);
			targetDir.mkdirs();
			/* Copy *.ptt , *.xml, *.sip to the target location */

			for (File file : logDir.listFiles(new FilenameFilter() {

				public boolean accept(File file, String name) {
					return name.endsWith("sip") || name.endsWith("ptt")
							|| name.endsWith("xml");
				}
			})) {

				File targetFile = new File(targetDirName + "/" + file.getName());
				if (targetFile.exists())
					targetFile.delete();
				file.renameTo(new File(targetDir, file.getName()));

			}
			/*
			 * this.testExecutionPanel.saveSipJpegTraces(targetDir
			 * .getAbsolutePath() + "/sipmessages.jpg");
			 * this.testExecutionPanel.savePttJpegTraces(targetDir
			 * .getAbsolutePath() + "/pttmessages.jpg");
			 * this.testExecutionPanel.saveSipRtfTraces(targetDir
			 * .getAbsolutePath() + "/sipmessages.rtf");
			 * this.testExecutionPanel.savePttRtfTraces(targetDir
			 * .getAbsolutePath() + "/pttmessages.rtf");
			 */

		} catch (Exception ex) {
			
			logger.error("An unepxected exception occured" , ex);
			ex.printStackTrace();
		}

	}

	private void loadProperties() {
		// File chooser used to open the trace file.
		JFileChooser fileChooser = new JFileChooser(System
				.getProperty("user.dir"));
		fileChooser.setDialogTitle("Open trace directory");
		fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		FileFilter traceFilter = new FileFilter() {

			@Override
			public boolean accept(File pathname) {
				if (pathname.getName().endsWith(".properties")
						|| pathname.isDirectory())
					return true;
				else
					return false;
			}

			@Override
			public String getDescription() {

				return "Startup Properties";
			}

		};
		fileChooser.setFileFilter(traceFilter);

		int returnValue = fileChooser.showOpenDialog(this);

		String savedName = DietsGUI.startupPropertiesFileName;

		try {

			if (returnValue == JFileChooser.APPROVE_OPTION) {

				DietsGUI.startupPropertiesFileName = fileChooser
						.getSelectedFile().getAbsolutePath();

				Properties props = new Properties();
				props.load(new FileInputStream(new File(
						startupPropertiesFileName)));
				DietsGUI.doStaticReIniitialization(props);
				this.dispose();

				javax.swing.SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						try {
							new DietsGUI();
						} catch (Exception ex) {
							logger.error("Exception starting the GUI", ex);
						}
					}
				});

			} else {
				logger.debug("Approve option not chosen -- returning "
						+ returnValue);
				return;

			}
		} catch (Exception ex) {
			this.showError("Error loading startup properties",
					"Reconfiguration Error");
			DietsGUI.startupPropertiesFileName = savedName;
			this.testExecutionPanel.enableAllButtons();
		}

	}

	private void renderTraces(String directoryName, boolean toSave)
			throws Exception {
		StringBuffer sbuf = new StringBuffer();

		File directoryFile = new File(directoryName);
		globalTopologyFileName = directoryFile.getAbsolutePath()
				+ "/globaltopology.xml";

		systemTopologyFileName = directoryFile.getAbsolutePath()
				+ "/systemtopology.xml";

		if (!new File(globalTopologyFileName).exists()) {
			logger.error("Cannot find " + globalTopologyFileName);
			return;
		}
		if (!new File(systemTopologyFileName).exists()) {
			logger.error("Cannot find" + systemTopologyFileName);
			return;
		}
		if (!new File(directoryFile.getAbsolutePath() + "/"
				+ ISSITesterConstants.CURRENT_TEST_CASE_XML).exists()) {

			logger.error("Cannot find " + directoryFile.getAbsolutePath() + "/"
					+ ISSITesterConstants.CURRENT_TEST_CASE_XML);
			return;
		}

		this.updateGlobalTopologyPane();
		this.updateSystemTopologyPane();
		TestRunInfo testRunInfo = new CurrentTestCaseParser()
				.parse(new File (directoryFile.getAbsolutePath() + "/"
						+ ISSITesterConstants.CURRENT_TEST_CASE_XML).toURI().toURL().toString());
		String currentTestDir = testRunInfo.directory;
		String currentTopology = testRunInfo.topology;
		this.testExecutionPanel.setCurrentTest(currentTestDir, currentTopology);

		// Add XML begin and end tags to the selected file since these
		// are not generated by the ISSI emulator

		String c = null;

		sbuf.append("<sipmessages>\n");

		for (File file : directoryFile.listFiles(new FilenameFilter() {

			public boolean accept(File dir, String name) {
				return name.endsWith("sip");
			}
		})) {
			BufferedReader fileReader = new BufferedReader(new FileReader(file));
			while ((c = fileReader.readLine()) != null)
				sbuf.append(c + "\r\n");
			fileReader.close();
		}

		sbuf.append("</sipmessages>\n");
		File pttTrace = new File(directoryName + "/messagelog.ptt");
		if (pttTrace.exists()) {
			BufferedReader breader = new BufferedReader(
					new FileReader(pttTrace));
			StringBuffer tracebuf = new StringBuffer()
					.append("<pttmessages>\n");
			String line = null;
			while ((line = breader.readLine()) != null)
				tracebuf.append(line + "\r\n");
			tracebuf.append("</pttmessages>\n");
			this.testExecutionPanel.renderSipPttTraces(sbuf.toString(),
					tracebuf.toString(), null, testRunInfo.isInteractive,
					toSave);
		} else {
			this.testExecutionPanel.renderSipPttTraces(sbuf.toString(), null,
					null, testRunInfo.isInteractive, toSave);
		}

		getCloseMenuItem().setEnabled(true);
		getCloseAllMenuItem().setEnabled(true);
	}

	/**
	 * Open and render the current saved log information in the ref traces
	 * directory.
	 * 
	 */

	public void renderRefTraceLogs() {
		String testName = this.testExecutionPanel.getCurrentTest()
				.getDirectoryName();
		String topology = this.testExecutionPanel.getCurrentTopologyName()
				.substring(
						0,
						this.testExecutionPanel.getCurrentTopologyName()
								.indexOf(".xml"));

		String dir = ISSITesterConstants.DEFAULT_TRACES_DIR + "/" + testSuite
				+ "/" + topology + ".logs";
		logger.debug("renderRefTraceLogs: " + dir);
		// System.out.println("dir = " + dir);
		if (!new File(dir).exists()) {
			logger.debug("renderRefTraceLogs(): no trace dir found " + dir);
			this.tabbedPane.removeAll();
			return;
		} 
		try {
			this.renderTraces(dir, false);
		} catch (Exception ex) {
			logger
					.debug("Error rendering reference trace -- need to generate one.");
		}

	}

	/**
	 * Open a previously stored trace. The trace directory contains
	 * <ul>
	 * <li> the current test name in a file called current-test-case.xml
	 * <li> the global topology used to run the test in a file called
	 * globaltopology.xml
	 * <li> the sip llogs.
	 * <li> the ptt logs.
	 * </ul>
	 * 
	 * @throws Exception --
	 *             if problems were encountered in opening the log.
	 */
	public void openFile() throws Exception {

		String directoryName = null;

		int returnValue = fileChooser.showOpenDialog(this);

		if (returnValue == JFileChooser.APPROVE_OPTION) {

			directoryName = fileChooser.getSelectedFile().getAbsolutePath();

		} else {
			logger.debug("Approve option not chosen -- returning "
					+ returnValue);
			return;

		}
		this.renderTraces(directoryName, false);

	}

	public void showWarning(String text, String title) {
		JOptionPane.showMessageDialog(this, text, title,
				JOptionPane.WARNING_MESSAGE);
	}

	public void showError(String text, String title) {
		JOptionPane.showMessageDialog(this, text, title,
				JOptionPane.ERROR_MESSAGE);
	}

	public void writeScroll(JTextArea textArea, String message) {

		textArea.append(message);
		int paneHeight = (int) textArea.getBounds().getHeight();
		textArea.scrollRectToVisible(new Rectangle(new Point(0, paneHeight)));

	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {

		JButton button = (JButton) e.getSource();
		button.setBorderPainted(true);

	}

	public void mouseExited(MouseEvent e) {
		JButton button = (JButton) e.getSource();
		button.setBorderPainted(false);

	}

	public void mousePressed(MouseEvent e) {

	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	static void updateGlobalTopologyValues() {
		globalTopologyFileName = ISSITesterConstants.getScenarioDir() + "/"
				+ ISSITesterConstants.GLOBAL_TOPOLOGY_XML;
		systemTopologyFileName = ISSITesterConstants.TESTSUITES_DIR + "/"
				+ testSuite + "/" + ISSITesterConstants.SYSTEM_TOPOLOGY_XML;

	}

	static String getSystemTopologyName() {
		return DietsGUI.systemTopologyFileName;

	}

	static void setIssiHttpServerConfig(
			ISSITesterConfiguration issiHttpServerConfig) {
		DietsGUI.issiTesterConfiguration = issiHttpServerConfig;
	}

	static ISSITesterConfiguration getIssiTesterConfiguration() {
		return issiTesterConfiguration;
	}

	public static void doStaticReIniitialization(Properties props)
			throws Exception {

		testerConfigurationFile = props
				.getProperty(DietsConfigProperties.DAEMON_CONFIG_PROPERTY);

		if (testerConfigurationFile == null) {
			JOptionPane
					.showMessageDialog(
							null,
							"Missing Property! tester configuration file not specified",
							"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw new Exception("Missing a required Parameter "
					+ DietsConfigProperties.DAEMON_CONFIG_PROPERTY);
		}

		testSuite = props.getProperty(DietsConfigProperties.TESTSUITE_PROPERTY,
				"conformance");
		ISSITesterConstants.setTestSuite(testSuite);

		standalone = props
				.getProperty(DietsConfigProperties.STANDALONE_PROPERTY) != null ? props
				.getProperty(DietsConfigProperties.STANDALONE_PROPERTY).equals(
						"true")
				: false;
		evaluatePassFail = props.getProperty("diets.notest") != null ? props
				.getProperty("diets.notest").equals("true") : true;

		DietsGUI.PREFERRED_WIDTH = props
				.getProperty(DietsConfigProperties.DIETSGUI_WIDTH) != null ? Integer
				.parseInt(props
						.getProperty(DietsConfigProperties.DIETSGUI_WIDTH))
				: DietsGUI.PREFERRED_WIDTH;
		DietsGUI.PREFERRED_HEIGHT = props
				.getProperty(DietsConfigProperties.DIETSGUI_HEIGHT) != null ? Integer
				.parseInt(props
						.getProperty(DietsConfigProperties.DIETSGUI_HEIGHT))
				: DietsGUI.PREFERRED_HEIGHT;

		testRegistry = ISSITesterConstants.getScenarioDir() + "/"
				+ ISSITesterConstants.TEST_REGISTRY_XML;

		systemTopologyFileName = ISSITesterConstants.TESTSUITES_DIR + "/"
				+ testSuite + "/" + ISSITesterConstants.SYSTEM_TOPOLOGY_XML;

		DietsGUI.traceSource = props.getProperty("dietsgui.traceSource",
				"conformance-tester");

		updateGlobalTopologyValues();

		try {

			issiTesterConfiguration = new ISSITesterConfigurationParser(
					testerConfigurationFile).parse();
		} catch (Exception ex) {
			logger.error("Error parsing config file ", ex);
			JOptionPane.showMessageDialog(null,
					"Error parsing configuration file "
							+ testerConfigurationFile,
					"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw ex;
		}

		TestRFSS.setTesterConfiguration(issiTesterConfiguration);

		globalTopologyFileName = ISSITesterConstants.getScenarioDir()
				+ "/globaltopology.xml";

		logger.debug("global topology = " + globalTopologyFileName);

		DietsGUI.ipAddresses = issiTesterConfiguration.getLocalAddresses();

		if ((ipAddresses == null || ipAddresses.isEmpty()) && standalone) {
			JOptionPane
					.showMessageDialog(
							null,
							"No local ip addresses could be configured -- cannot run standalone !",
							"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw new Exception("Configuration Exception -- startup");

		}

		SystemTopologyParser systemTopologyParser = new SystemTopologyParser(
				issiTesterConfiguration);

		TopologyConfig systemTopology = null;

		try {
			systemTopology = systemTopologyParser.parse(systemTopologyFileName);
		} catch (Exception ex) {
			logger.error("Error occured while parsing system topology "
					+ systemTopologyFileName);
			JOptionPane
					.showMessageDialog(
							null,
							"Error in system topology "
									+ systemTopologyFileName
									+ "\nException Message is : "
									+ ex.getMessage()
									+ "\nCheck Tester Configuration file "
									+ testerConfigurationFile
									+ "\nCheck System topology file and make sure node identifiers are mapped for the entire file",
							"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw ex;
		}

		try {
			TopologyConfig globalTopology = new GlobalTopologyParser(true)
					.parse(systemTopology, globalTopologyFileName);
		} catch (Exception ex) {
			logger.error(
					"Error occured while parsing test suite global topology "
							+ globalTopologyFileName, ex);
			JOptionPane
					.showMessageDialog(
							null,
							"Error occured while parsing test suite global topology "
									+ globalTopologyFileName
									+ "\nException type is : "
									+ ex.getClass().getName()
									+ "\nException message is :"
									+ ex.getMessage()
									+ "\nMake sure that the file exists, is valid syntax "
									+ " RFSSs referenced in "
									+ globalTopologyFileName
									+ "\nare defined in "
									+ systemTopologyFileName,
							"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw ex;
		}

		try {

			DietsGUI.tests = getAvailableTestCases();

		} catch (Exception e) {

			logger.error("Unexpected error getting test cases", e);
			JOptionPane.showMessageDialog(null,
					"Error in getting test cases - check test suite",
					"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw e;

		}

		if (DietsGUI.issiTesterConfiguration.getEmulatorConfigurations().size() == 0
				&& !DietsGUI.standalone) {

			JOptionPane
					.showMessageDialog(
							null,
							"Error in configuration file - no web servers were defined so must specify standalone",
							"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			throw new Exception(
					"error in configuration file - should specify standalone flag");
		}

	}

	public static void main(String[] args) throws Exception {

		Properties props = new Properties();
		String fileName = null;
		SplashScreen screen = new SplashScreen("gfx/antd-itl-logo-2004-04.jpg",
				new JFrame(), 1000);

		for (int i = 0; i < args.length; i++) {
			/*******************************************************************
			 * if (args[i].equals("-globalTopology")) { globalTopology =
			 * ISSITesterConstants.DEFAULT_SCENARIO_DIR + "/" + args[++i]; }
			 * else
			 ******************************************************************/

			if (args[i].equals("-startup")) {
				fileName = args[++i];
				DietsGUI.startupPropertiesFileName = fileName;

				InputStream inStream = new FileInputStream(new File(fileName));

				props.load(inStream);
				inStream.close();
			} else {
				JOptionPane.showMessageDialog(null, "invalid argument "
						+ args[i], "DIETSGUI startup error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
		}

		if (DietsGUI.startupPropertiesFileName == null) {
			JOptionPane.showMessageDialog(null, "missing startup parameter ",
					"DIETSGUI startup error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		DietsGUI.doStaticReIniitialization(props);

		if (!standalone) {
			Logger logger = Logger.getLogger("gov.nist.p25.issi.testlauncher");
			logger.addAppender(new FileAppender(new SimpleLayout(),
					"logs/guidebuglog.txt"));
		}

		// Schedule a job for the event-dispatching thread
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					DietsGUI gui = new DietsGUI();

					gui.addWindowListener(new WindowAdapter() {

						public void windowClosing(WindowEvent e) {

							System.exit(0);

						}
					});
				} catch (Exception ex) {
					logger.error("Exception starting the GUI", ex);
				}
			}
		});

	}

	public static void setTraceSource(String traceSource) {
		DietsGUI.traceSource = traceSource;
	}

	public static String getTraceSource() {
		return traceSource;
	}

}