package gov.nist.p25.issi.testlauncher;

import gov.nist.p25.issi.issiconfig.GlobalTopologyParser;
import gov.nist.p25.issi.issiconfig.RfssConfig;
import gov.nist.p25.issi.issiconfig.SuConfig;
import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;
import gov.nist.p25.issi.issiconfig.TopologyConfigParser;
import gov.nist.p25.issi.rfss.ISSITesterConstants;
import gov.nist.p25.issi.rfss.tester.TestScript;
import gov.nist.p25.issi.rfss.tester.TestScriptParser;
import gov.nist.p25.issi.traceviewer.MessageData;
import gov.nist.p25.issi.traceviewer.PttSessionInfo;
import gov.nist.p25.issi.traceviewer.PttTraceLoader;
import gov.nist.p25.issi.traceviewer.RfssData;
import gov.nist.p25.issi.traceviewer.RfssRuntimeDataParser;
import gov.nist.p25.issi.traceviewer.SipMessageData;
import gov.nist.p25.issi.traceviewer.SipTraceLoader;
import gov.nist.p25.issi.traceviewer.TraceFileFilter;
import gov.nist.p25.issi.traceviewer.TracePanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import org.apache.log4j.Logger;
import org.xml.sax.InputSource;

/**
 * Test execution panel.
 * 
 * @author mranga@nist.gov
 * @version $Revision: 1.25 $, $Date: 2007/12/22 13:57:34 $
 * @since 1.5
 */
public class TestExecutionPanel extends JPanel implements MouseListener,
		ActionListener {

	public static SimpleAttributeSet ERROR_MESSAGE_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet STACK_TRACE_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet INFO_TEXT = new SimpleAttributeSet();

	public static SimpleAttributeSet FATAL_MESSAGE_TEXT = new SimpleAttributeSet();

	private DietsGUI gui = null;

	private JList jlist = null;

	private JButton runButton = null;

	private JButton runInteractiveButton = null;

	// private JButton viewButton = null;

	private String currentTestName = "";

	private TestExecutionTimerTask testExecutionTimerTask = null;

	/** The timer for TimerTasks associated with this receiver. */
	static Timer timer = new Timer();

	private static Logger logger = Logger.getLogger(TestExecutionPanel.class);

	private ProgressMonitor progressMonitor = null;

	private String progressScreenMessage = "";

	private JTextPane descriptionPane = null;

	private JTextPane scriptPane = null;

	private JTextPane configurePane = null;

	private JTextPane sipTraceLogPane = null;

	private JTextPane pttTraceLogPane = null;

	private JTextPane globalTopologyPane = null;

	private ConfigDialog customDialog;

	private String currentTopologyName;

	private JTextPane statusLogPane;

	private String sipRtfTrace;

	private String pttRtfTrace;

	private TracePanel pttTracePanel;

	private TracePanel sipTracePanel;

	private JTabbedPane tabbedPane;

	private JScrollPane statusLogScrollPane;

	private String localTopologyName;

	private JTextPane systemTopologyPane;

	static {

		StyleConstants.setForeground(ERROR_MESSAGE_TEXT, Color.red);
		StyleConstants.setForeground(STACK_TRACE_TEXT, Color.blue);
		StyleConstants.setForeground(INFO_TEXT, Color.black);
		StyleConstants.setForeground(FATAL_MESSAGE_TEXT, Color.red);
		StyleConstants.setUnderline(FATAL_MESSAGE_TEXT, true);
		StyleConstants.setFontSize(FATAL_MESSAGE_TEXT, 14);

	}

	/**
	 * A class that waits for test execution to complete. This is basically an
	 * open loop timer. It is used in non-interactive execution.
	 * 
	 */
	class TestExecutionTimerTask extends TimerTask {

		int secondsRemaining;

		DietsGUI gui;

		boolean interactive;

		public TestExecutionTimerTask(DietsGUI gui, int seconds,
				boolean interactive) {
			gui.getStatusProgressBar().setIndeterminate(true);
			this.gui = gui;
			setProgressMonitor(new ProgressMonitor(gui));
			this.secondsRemaining = seconds;
			this.interactive = interactive;

		}

		public void run() {

			secondsRemaining = secondsRemaining - 1000;
			setProgressScreenMessage("Running.  Time remaining: "
					+ (secondsRemaining / 1000) + " seconds");

			if (secondsRemaining == 0) {
				this.cancel();
				if (!interactive)
					getSipPttTraces(interactive);
			}

		}

	}

	private String formatLocalTopology() {
		try {
			String topologyFileName = ISSITesterConstants.getScenarioDir()
					+ "/" + this.currentTopologyName;

			GlobalTopologyParser globalTopologyParser = new GlobalTopologyParser(
					true);
			TopologyConfig systemTopology = new SystemTopologyParser(DietsGUI
					.getIssiTesterConfiguration()).parse(DietsGUI
					.getSystemTopologyName());
			TopologyConfig globalTopology = globalTopologyParser.parse(
					systemTopology, DietsGUI.getGlobalTopologyName());
			TopologyConfigParser parser = new TopologyConfigParser();
			TopologyConfig testTopology = parser.parse(globalTopology,
					topologyFileName);
			StringBuffer sbuf = new StringBuffer();
			sbuf.append("TEST CONFIGURATIONS FOR RFSS AND SUs");
			for (RfssConfig rfssConfig : testTopology.getRfssConfigurations()) {
				sbuf.append("\n\n");
				sbuf.append("-----------RFSS Configuration ----------\n");
				sbuf.append("rfssName = " + rfssConfig.getRfssName() + "\n");
				sbuf.append("rfssId = " + rfssConfig.getRfssId() + "\n");
				sbuf.append("wacn Id = " + rfssConfig.getWacnId() + "\n");
				sbuf.append("system Id = " + rfssConfig.getSystemId() + "\n");
				sbuf.append("rfssIpAddress = " + rfssConfig.getIpAddress()
						+ "\n");
				sbuf.append("Sip Signaling Port = " + rfssConfig.getSipPort()
						+ "\n");
				sbuf.append("isEmulated = " + rfssConfig.isEmulated() + "\n");
				
				if (rfssConfig.isEmulated()) {
					sbuf
							.append("---- Emulator RFSS SETTINGS SPECIFIC TO THIS TEST (-1 means UNBOUNDED)---\n");
					sbuf.append("isRfResourcesAvailable = "
							+ rfssConfig.isRfResourcesAvailable() + "\n");
					sbuf.append("numberOfRtpPortsAvailable = "
							+ rfssConfig.getMaxRtpPorts() + "\n");
					sbuf.append("servedGroupLeaseTime (sec) = "
							+ rfssConfig.getServedGroupLeaseTime() + "\n");
					sbuf.append("groupRegistrationExpiresTime (sec) = "
							+ rfssConfig.getGroupRegistrationExpiresTime()
							+ "\n");
					sbuf.append("registerConfirmTime (sec) = "
							+ rfssConfig.getRegisterConfirmTime() + "\n");

					sbuf.append("groupCallInviteProcessingTime ( sec ) = "
							+ rfssConfig.getGroupCallInviteProcessingTime()
							+ "\n");
					sbuf.append("isAdvancedRtpResourceManagementSupported = " + rfssConfig.isAdvancedRtpResourceManagementSupported() + "\n");
					sbuf.append("isSuCredentialsQueriedBeforeRegister = " + rfssConfig.isSuCredentialsQueriedBeforeRegister() + "\n");
					sbuf.append("isPttRequestGranted = " + rfssConfig.isPttRequestGranted() + "\n");
					sbuf.append("isCallingSuInitialTransmitter = " + rfssConfig.isCallingSuInitialTransmitter()  + "\n");
					sbuf.append("isNotInterestedInLosingAudio = " + rfssConfig.isNotInterestedInLosingAudio() + "\n");
				

				}
				sbuf
						.append("----------------------------------------------------\n");

			}
			for (SuConfig suConfig : testTopology.getSuConfigurations()) {
				sbuf.append("\n\n");
				
				sbuf
						.append("-------------------- SU Configuration --------------\n");
				sbuf.append("Symbolic Name = " + suConfig.getSuName() + "\n");
				sbuf.append("Unit ID = %x"
						+ Integer.toHexString(suConfig.getSuId()) + "\n");
				sbuf.append("Home Rfss = "
						+ suConfig.getHomeRfss().getRfssName() + "\n");
				sbuf
						.append("Initial Serving RFSS = "
								+ suConfig.getInitialServingRfss()
										.getRfssName() + "\n");
				sbuf.append("Wacn Id = " + suConfig.getWacnId());
				sbuf.append("System id = " + suConfig.getSystemId() + "\n");
				sbuf.append("isEmulated = " + suConfig.isEmulated() + "\n");
				sbuf.append("ServiceProfile = [\n");
				sbuf.append(suConfig.getUserServiceProfile().toString() + "]\n");
				if (suConfig.isEmulated()) {
					sbuf
							.append("-------- Emulator SU Settings ( -1 means UNBOUNDED): ---\n");
					sbuf.append("Force Register = " + suConfig.forceRegister()
							+ "\n");
					sbuf.append("home RFSS registers me for "
							+ suConfig.getHomeRfssRegistersMeFor()
							+ " seconds (0 means not registered at home)\n");
					sbuf.append("home Rfss knows about me for "
							+ suConfig.getHomeRfssKnowsAboutMeFor()
							+ " seconds (0 means not known to home)\n");
					sbuf.append("serving Rfss references me for "
							+ suConfig.getServingRfssReferencesMeFor()
							+ " seconds (0 means not registered at serving)\n");
					sbuf.append("Invite processing delay (user response time) "
							+ suConfig.getInviteProcessingDelay()
							+ " seconds (>0 will generate provisional response)\n");
					sbuf.append("isPresenceAssertedOnRegister = " + suConfig.isCheckSuPresenceOnRegister() + "\n");
					

				}
				sbuf
						.append("---------------------------------------------------\n");

			}

			for (gov.nist.p25.issi.issiconfig.GroupConfig gc : testTopology
					.getGroupConfigurations()) {
				sbuf.append("\n\n");
				sbuf.append("------------- GROUP Configuration ---------\n");
				sbuf.append("Symbolic name = " + gc.getGroupName() + "\n");
				sbuf.append("Group ID = %x"
						+ Integer.toHexString(gc.getGroupId()) + "\n");
				sbuf.append("Home Rfss Name = "
						+ gc.getHomeRfss().getRfssName() + "\n");
				sbuf.append("System id = " + gc.getSysConfig().getSystemId()
						+ "\n");
				sbuf
						.append("Wacn Id = " + gc.getSysConfig().getWacnId()
								+ "\n");
				sbuf.append(gc.getGroupServiceProfile().toString());
				sbuf.append("Subsciber Units in this Group = [");
				for (Iterator<SuConfig> it = gc.getSubscribers(); it.hasNext();) {
					sbuf.append(it.next().getSuName());
					if (it.hasNext())
						sbuf.append(", ");
				}
				sbuf.append(" ]\n");

			}

			return sbuf.toString();

		} catch (Exception ex) {
			logger.error("Could not parse topology file -- check it ! ", ex);
			return "";
		}

	}

	public TestExecutionPanel(Vector<TestCaseDescriptor> tests, DietsGUI gui) {

		super();
		this.gui = gui;

		this.setLayout(new BorderLayout());
		this.setBorder(new TitledBorder("Conformance Tests"));

		JPanel listPanel = new JPanel();
		listPanel.setMinimumSize(new Dimension(325, 300));
		listPanel.setPreferredSize(new Dimension(400, 300));
		listPanel.setMaximumSize(new Dimension(400, 300));
		listPanel.setLayout(new BorderLayout());
		jlist = new JList(tests);
		jlist.addMouseListener(this);
		jlist.setSelectedIndex(0);
		JScrollPane scrollList = new JScrollPane(jlist);

		this.runButton = new JButton("Generate Trace");
		this.runButton.setToolTipText("Generate Reference Trace");
		this.runButton.addActionListener(this);

		this.runInteractiveButton = new JButton("Run Test");
		this.runInteractiveButton
				.setToolTipText("Run in interactive mode - simulates actual operation");
		this.runInteractiveButton.addActionListener(this);

		/*
		 * this.configureButton = new JButton("Edit Topology");
		 * this.configureButton.setToolTipText("Edit Global Topology");
		 * this.configureButton.addActionListener(this);
		 */

		JPanel buttonPanel = new JPanel(new GridLayout(1, 2));
		buttonPanel.add(runButton);
		buttonPanel.add(runInteractiveButton);
		// buttonPanel.add(configureButton);

		listPanel.add(scrollList, BorderLayout.CENTER);
		listPanel.add(buttonPanel, BorderLayout.SOUTH);

		// Tabbed pane components
		descriptionPane = new JTextPane();
		descriptionPane.setEditable(false);
		descriptionPane.setAutoscrolls(true);
		TestCaseDescriptor descriptor = (TestCaseDescriptor) jlist.getModel()
				.getElementAt(0);
		descriptionPane.setText("Test Number : " + descriptor.getTestNumber()
				+ "\n\n" + descriptor.getTestDescription());
		JScrollPane descriptionScrollPane = new JScrollPane(descriptionPane);

		scriptPane = new JTextPane();
		scriptPane.setEditable(false);
		JScrollPane scriptScrollPane = new JScrollPane(scriptPane);
		scriptPane.setAutoscrolls(true);

		try {
			TestCaseDescriptor selectedTest = (TestCaseDescriptor) jlist
					.getSelectedValue();

			this.currentTestName = (String) selectedTest.getFileName();
			this.currentTopologyName = selectedTest.getTopologyFileName();

			File testFile = new File(ISSITesterConstants.getScenarioDir() + "/"
					+ currentTestName);
			FileReader reader = new FileReader(testFile);
			int length = (int) testFile.length();
			char[] rbuf = new char[length];
			reader.read(rbuf);
			String ftext = new String(rbuf);
			scriptPane.setText(ftext.trim());
			scriptPane.setCaretPosition(0);

		} catch (Exception ex) {

			ex.printStackTrace();

		}

		configurePane = new JTextPane();
		configurePane.setEditable(false);
		JScrollPane configureScrollPane = new JScrollPane(configurePane);
		configurePane.setAutoscrolls(true);

		try {
			String localTopology = this.formatLocalTopology();
			configurePane.setText(localTopology.trim());
			configurePane.setCaretPosition(0);

		} catch (Exception e) {

			e.printStackTrace();

		}

		sipTraceLogPane = new JTextPane();
		sipTraceLogPane.setEditable(false);
		JScrollPane sipLogScrollPane = new JScrollPane(sipTraceLogPane);
		sipTraceLogPane.setAutoscrolls(true);

		sipTraceLogPane.setToolTipText("Right Click to Save ");

		sipTraceLogPane.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent me) {
				// TODO Auto-generated method stub

				if (me.getButton() == 3) {

					JFileChooser fileChooser = new JFileChooser();
					fileChooser.setDialogTitle("Save As");
					TraceFileFilter traceFilter = new TraceFileFilter();
					traceFilter.addExtension("txt");
					traceFilter.addExtension("rtf");
					traceFilter.setDescription("Text or RTF Files");
					fileChooser.setFileFilter(traceFilter);

					int returnVal = fileChooser
							.showSaveDialog(TestExecutionPanel.this.gui);

					File file = null;
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						logger.debug("Saving: " + file.getName());
					} else {
						logger.debug("Save command cancelled by user.");
						return;
					}
					logger.debug("path for save: " + file.getAbsolutePath());
					String message = (traceFilter.getExtension(file)
							.endsWith("txt")) ? sipTraceLogPane.getText()
							: sipRtfTrace;

					if (message == null || message.trim().equals("")) {
						JOptionPane.showMessageDialog(null, "No data to save",
								"Save Trace Info",
								JOptionPane.INFORMATION_MESSAGE);
						return;
					}

					try {

						OutputStream out = new FileOutputStream(new File(file
								.getAbsolutePath()));
						out.write(message.getBytes());
						out.close();
					} catch (Exception e) {
						logger
								.error(
										"An exception occured while trying to save file",
										e);
						JOptionPane.showMessageDialog(null,
								"An Error Occured while saving the file",
								"Save File Status", JOptionPane.ERROR_MESSAGE);
					}

				}
			}

			public void mouseEntered(MouseEvent e) {

			}

			public void mouseExited(MouseEvent e) {

			}

			public void mousePressed(MouseEvent e) {

			}

			public void mouseReleased(MouseEvent e) {

			}
		});

		pttTraceLogPane = new JTextPane();
		pttTraceLogPane.setEditable(false);
		JScrollPane pttLogScrollPane = new JScrollPane(pttTraceLogPane);
		pttTraceLogPane.setAutoscrolls(true);
		pttTraceLogPane.setToolTipText("Right Click to Save ");

		globalTopologyPane = new JTextPane();
		globalTopologyPane.setEditable(false);
		JScrollPane globalTopologyScrollPane = new JScrollPane(
				globalTopologyPane);
		globalTopologyPane.setAutoscrolls(true);

		this.systemTopologyPane = new JTextPane();
		systemTopologyPane.setEditable(false);
		JScrollPane systemTopologyScrollPane = new JScrollPane(
				systemTopologyPane);
		systemTopologyPane.setAutoscrolls(true);

		pttTraceLogPane.addMouseListener(new MouseListener() {

			public void mouseClicked(MouseEvent me) {
				// TODO Auto-generated method stub

				if (me.getButton() == 3) {

					JFileChooser fileChooser = new JFileChooser();
					fileChooser.setDialogTitle("Save As");
					TraceFileFilter traceFilter = new TraceFileFilter();
					traceFilter.addExtension("txt");
					traceFilter.addExtension("rtf");
					traceFilter.setDescription("Text or RTF Files");
					fileChooser.setFileFilter(traceFilter);
					File file = null;
					int returnVal = fileChooser
							.showSaveDialog(TestExecutionPanel.this.gui);

					if (returnVal == JFileChooser.APPROVE_OPTION) {
						file = fileChooser.getSelectedFile();
						logger.debug("Saving: " + file.getName());
					} else {
						logger.debug("Save command cancelled by user.");
						return;
					}
					logger.debug("path for save: " + file.getAbsolutePath());
					String message = traceFilter.getExtension(file).endsWith(
							"txt") ? pttTraceLogPane.getText() : pttRtfTrace;

					if (message == null || message.trim().equals("")) {
						JOptionPane.showMessageDialog(null, "No data to save",
								"Save Trace Info",
								JOptionPane.INFORMATION_MESSAGE);
						return;
					}

					try {

						OutputStream out = new FileOutputStream(new File(file
								.getAbsolutePath()
								+ ".txt"));
						out.write(message.getBytes());
						out.close();
					} catch (Exception e) {
						logger
								.error(
										"An exception occured while trying to save file",
										e);
						JOptionPane.showMessageDialog(null,
								"An Error Occured while saving the file",
								"Save File Status", JOptionPane.ERROR_MESSAGE);
					}

				}
			}

			public void mouseEntered(MouseEvent e) {

			}

			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mousePressed(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub

			}
		});

		this.statusLogPane = new JTextPane();
		statusLogPane.setEditable(false);
		this.statusLogScrollPane = new JScrollPane(statusLogPane);
		statusLogPane.setAutoscrolls(true);

		this.tabbedPane = new JTabbedPane();
		tabbedPane.add("Description", descriptionScrollPane);
		tabbedPane.setSelectedComponent(descriptionScrollPane);
		tabbedPane.add("System Topology", systemTopologyScrollPane);
		tabbedPane.add("Global Topology", globalTopologyScrollPane);

		tabbedPane.add("Test Configuration", configureScrollPane);
		tabbedPane.add("Test Script", scriptScrollPane);
		tabbedPane.add("SIP Trace Log", sipLogScrollPane);
		tabbedPane.add("PTT Trace Log", pttLogScrollPane);
		tabbedPane.add("Error Diagnostic", statusLogScrollPane);

		sipLogScrollPane.addMouseListener(this);
		pttLogScrollPane.addMouseListener(this);

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				listPanel, tabbedPane);
		this.add(splitPane, BorderLayout.CENTER);

	}

	public void logError(String failureMessage, String errorTrace) {

		try {

			Document doc = statusLogPane.getDocument();
			doc.remove(0, doc.getLength());

			doc.insertString(doc.getLength(), failureMessage,
					ERROR_MESSAGE_TEXT);
			doc.insertString(doc.getLength(), errorTrace, ERROR_MESSAGE_TEXT);

			statusLogPane.setCaretPosition(doc.getLength());
			this.tabbedPane.setSelectedComponent(this.statusLogScrollPane);

		} catch (Exception ex) {
			logger.error("Unexpected exception", ex);
		}

	}

	public void logError(String failureMessage) {

		try {
			Document doc = statusLogPane.getDocument();
			doc.remove(0, doc.getLength());

			doc.insertString(doc.getLength(), failureMessage + "\n",
					ERROR_MESSAGE_TEXT);

			statusLogPane.setCaretPosition(doc.getLength());
			this.tabbedPane.setSelectedComponent(statusLogScrollPane);
		} catch (Exception ex) {
			logger.error("Unexpected exception", ex);
		}

	}

	public void logFatal(String failureMessage, String stackTrace) {

		try {
			Document doc = statusLogPane.getDocument();
			doc.remove(0, doc.getLength());

			doc.insertString(doc.getLength(), failureMessage + "\n",
					FATAL_MESSAGE_TEXT);
			doc.insertString(doc.getLength(), stackTrace, FATAL_MESSAGE_TEXT);

			statusLogPane.setCaretPosition(doc.getLength());
			this.tabbedPane.setSelectedComponent(statusLogScrollPane);
		} catch (Exception ex) {
			logger.error("Unexpected exception", ex);
		}

	}

	public void logStatusMessage(String statusMessage) {

		try {
			Document doc = statusLogPane.getDocument();

			doc.insertString(doc.getLength(), statusMessage + "\n", INFO_TEXT);

			statusLogPane.setCaretPosition(doc.getLength());
		} catch (Exception ex) {
			ex.printStackTrace();
		}

	}

	public void actionPerformed(ActionEvent actionEvent) {

		DietsGUI.updateGlobalTopologyValues();

		Object source = actionEvent.getSource();

		if (source == this.runButton) {

			disableAllButtons();

			try {
				TestCaseDescriptor selectedTest = (TestCaseDescriptor) jlist
						.getSelectedValue();
				this.currentTestName = (String) selectedTest.getFileName();
				this.localTopologyName = selectedTest.getTopologyFileName();
				logger.debug("globalTopology = "
						+ DietsGUI.getGlobalTopologyName());
				// public TestScriptParser(String fileName, String
				// topologyFileName , String globalTopology, String
				// systemTopology, boolean interactive) throws Exception {

				TestScriptParser parser = new TestScriptParser(
						ISSITesterConstants.getScenarioDir() + "/"
								+ currentTestName, ISSITesterConstants
								.getScenarioDir()
								+ "/" + localTopologyName, DietsGUI
								.getGlobalTopologyName(), DietsGUI
								.getSystemTopologyName(), false);
				TestScript testScript = parser.parse();
				this.gui.getRemoteController().loadTest(
						ISSITesterConstants.getScenarioDir(), currentTestName,
						selectedTest.getTestNumber(),
						localTopologyName, DietsGUI.getGlobalTopologyName(),
						DietsGUI.getSystemTopologyName(), false,
						DietsGUI.evaluatePassFail);
				Thread.sleep(1000);
				this.gui.getRemoteController().runTest();

				this.startTestExecutionTimerTask(
						ISSITesterConstants.TEST_RUNS_FOR + 5 * 1000, false);

			} catch (Exception e) {
				e.printStackTrace();
				JOptionPane
						.showMessageDialog(
								null,
								"Error occured in generating trace. \nCheck the configuration file for the tester",
								"Communication Error",
								JOptionPane.ERROR_MESSAGE);
				logger.error("Error communicating with the tester!!", e);
				this.enableAllButtons();

			}

		} else if (source == this.runInteractiveButton) {

			try {

				disableAllButtons();

				TestCaseDescriptor selectedTest = (TestCaseDescriptor) jlist
						.getSelectedValue();
				this.currentTestName = (String) selectedTest.getFileName();
				this.localTopologyName = selectedTest.getTopologyFileName();
				this.gui.getRemoteController().loadTest(
						ISSITesterConstants.getScenarioDir(), currentTestName,
						selectedTest.getTestNumber(),
						this.localTopologyName,
						DietsGUI.getGlobalTopologyName(),
						DietsGUI.getSystemTopologyName(), true,
						DietsGUI.evaluatePassFail);
				Thread.sleep(1000);
				this.gui.getRemoteController().runTest();

				TestScript testScript = new TestScriptParser(
						ISSITesterConstants.getScenarioDir() + "/"
								+ currentTestName, ISSITesterConstants
								.getScenarioDir()
								+ "/" + this.localTopologyName, DietsGUI
								.getGlobalTopologyName(), DietsGUI
								.getSystemTopologyName(), true).parse();
				new ScenarioPrompter(this.gui, this.gui.getRemoteController(),
						testScript).startPrompting();

				// Wait TEST_TIMEOUT before trying to retrieve traces.
				// testExecutionTimerTask = new TestExecutionTimerTask(gui);
				// timer.schedule(testExecutionTimerTask, 0, 1000);
			} catch (Exception ex) {
				this.gui
						.showWarning(
								"Error Communicaticating with the tester or running the test!\n"
										+ "Exception message : "
										+ ex.getMessage()
										+ "\n"
										+ "Make sure test daemons are running (check tester configuration)\n"
										+ "Make sure reference traces are generated before running the test!",
								"Test Execution Error!");
				logger.error("Communication error running test! ", ex);
				this.enableAllButtons();
			}

		}

	}

	public void mouseClicked(MouseEvent me) {

	}

	public void mouseEntered(MouseEvent me) {

	}

	public void mousePressed(MouseEvent me) {

		int index = jlist.locationToIndex(me.getPoint());
		jlist.setSelectedIndex(index);

		TestCaseDescriptor descriptor = (TestCaseDescriptor) jlist.getModel()
				.getElementAt(index);

		descriptionPane.setText("Category : " + descriptor.getCategory()
				+ "\nTest Number : " + descriptor.getTestNumber() + "\n\n"
				+ descriptor.getTestDescription());

		try {
			TestCaseDescriptor selectedTest = (TestCaseDescriptor) jlist
					.getSelectedValue();
			this.currentTestName = (String) selectedTest.getFileName();
			this.currentTopologyName = selectedTest.getTopologyFileName();
			File testFile = new File(ISSITesterConstants.getScenarioDir() + "/"
					+ currentTestName);

			FileReader reader = new FileReader(testFile);

			int length = (int) testFile.length();
			char[] rbuf = new char[length];
			reader.read(rbuf);
			String ftext = new String(rbuf);
			scriptPane.setText(ftext);
			scriptPane.setCaretPosition(0);
			reader.close(); // to avoid memory leaks.
			this.gui.renderRefTraceLogs();

		} catch (Exception ex) {
			logger.error("An unexpected execution error occured", ex);
			return;
		}

		try {

			String topologyFileName = ISSITesterConstants.getScenarioDir()
					+ "/" + this.currentTopologyName;
			FileReader reader = new FileReader(topologyFileName);
			String ftext = this.formatLocalTopology();
			configurePane.setText(ftext);
			configurePane.setCaretPosition(0);
			reader.close();
		} catch (Exception e) {

			logger.error("An unexpected execution error occured", e);
		}
	}

	public void mouseReleased(MouseEvent me) {

	}

	public void mouseExited(MouseEvent me) {

	}

	public void disableAllButtons() {

		this.runButton.setEnabled(false);
		this.runInteractiveButton.setEnabled(false);

	}

	public void enableAllButtons() {
		this.runButton.setEnabled(true);
		this.runInteractiveButton.setEnabled(true);
	}

	/**
	 * Get the PTT mmf/smf allocation information from the Emulated RFSS.
	 * 
	 * @return
	 * @throws Exception
	 */
	public Hashtable<String, HashSet<PttSessionInfo>> getPttSessionInfo() {
		Hashtable<String, HashSet<PttSessionInfo>> retval = null;

		try {

			StringBuffer toParse = new StringBuffer()
					.append("<?xml version=\"1.0\" ?>\n")
					.append(
							"<!DOCTYPE issi-runtime-data SYSTEM \"http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/issi-runtime-data.dtd\">\n")
					.append("<issi-runtime-data>\n");
			toParse.append(gui.getRemoteController().getRfssPttSessionInfo());

			toParse.append("\n</issi-runtime-data>");
			InputSource inputSource = new InputSource(new ByteArrayInputStream(
					toParse.toString().getBytes()));
			RfssRuntimeDataParser parser = new RfssRuntimeDataParser(
					inputSource);

			retval = parser.parse();
		} catch (Exception e) {
			logger.fatal("An unexpected parse exception occured", e);

		}
		return retval;
	}

	public void getSipPttTraces(boolean isInteractive) {
		String sipTraces = null;
		this.setProgressScreenMessage("Getting SIP traces...");
		StringBuffer sbuf = new StringBuffer();
		sbuf.append("<messages>\n");

		if (isInteractive) {
			try {
				if (DietsGUI.getTraceSource().equals("packet-monitor")) {
					sbuf.append(gui.getRemoteController().getSipTraces());
				} else {
					sbuf.append(gui.getRemoteController().getStackSipLogs());
				}
			} catch (Exception e) {
				logger.error("trace fetch error", e);
				gui.showError(
						"could not get sip traces - unexpected exception "
								+ e.getMessage(), "Trace fetch error");
				return;

			}

		} else {
			try {
				sbuf.append(gui.getRemoteController().getStackSipLog());
			} catch (Exception e) {
				logger.error(
						"could not get sip traces - unexpected exception ", e);
				gui.showError(
						"could not get sip traces - unexpected exception "
								+ e.getMessage(), "Trace fetch error");
				return;

			}
		}
		sbuf.append("</messages>");
		sipTraces = sbuf.toString();

		logger.debug("retrieved Sip traces " + sipTraces);

		String pttTraces = null;
		sbuf = new StringBuffer();
		sbuf.append("<pttmessages>\n");

		if (isInteractive) {
			try {
				if (DietsGUI.getTraceSource().equals("packet-monitor")) {
					sbuf.append(this.gui.getRemoteController().getPttTraces());
				} else {
					sbuf.append(this.gui.getRemoteController()
							.getStackPttLogs());
				}
			} catch (Exception e) {
				logger.error("trace fetch error", e);
				gui.showError("could not get ptt traces " + e.getMessage(),
						"Trace fetch error");
				return;

			}
		} else {
			try {
				sbuf.append(this.gui.getRemoteController().getStackPttLog());
			} catch (Exception ex) {
				logger.error(
						"could not get Ptt traces -- unexpected exception", ex);
				gui.showError(
						"could not get sip traces - unexpected exception "
								+ ex.getMessage(), "Trace fetch error");
				return;
			}
		}
		sbuf.append("</pttmessages>\n");
		pttTraces = sbuf.toString();

		Hashtable<String, HashSet<PttSessionInfo>> rfssSessionData = getPttSessionInfo();

		this.renderSipPttTraces(sipTraces, pttTraces, rfssSessionData,
				isInteractive,!isInteractive);

		this.gui.resetRemoteController();

		// logger.debug("Here are the SIP traces = " +sipTraces );

	}

	public void renderSipPttTraces(String sipTraces, String pttTraces,
			Hashtable<String, HashSet<PttSessionInfo>> rfssSessionData,
			boolean isInteractive, boolean toSave) {
		logger.debug("renderSipPttTraces: isInteractive " + isInteractive + " toSave " + toSave);
		boolean remoteRetrievalSucceeded = true;

		// First cancel the polling task
		if (testExecutionTimerTask != null)
			testExecutionTimerTask.cancel();

		Collection<RfssData> rfssList = null;
		try {
			try {

				Hashtable<String, Color> colorMap = new Hashtable<String, Color>();

				byte[] stringAsBytes = sipTraces.getBytes();
				ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
						stringAsBytes);
				TopologyConfig systemTopology = new SystemTopologyParser(
						DietsGUI.getIssiTesterConfiguration()).parse(DietsGUI
						.getSystemTopologyName());
				TopologyConfig globalTopology = new GlobalTopologyParser(
						isInteractive).parse(systemTopology, DietsGUI
						.getGlobalTopologyName());
				TopologyConfig topologyConfig = new TopologyConfigParser()
						.parse(globalTopology, ISSITesterConstants
								.getScenarioDir()
								+ "/" + this.currentTopologyName);
				SipTraceLoader traceLoader = new SipTraceLoader(
						byteArrayInputStream, rfssSessionData, colorMap,
						topologyConfig);

				rfssList = traceLoader.getSortedRfssList();

				ArrayList<MessageData> records = traceLoader.getRecords();

				int currentLength = sipTraceLogPane.getDocument().getLength();
				// Clear up the existing stuff.
				sipTraceLogPane.getDocument().remove(0, currentLength);

				StringBuffer sbuf = new StringBuffer();
				for (MessageData messageData : records) {
					sbuf.append("F" + messageData.getId() + ":\n");
					sbuf.append(messageData.getData().trim() + "\n\n");
				}

				/*
				 * Generate the RTF traces for SIP.
				 */

				File rtfHeaderFile = new File("doc/rtf-header.txt");
				int length = (int) rtfHeaderFile.length();
				char[] headerbytes = new char[length];
				FileReader fileReader = new FileReader(rtfHeaderFile);
				fileReader.read(headerbytes);
				fileReader.close();
				StringBuffer rtfBuffer = new StringBuffer().append(new String(
						headerbytes));
				for (MessageData messageData : records) {
					String rtf = messageData.getRtfText();
					rtfBuffer.append(rtf);
				}
				rtfBuffer.append("}");
				sipTraceLogPane.getDocument().insertString(
						sipTraceLogPane.getDocument().getLength(),
						sbuf.toString(), INFO_TEXT);
				this.sipRtfTrace = rtfBuffer.toString();

				// Create a new tabbed panel
				this.sipTracePanel = new TracePanel(gui, gui.tabbedPane,
						currentTestName, rfssList, records, colorMap);

				// close existing tabs.
				gui.tabbedPane.removeAll();
				gui.tabbedPane.add("SIP", sipTracePanel);
				gui.tabbedPane.setSelectedComponent(sipTracePanel);
				gui.getCloseMenuItem().setEnabled(true);
				gui.getCloseAllMenuItem().setEnabled(true);

			} catch (Exception ex) {

				ex.printStackTrace();
				remoteRetrievalSucceeded = false;

				this.setProgressScreenMessage("Done.");
				if (getProgressMonitor() != null)
					this.getProgressMonitor().done();
				enableAllButtons();

				try {
					sipTraceLogPane.getDocument().insertString(
							sipTraceLogPane.getDocument().getLength(),
							"Error Fetching SIP Trace From Packet Monitor.  Use File/Open "
									+ "open SIP and PTT messages manually.\n",
							ERROR_MESSAGE_TEXT);
					//gui.openFile();
				} catch (Exception e) {

					logger.error("Could not open the log file ", e);

				}

			}

			// If remote retrieval of SIP traces failed, return here
			if (!remoteRetrievalSucceeded)
				return;

			// Now get PTT traces
			this.setProgressScreenMessage("Getting PTT traces...");

			try {

				if (pttTraces == null) {

					// There are no PTT traces so just return.
					this.setProgressScreenMessage("Done.");
					if (this.getProgressMonitor() != null)
						this.getProgressMonitor().done();
					enableAllButtons();
					return;

				}

				Hashtable<String, Color> colorMap = new Hashtable<String, Color>();

				logger.debug("ptt traces = " + pttTraces);
				byte[] stringAsBytes = pttTraces.getBytes();
				ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(
						stringAsBytes);

				PttTraceLoader traceLoader = new PttTraceLoader(
						byteArrayInputStream, 
						//rfssList, 
						colorMap);
				StringBuffer sbuf = new StringBuffer();

				for (MessageData mdata : traceLoader.getRecords()) {
					sbuf.append("F" + mdata.getId() + "\n");
					sbuf.append(mdata.getData());
					sbuf.append("\n\n");

				}

				pttTraceLogPane.getDocument().insertString(
						pttTraceLogPane.getDocument().getLength(),
						sbuf.toString(), INFO_TEXT);

				// Create a new tabbed panel
				this.pttTracePanel = new TracePanel(gui, gui.tabbedPane,
						currentTestName, rfssList, traceLoader.getRecords(),
						colorMap);

				gui.tabbedPane.add("PTT", pttTracePanel);
				gui.tabbedPane.setSelectedComponent(pttTracePanel);
				gui.getCloseMenuItem().setEnabled(true);
				gui.getCloseAllMenuItem().setEnabled(true);

				/*
				 * The PTT rtf buffer.
				 */
				File rtfHeaderFile = new File("doc/rtf-header.txt");
				int length = (int) rtfHeaderFile.length();
				char[] headerbytes = new char[length];
				FileReader fileReader = new FileReader(rtfHeaderFile);
				fileReader.read(headerbytes);
				fileReader.close();
				StringBuffer rtfBuffer = new StringBuffer().append(new String(
						headerbytes));

				for (MessageData messageData : traceLoader.getRecords()) {
					String rtf = messageData.getRtfText();
					rtfBuffer.append(rtf);
				}
				rtfBuffer.append("}");
				this.pttRtfTrace = rtfBuffer.toString();

			} catch (Exception ex) {

				this
						.setProgressScreenMessage("ERROR retrieving PTT trace from "
								+ "packet monitor or tester.");

				try {
					pttTraceLogPane.getDocument().insertString(
							pttTraceLogPane.getDocument().getLength(),
							"Error Fetching PTT Trace From Packet Monitor \n"
									+ ex.getMessage() + "\n",
							ERROR_MESSAGE_TEXT);

				} catch (Exception e) {

					e.printStackTrace();

				}

			}
			this.setProgressScreenMessage("Done.");
			if (getProgressMonitor() != null)
				this.getProgressMonitor().done();
			enableAllButtons();
		} finally {
			try {
				if (toSave)
					this.gui.saveFiles();
			} catch (Exception ex) {
				logger.error("Error saving files", ex);
			}
		}

	}

	/**
	 * @param progressScreenMessage
	 *            The progressScreenMessage to set.
	 */
	public void setProgressScreenMessage(String progressScreenMessage) {
		this.progressScreenMessage = progressScreenMessage;
		if (getProgressMonitor() != null) {
			synchronized (getProgressMonitor()) {
				this.getProgressMonitor().notify();
			}
		}
	}

	/**
	 * @return Returns the progressScreenMessage.
	 */
	public String getProgressScreenMessage() {
		return progressScreenMessage;
	}

	public void setProgressMonitor(ProgressMonitor monitor) {
		this.progressMonitor = monitor;

	}

	public String getCurrentTopologyName() {
		return this.currentTopologyName;
	}

	public void setCurrentTest(String testDir, String topologyFileName) {
		this.currentTestName = testDir + "/testscript.xml";
		this.currentTopologyName = topologyFileName;
	}

	public TestCaseDescriptor getCurrentTest() {
		return (TestCaseDescriptor) this.jlist.getSelectedValue();
	}

	/**
	 * @return the globalTopologyPane
	 */
	public JTextPane getGlobalTopologyPane() {
		return globalTopologyPane;
	}

	public JTextPane getStatusPane() {

		return this.statusLogPane;

	}

	public JTextPane getSystemTopologyPane() {
		return systemTopologyPane;
	}

	public void startTestExecutionTimerTask(int milisec, boolean interactive) {
		// Wait TEST_TIMEOUT before trying to retrieve traces.
		testExecutionTimerTask = new TestExecutionTimerTask(gui, milisec,
				interactive);
		timer.schedule(testExecutionTimerTask, 0, 1000);
	}

	public void saveSipJpegTraces(String fileName) {
		this.sipTracePanel.getDiagramPanel().saveAs(fileName);
	}

	public void savePttJpegTraces(String fileName) {
		this.pttTracePanel.getDiagramPanel().saveAs(fileName);
	}

	public void saveSipRtfTraces(String fileName) {
		if (sipRtfTrace == null)
			return;
		try {

			OutputStream out = new FileOutputStream(new File(fileName));
			out.write(sipRtfTrace.getBytes());
			out.close();
		} catch (Exception e) {
			logger.error("An exception occured while trying to save file", e);
			JOptionPane.showMessageDialog(null,
					"An Error Occured while saving the file",
					"Save File Status", JOptionPane.ERROR_MESSAGE);
		}

	}

	public void savePttRtfTraces(String fileName) {
		if (pttRtfTrace == null)
			return;
		try {

			OutputStream out = new FileOutputStream(new File(fileName));
			out.write(pttRtfTrace.getBytes());
			out.close();
		} catch (Exception e) {
			logger.error("An exception occured while trying to save file", e);
			JOptionPane.showMessageDialog(null,
					"An Error Occured while saving the file",
					"Save File Status", JOptionPane.ERROR_MESSAGE);
		}

	}

	public void setConfigurePane(JTextPane configurePane) {
		this.configurePane = configurePane;
	}

	public JTextPane getConfigurePane() {
		return configurePane;
	}

	ProgressMonitor getProgressMonitor() {
		return progressMonitor;
	}

	/**
	 * Clear the SIP and ptt traces -- called if no ref tracce exists.
	 */
	public void clearSipPttTraces() {
		this.tabbedPane.removeAll();
		
	}
}
