package gov.nist.p25.issi.testlauncher;

import gov.nist.p25.issi.rfss.ISSITesterConstants;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Vector;

import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * An XML parser to parse the test registry. The test registry stores
 * a seuqence of tags - one for every test case.
 * 
 * @author M. Ranganathan
 *
 */
public class TestRegistryParser extends DefaultHandler {
	private XMLReader saxParser;
	private StringBuffer cdatabuffer = new StringBuffer();
	private Hashtable<String,TestCaseDescriptor> testCases; 
	private  Vector<TestCaseDescriptor> testCaseList;
	private TestCaseDescriptor testCaseDescriptor;
	private String testRegistryName;
	//	 /////////////////////////////////////////////////////////////////////////////////////////
	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/testregistry.dtd")) {
				FileReader fileReader = new FileReader( new File("dtd/testregistry.dtd"));
				InputSource inputSource  = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}
	
	@Override
	public void characters(char[] ch, int start, int length) {
		this.cdatabuffer.append(ch, start, length);

	}
	@Override
	public void endElement(String namespaceURI, String local, String name)
			throws SAXException {
		if (name.equals("description")) {
			testCaseDescriptor.setTestDescription (this.cdatabuffer.toString());
			// Reset to ""
			this.cdatabuffer.setLength(0);
		} else if (name.equals("testcase")) {
			this.testCases.put(testCaseDescriptor.toString(),testCaseDescriptor);
			this.testCaseList.add(testCaseDescriptor);
		}
	}
	@Override
	public void startElement(String namespaceURI, String local, String name,
			Attributes attrs) throws SAXException {
		if (name.equals("testcase")) {
			String dir= attrs.getValue("testDirectory");
			String testNum = attrs.getValue("testNumber");
			String category = attrs.getValue("category");
			String topologyName = attrs.getValue("topology");
			if (topologyName == null) topologyName= ISSITesterConstants.TOPOLOGY_XML;
			testCaseDescriptor = new TestCaseDescriptor(dir,topologyName,testNum,category);
		}

	}
	
	public Hashtable<String,TestCaseDescriptor> parse() throws Exception {
		this.saxParser.parse(testRegistryName);
		return this.testCases;
	}
	
	
	
	public TestRegistryParser(String testRegistryName) {
		try {
			this.testRegistryName = testRegistryName;
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
			this.saxParser.setEntityResolver(new MyResolver())	;
			this.testCases =   new Hashtable<String,TestCaseDescriptor>();
			this.testCaseList =  new Vector<TestCaseDescriptor>();
			saxParser.setContentHandler(this);

			saxParser
					.setFeature("http://xml.org/sax/features/validation", true);
		} catch (Exception ex) {
			ex.printStackTrace();
			//System.exit(0);
		}

	}
	
	/**
	 * @return the testCaseList
	 */
	public Vector<TestCaseDescriptor> getTestCaseList() {
		return testCaseList;
	}

}
