package gov.nist.p25.issi.traceverifier;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import gov.nist.p25.issi.issiconfig.RfssConfig;
import gov.nist.p25.issi.issiconfig.SystemTopologyParser;
import gov.nist.p25.issi.issiconfig.TopologyConfig;
import gov.nist.p25.issi.rfss.tester.ISSITesterConfiguration;
import gov.nist.p25.issi.traceviewer.MessageData;
import gov.nist.p25.issi.traceviewer.PttMessageData;
import gov.nist.p25.issi.traceviewer.PttTraceLoader;

public class PttTraceVerifier {
	private static final Logger logger = Logger
			.getLogger(PttTraceVerifier.class);

	public static VerificationRecord testCapturedMessages(
			ISSITesterConfiguration config, String basedir, String testSuiteName,
			String testName, String topologyFileName) {

		VerificationRecord record = new VerificationRecord();
		record.testName = testName;
		record.suiteName = testSuiteName;
		try {
			String fileName = "messagelog.ptt";

			String dirName = basedir + 
					"testrun/"
					+ testSuiteName
					+ "/"
					+ topologyFileName.substring(0, topologyFileName
							.indexOf(".xml")) + ".logs";
			String dirFileName = dirName + "/" + fileName;

			logger.debug("PttTraceVerifier: Directory for captured message = " + dirName);
			File dirFile = new File(dirFileName);
			if (!dirFile.exists())
				return record;

			logger.debug("trace file name  = " + dirFileName);
			FileReader freader = new FileReader(dirFile);
			BufferedReader breader = new BufferedReader(freader);
			String systemTopologyName = dirName + "/systemtopology.xml";
			TopologyConfig systemTopology = new SystemTopologyParser(config)
					.parse(systemTopologyName);
			StringBuffer sbuf = new StringBuffer().append("<pttmessages>\n");
			String line = null;
			while ((line = breader.readLine()) != null) {
				sbuf.append(line + "\n");
			}
			sbuf.append("</pttmessages>\n");
			byte[] bytes = sbuf.toString().getBytes();
			ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
			PttTraceLoader pttTraceLoader = new PttTraceLoader(bis, null);
			freader.close();

			String refDirName = "refmessages/"
					+ testSuiteName
					+ "/"
					+ topologyFileName.substring(0, topologyFileName
							.indexOf(".xml")) + ".logs";
			dirFileName = refDirName + "/" + fileName;

			if (!new File(dirFileName).exists()) {
				logger.debug("Reference messages not found -- returning");
				return record;
			}

			logger.debug("trace file name  = " + dirFileName);

			freader = new FileReader(dirFile);
			breader = new BufferedReader(freader);
			systemTopologyName = refDirName + "/systemtopology.xml";

			sbuf = new StringBuffer().append("<pttmessages>\n");
			line = null;
			while ((line = breader.readLine()) != null) {
				sbuf.append(line + "\n");
			}
			sbuf.append("</pttmessages>\n");
			bytes = sbuf.toString().getBytes();
			bis = new ByteArrayInputStream(bytes);
			PttTraceLoader pttRefTraceLoader = new PttTraceLoader(bis, null);

			for (RfssConfig rfssConfig : systemTopology.getRfssConfigurations()) {

				ArrayList<MessageData> refMessages = pttRefTraceLoader
						.getRecords(rfssConfig.getDomainName());
				if (refMessages.isEmpty()) {
					logger.debug("No reference messages found for 	"
							+ rfssConfig.getRfssName());
					continue;
				}

				ArrayList<MessageData> capturedMessages = pttTraceLoader
						.getRecords(rfssConfig.getDomainName());
				for (MessageData messageData : refMessages) {
					PttMessageData pttMessageData = (PttMessageData) messageData;
					record.statusFlag = StatusFlag.FAIL;
					for (MessageData messageData1 : capturedMessages) {
						PttMessageData pttMessageData1 = (PttMessageData) messageData;
						if (pttMessageData1.match(pttMessageData)) {
							record.statusFlag = StatusFlag.PASS;

							break;
						}
					}

					if (record.statusFlag != StatusFlag.PASS) {
						break;
					}

				}
				logger.debug("Ptt Verification status for Rfss "
						+ rfssConfig.getRfssName() + " Status "
						+ record.statusFlag);
				if ( record.statusFlag != StatusFlag.PASS) break;

			}

			return record;
		} catch (Exception ex) {
			logger.error("Unexepcted exception occured", ex);
			return record;

		}

	}
}
