/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */
package gov.nist.p25.issi.traceviewer;

import java.awt.Dimension;

/**
 * @author steveq@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/07/05 00:20:59 $
 * @since 1.5
 */
public abstract class MessageData {
	
	
	protected   int id;
	
	protected String fromRfssId;

	protected String toRfssId;

	protected String fromPort;

	protected String toPort;

	protected String messageType;

	private String data;

	private String time;

	private int x;

	private int y;

	private Dimension dimension;

	private String colorMapKey;

	private boolean selected = false;
	
	private boolean isSender  = false;
	
	
	public MessageData(String fromRfssId, String toRfssId, String fromPort, String toPort, String messageType,
			String data, String time, String colorMapKey, boolean selected, boolean isSender) {
		this.fromRfssId = fromRfssId;
		this.toRfssId = toRfssId;
		this.fromPort = fromPort;
		this.toPort = toPort;
		this.messageType = messageType;
		this.data = data;
		this.time = time;
		this.setColorMapKey(colorMapKey);
		this.setSelected(selected);
		this.id = -1;
		this.setSender(isSender);
	}

	public String getFromRfssId() {

		return  ! isSender() ?fromRfssId : toRfssId;

	}

	public String getToRfssId() {

		return ! isSender() ? toRfssId : fromRfssId;
	
	}

	public String getFromPort() {

		return !isSender() ? fromPort : toPort;

	}

	public String getToPort() {

		return !isSender() ? toPort : fromPort;

	}

	public String getMessageType() {

		return messageType;

	}

	public String getData() {

		return data;

	}

	public String getTime() {
		return this.time;
	}

	public String colorMapKey() {
		return getColorMapKey();
	}

	/**
	 * @param id the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}
	
	
	
	/**
	 * Get the text as an RTF formatted string
	 */
	public abstract String getRtfText();

	/**
	 * @param y the y to set
	 */
	public void setY(int y) {
		this.y = y;
	}

	/**
	 * @return the y
	 */
	public int getY() {
		return y;
	}

	/**
	 * @param x the x to set
	 */
	public void setX(int x) {
		this.x = x;
	}

	/**
	 * @return the x
	 */
	public int getX() {
		return x;
	}

	/**
	 * @param dimension the dimension to set
	 */
	public void setDimension(Dimension dimension) {
		this.dimension = dimension;
	}

	/**
	 * @return the dimension
	 */
	public Dimension getDimension() {
		return dimension;
	}

	/**
	 * @param colorMapKey the colorMapKey to set
	 */
	private void setColorMapKey(String colorMapKey) {
		this.colorMapKey = colorMapKey;
	}

	/**
	 * @return the colorMapKey
	 */
	 public String getColorMapKey() {
		return colorMapKey;
	}

	/**
	 * @param selected the selected to set
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @param isSender the isSender to set
	 */
	void setSender(boolean isSender) {
		this.isSender = isSender;
	}

	/**
	 * @return the isSender
	 */
	boolean isSender() {
		return isSender;
	}


	
	

}
