/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */

/**
 * This class adds functionality to SIP and PTT message data.
 * 
 * @author mranga@nist.gov
 * @author steveq@nist.gov
 * @version $Revision: 1.5 $, $Date: 2007/10/27 03:23:33 $
 * @since 1.5
 */
package gov.nist.p25.issi.traceviewer;

import java.util.Properties;

import gov.nist.p25.issi.p25payload.P25Payload;
import gov.nist.p25.issi.utils.RtfConstants;

public class PttMessageData extends MessageData implements Comparable<PttMessageData>,RtfConstants{
	
	private String sequenceNumber;
	
	private String rtfData ;

	private Properties properties;
	
	public PttMessageData(
			String remoteRfssId,
			String myRfssId, 
			String myRtpRecvPort, 
			String remoteRtpRecvPort, 
			String messageType, 
			String data, 
			String time, 
			String sequenceNumber,
			String rtfData ,
			boolean selected,
			boolean isSender,
			Properties props) {
		
		super(remoteRfssId, myRfssId, remoteRtpRecvPort, myRtpRecvPort, messageType, 
				data, time, messageType, selected,isSender);
		this.sequenceNumber = sequenceNumber;
		this.rtfData = rtfData;
		this.properties = props;
	
	}

	
	

	@Override
	public String getRtfText() {
		StringBuffer sbuf = new StringBuffer();
		sbuf.append("{\\par F" + id + "\\par }\n");
		/*
		 * Dunno what the bloody hell this is but I cut and paste from a
		 * word document.
		 */
		sbuf.append(PREAMBLE);
		sbuf.append(this.rtfData);
		sbuf.append(END_MESSAGE);
		return sbuf.toString();
	}




	public int compareTo(PttMessageData m2) {
		PttMessageData m1 = this;
		try {
			long ts1 = Long.parseLong(m1.getTime());
			long ts2 = Long.parseLong(m2.getTime());
			if (ts1 < ts2)
				return -1;
			else if (ts1 > ts2)
				return 1;
			else {	
				int sq1 = Integer.parseInt(sequenceNumber);
				int sq2 = Integer.parseInt(m2.sequenceNumber);
				if ( sq1 < sq2) return -1;
				else if (sq1 > sq2) return 1;
				else return 0;
					
			}
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
			//System.exit(0);
			return 0;
		}
	}


    public boolean match( PttMessageData that) {
    	
    	for ( String name : that.properties.stringPropertyNames() ) {
    		if ( this.properties.get(name) == null) return false;
    		if ( ! this.properties.get(name).equals(that.properties.get(name))) return false;
    	}
    	return true;
    }



	/**
	 * @return the properties
	 */
	public Properties getProperties() {
		return properties;
	}
}
