/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */
package gov.nist.p25.issi.traceviewer;

import gov.nist.p25.issi.issiconfig.RfssConfig;

import java.util.*;
import java.awt.*;

/**
 * This class implements an RFSS node and used to display both
 * SIP and PTT traces.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/07/05 00:20:59 $
 * @since 1.5
 */
public class RfssData implements Comparable<RfssData> {

	///////////////////////////////////////////////////////////////////////
	// Variables
	///////////////////////////////////////////////////////////////////////
	
	/** Radical name. */
	private String id;	
	
	/** IP Address. */
	private String address;
	/** Incoming SIP or PTT (RTP) ports. */
	private ArrayList<String> ports;

	/** x coordinate. */
	private int x;

	/** y coordinate. */
	private int y;

	/** Size of RFSS image. */
	private Dimension dimension;

	/** Determines if this RFSS is selected on the screen. */
	private boolean selected = false;

	/** The time we receive the RFSS data. */
	private long time;
	
	/** The rfss configuration if there is one attached to this **/
	
	private RfssConfig rfssConfig;
	
	/** A set of Sessions retrieved at runtime. */
	
	private HashSet<PttSessionInfo> pttSessionInfo;

	private int counter;
	

	///////////////////////////////////////////////////////////////////////
	// Constructors
	///////////////////////////////////////////////////////////////////////
	
	private RfssData () {
		this.ports =    new ArrayList<String>();
		this.pttSessionInfo = new HashSet<PttSessionInfo> ();
	}
	
	/** 
	 * Construct an RfssData object with no time attribute.
	 */
	public RfssData(String id, String address, String port) {
		this();
		this.setId(id);
		this.setAddress(address);
		this.ports.add(port);
		
	}

	
	
	
	
	
	/**
	 * @param id the id to set
	 */
	private void setId(String id) {
		this.id = id;
	}

	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @param address the address to set
	 */
	private void setAddress(String address) {
		this.address = address;
	}

	/**
	 * @return the address
	 */
	public String getAddress() {
		return address;
	}

	

	/**
	 * @return the ports
	 */
	public ArrayList<String> getPorts() {
		return ports;
	}

	/**
	 * @param dimension the dimension to set
	 */
	public void setDimension(Dimension dimension) {
		this.dimension = dimension;
	}

	/**
	 * @return the dimension
	 */
	public Dimension getDimension() {
		return dimension;
	}

	/**
	 * @param selected the selected to set
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}

	/**
	 * @return the selected
	 */
	public boolean isSelected() {
		return selected;
	}

	/**
	 * @param time the time to set
	 */
	public void setTime(long time, int counter) {
		this.time = time;
		this.counter = counter;
	}

	/**
	 * @return the time the first message was seen.
	 */
	public long getTime() {
		return time;
	}

	/**
	 * @param rfssConfig the rfssConfig to set
	 */
	public void setRfssConfig(RfssConfig rfssConfig) {
		this.rfssConfig = rfssConfig;
	}

	/**
	 * @return the rfssConfig
	 */
	public RfssConfig getRfssConfig() {
		return rfssConfig;
	}

	/**
	 * @param x the x to set
	 */
	public void setX(int x) {
		this.x = x;
	}

	/**
	 * @return the x
	 */
	public int getX() {
		return x;
	}

	/**
	 * @param y the y to set
	 */
	public void setY(int y) {
		this.y = y;
	}

	/**
	 * @return the y
	 */
	public int getY() {
		return y;
	}

	/**
	 * Add a port to the list of ports that have been seen at this Rfss.
	 * Note that this is a small list so linear search is fine.
	 * 
	 * @param portToAdd
	 */
	public void addPort(String portToAdd) {
		for ( String p : this.ports) {
			if (p.equals(portToAdd)) return;
		}
		this.ports.add(portToAdd);
		
	}

	/**
	 * @param pttSessionInfo the pttSessionInfo to set
	 */
	public void setPttSessionInfo(HashSet<PttSessionInfo> pttSessionInfo) {
		this.pttSessionInfo = pttSessionInfo;
	}

	/**
	 * @return the pttSessionInfo
	 */
	public HashSet<PttSessionInfo> getPttSessionInfo() {
		return pttSessionInfo;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(RfssData rfssData) {
		
		if (rfssData.getTime() == this.getTime()) {
			return this.counter < rfssData.counter? -1 : 1;
		} else return rfssData.getTime() < this.getTime() ?1 : -1;
	}
	
}
