package gov.nist.p25.issi.traceviewer;


import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Vector;

import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

public class RfssRuntimeDataParser extends DefaultHandler {
	private Hashtable <String, HashSet<PttSessionInfo> >rfssSessionInfoTable;
	private XMLReader saxParser;
	private InputSource iSource;
	private static Logger logger = Logger.getLogger(RfssRuntimeDataParser.class);
	private HashSet<PttSessionInfo> currentPttSessionInfo;		
	private String currentRfssId;
	
	// /////////////////////////////////////////////////////////////////////////////////////////
	// Entity resolver -- allows you to work offline
	// ////////////////////////////////////////////////////////////////////////////////////////
	
	class MyResolver implements EntityResolver {

		public InputSource resolveEntity(String publicId, String systemId)
				throws SAXException, IOException {
			if (systemId
					.equals("http://www-x.antd.nist.gov:8080/p25/issi-emulator/dtd/issi-runtime-data.dtd")) {
				FileReader fileReader = new FileReader( new File("dtd/issi-runtime-data.dtd"));
				InputSource inputSource  = new InputSource(fileReader);
				return inputSource;

			} else {
				return null;
			}
		}

	}
	
	public RfssRuntimeDataParser(InputSource inputSource) {
		try {
			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			this.saxParser = saxParserFactory.newSAXParser().getXMLReader();
			this.saxParser.setEntityResolver(new MyResolver())	;
			this.rfssSessionInfoTable = new Hashtable<String,HashSet<PttSessionInfo>>();
			saxParser.setContentHandler(this);
			this.iSource = inputSource;
			saxParser
					.setFeature("http://xml.org/sax/features/validation", true);
		} catch (Exception ex) {
			ex.printStackTrace();
			logger.fatal("Could not create parser", ex);
		}

	}
	
	public Hashtable<String,HashSet<PttSessionInfo>> parse() throws IOException, SAXException  {
		this.saxParser.parse(this.iSource);
		return this.rfssSessionInfoTable;
	}
	
	@Override
	public void startElement(String namespaceURI, String local, String name,
			Attributes attrs) throws SAXException {
		
		if ( name.equals("rfss-session-info")) {
			this.currentPttSessionInfo = new HashSet<PttSessionInfo>();
			this.currentRfssId = attrs.getValue("rfssId");
		} else if (name.equals("ptt-session")) {
				/* PttSessionInfo (String sessionType, String rfssId,
					String myRtpRecvPort, String remoteRtpRecvPort,
					String remoteIpAddress, String unitId, String linkType ) */
			PttSessionInfo pttSessionInfo = new PttSessionInfo
				(attrs.getValue("sessionType"),
					attrs.getValue("rfssId"),
					attrs.getValue("myRtpRecvPort"),
					attrs.getValue("remoteRtpRecvPort"),
					attrs.getValue("remoteIpAddress"),
					attrs.getValue("unitId"),
					attrs.getValue("linkType")
					);
			this.currentPttSessionInfo.add(pttSessionInfo);
			
		}

	}
	@Override
	public void endElement(String namespaceURI, String local, String name)
			throws SAXException {
	
		if ( name.equals("rfss-session-info")) {
			this.rfssSessionInfoTable.put(this.currentRfssId, this.currentPttSessionInfo);
		}
	}
}
