/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */
package gov.nist.p25.issi.traceviewer;


import java.util.*;

/**
 * Utilities for RFSS data.
 * 
 * @author steveq@nist.gov
 * @author mranga@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/08/17 01:49:45 $
 * @since 1.5
 */
public class RfssUtil {


	public static boolean rfssExists(Set<RfssData> rfssList, 
			String rfssId) {

		boolean exists = false;

		for (RfssData data : rfssList) {

			
			if (data.getId().equals(rfssId)) {

				exists = true;
				

			} 
		}

		return exists;
	}

	/**
	 * Get the RFSS configuration data corresponding to a given rfss ID. This should
	 * be re-written. 
	 * 
	 * @param rfssList
	 * @param rfssId
	 * @return
	 */
	public static RfssData getRfss ( Collection<RfssData> rfssList, 
			String rfssId) {

		boolean exists = false;
		RfssData retval = null;
		
		for (RfssData data : rfssList) {
			
			
			if ( data.getRfssConfig() != null  &&
					data.getRfssConfig().getDomainName().equals(rfssId) ) {
				exists = true;
				retval = data;
				
			} else  if (data.getId().equals(rfssId)) {

				exists = true;
				retval =  data;
			

			} 
		}

		return exists?  retval : null;
	}
	
	


	public static boolean portExists(ArrayList ports, String seekPort) {

		boolean exists = false;

		for (int i = 0; i < ports.size(); i++) {

			String port = (String) ports.get(i);

			if (port.equals(seekPort)) {

				exists = true;
				break;

			} 
		}

		return exists;
	}

}
