/* The software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement and upon your providing appropriate
 * acknowledgments of NIST's ownership of the software.
 */
package gov.nist.p25.issi.traceviewer;


import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * This class implements a trace panel.
 * 
 * @author steveq@nist.gov
 * @author mranga@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/10/02 16:47:10 $
 * @since 1.5
 */
public class TracePanel extends JPanel {

	// /////////////////////////////////////////////////////////////////////
	// Variables
	// /////////////////////////////////////////////////////////////////////

	private JTabbedPane tabbedPane = null;
	
	private String fileName = "";

	private Collection<RfssData> rfssList = null;

	private ArrayList<MessageData> messageList = null;

	private Hashtable<String, Color> colorMap = null;

	private DiagramPanel diagramPanel = null;

	private JScrollPane scrollPane = null;

	private JTextArea textArea = null;

	private JLabel zoomLabel = null;

	
	private JFileChooser fileChooser = null;

	private JFrame frame;

	// /////////////////////////////////////////////////////////////////////
	// Constructor
	// /////////////////////////////////////////////////////////////////////

	public TracePanel(JFrame frame, JTabbedPane tabbedPane, String fileName, 
			Collection<RfssData> rfssList,
			ArrayList<MessageData> messageList,
			Hashtable<String, Color> colorMap) {

		this.tabbedPane = tabbedPane;
		this.fileName = fileName;
		this.rfssList = rfssList;
		// Sort these so they are in the same partial order as the Rfss
		// list.
		
		this.messageList = messageList;
		this.colorMap = colorMap;
		this.frame = frame;

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}

		zoomLabel = new JLabel(" Zoom: 100%");
		zoomLabel.setPreferredSize(new Dimension(100, 22));
		zoomLabel.setMaximumSize(new Dimension(100, 22));

		// ////////////////////////////////
		// Panels
		// ////////////////////////////////

		scrollPane = new JScrollPane();
		
		diagramPanel = new DiagramPanel(this);
		diagramPanel.setData(rfssList, messageList);

		scrollPane.setViewportView(diagramPanel);
		diagramPanel.setPreferredSize(new Dimension(10000, 30000));
		//diagramPanel.setPreferredSize(new Dimension(2000, 1500));
		//diagramPanel.setData(rfssList, messageList);
		scrollPane = new JScrollPane(diagramPanel);

		textArea = new JTextArea();
		textArea.setTabSize(2);
		textArea.setFont(new Font("Tahoma", Font.PLAIN, 11));
		textArea.setEditable(false);
		JScrollPane textScrollPane = new JScrollPane(textArea);

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPane.setOneTouchExpandable(true);
		splitPane.setResizeWeight(0.80);
		splitPane.add(scrollPane);
		splitPane.add(textScrollPane);

		// ////////////////////////////////
		// Frame and Window
		// ////////////////////////////////

		this.setLayout(new BorderLayout());
		this.add(zoomLabel, BorderLayout.NORTH);
		this.add(splitPane, BorderLayout.CENTER);
		this.add(new JPanel(), BorderLayout.SOUTH);

	}

	// /////////////////////////////////////////////////////////////////////
	// Methods
	// /////////////////////////////////////////////////////////////////////

	public void setZoomLabel(ZoomLevel zoomLevel) {

		zoomLabel.setText(" Zoom: " + zoomLevel.toString() + "% ");

	}

	public JScrollPane getScrollPane() {

		return scrollPane;

	}

	

	public JTextArea getTextArea() {

		return textArea;

	}

	public Hashtable<String,Color> getColorMap() {

		return colorMap;

	}

	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("Exit")) {

			//System.exit(0);
			this.frame.remove(this);

		} 
	}

	

	public void writeScroll(JTextArea textArea, String message) {

		textArea.append(message);
		int paneHeight = (int) textArea.getBounds().getHeight();
		textArea.scrollRectToVisible(new Rectangle(new Point(0, paneHeight)));

	}
	
	public DiagramPanel getDiagramPanel() {
		return this.diagramPanel;
	}
	
	

	/**
	 * @return the rfssList
	 */
	public Collection<RfssData> getRfssList() {
		return rfssList;
	}

	

	/**
	 * @return the messageList
	 */
	public ArrayList<MessageData> getMessageList() {
		return messageList;
	}

	

	/**
	 * @return the gui
	 */
	public JTabbedPane getTabbedPane() {
		return tabbedPane;
	}

	

	/**
	 * @return the fileName
	 */
	public String getFileName() {
		return fileName;
	}


	public JFrame getFrame() {
		return frame;
	}

}
