/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.transctlmgr.ptt;

/**
 * This class defines a listener for connection maintenance heartbeats.
 * 
 * @author mranga@nist.gov
 * @version $Revision: 1.4 $, $Date: 2007/04/15 03:39:43 $
 * @since 1.5
 * @see ISSI spec v8, Section 7.2, Figure 65 Summary of Transitions
 */
public interface HeartbeatListener {
	
	/**
	 * Indicates that a heartbeat was received from a peer SMF or MMF.
	 */
	public void receivedHeartbeat(int TSN);
	
	/**
	 * Indicates that a heartbeat query was received from a peer SMF or MMF.
	 */
	public void receivedHeartbeatQuery(PttSessionInterface pttSession, int TSN);
	
	/**
	 * Indicates that a heartbeat timeout has occured.
	 */
	public void heartbeatTimeout(PttSessionInterface pttSession);

}
