/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.transctlmgr.ptt;

import gov.nist.p25.issi.p25payload.*;

/**
 * This class implements an MMF packet event.
 * 
 * @author mranga@nist.gov
 * @version $Revision: 1.7 $, $Date: 2007/03/29 23:11:45 $
 * @since 1.5
 */
public class MmfPacketEvent extends PttEvent {

	
	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** Determines if a de-serialized file is compatible with this class. */
	private static final long serialVersionUID = 0;
	
	/** The MMF session associated with this event. */
	private MmfSession mmfSession = null;

	
	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	/**
	 * Construct an MMF packet event.
	 * @param mmfSession The MMF session associated with this event.
	 * @param p25Payload The PTT packet associated with this event.
	 */
	public MmfPacketEvent(MmfSession mmfSession, P25Payload p25Payload) {

		super(mmfSession, p25Payload);
		this.mmfSession = mmfSession;

	}

	
	/***************************************************************************
	 * Methods
	 **************************************************************************/

	
	/**
	 * Get the MMF session associated with this event.
	 * @return The MMF session associated with this event.
	 */
	public MmfSession getMmfSession() {

		return mmfSession;
		
	}

}
