/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.transctlmgr.ptt;

import gov.nist.p25.issi.p25payload.*;
import gov.nist.rtp.*;

import java.io.IOException;
import java.util.Hashtable;

import org.apache.log4j.Logger;

/**
 * This class implements an MMF session.
 * 
 * @author steveq@nist.gov
 * @author mranga@nist.gov
 * 
 * @version $Revision: 1.21 $, $Date: 2007/08/21 04:06:22 $
 * @since 1.5
 */
public class MmfSession extends PttSession {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** Map of incoming tsn to outgoing tsn */
	private Hashtable<Integer,Integer> incomingTsnToOutgoingTsnMap = new Hashtable<Integer,Integer> ();

	/** The MMF receiver for this session. */
	private MmfReceiver mmfReceiver = null;

	/** The MMF transmitter for this session. */
	private MmfTransmitter mmfTransmitter = null;

	/** The logger for this class. */
	private static Logger logger = Logger.getLogger(MmfSession.class);

	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	/**
	 * Construct an MMF session.
	 */
	MmfSession(RtpSession rtpSession, LinkType linkType, PttManager manager) throws RtpException, IOException {

		super(rtpSession, linkType,manager);
		rtpSession.addRtpListener(this);

		mmfTransmitter = new MmfTransmitter(this);
		mmfReceiver = new MmfReceiver(this);
	
		// rtpSession.setApplicationData(this);

		// Start mute and heartbeat components
		startMuteComponents();
		startHeartbeatComponents();
		

	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Handle incoming PTT packets.
	 */
	void handlePttPacket(P25Payload p25Payload) {

		PacketType packetType = p25Payload.getISSIPacketType().getPacketType();

		if (mmfReceiver.isPacketTypeHandled(packetType)) {

			mmfReceiver.handlePttPacket(p25Payload);

		} else {

			// Silently discard packet

		}

	}

	/*
	 * mranga: I made the reciever and transmitter fields private and added
	 * these getters. These fields were previously public.
	 */

	/**
	 * @return the mmfReceiver
	 */
	public MmfReceiver getMmfReceiver() {
		return mmfReceiver;
	}

	/**
	 * @return the mmfTransmitter
	 */
	public MmfTransmitter getMmfTransmitter() {
		return mmfTransmitter;
	}

	public void addNewTsnToMap(int incomingTsn, int outgoingTsn) {

		this.incomingTsnToOutgoingTsnMap.put(incomingTsn, outgoingTsn);

	}
	
	public int getOutgoingTsn(int incomingTsn) {
		return this.incomingTsnToOutgoingTsnMap.get(incomingTsn);
	}


}
