/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement.
 */
package gov.nist.p25.issi.transctlmgr.ptt;

/**
 * This class implements Mute enumeration.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/03/29 23:11:44 $
 * @since 1.5
 */
enum Mute {
	
	UNMUTED(0, false, "UNMUTED"), 
	MUTED(1, true, "MUTED");
	
	/** The int value of the muted state. */
	private int mValue;
	
	/** The boolean value of the muted state. */
	private boolean isMuted;
	
	/** The description of the muted state. */
	private String description;
	
	
	/**
	 * Construct a Mute object.
	 * @param mValue The value of the muted object.
	 * @param isMuted True if muted, false otherwise.
	 * @param description The description of the muted state.
	 */
	Mute(int mValue, boolean isMuted, String description) {
		
		this.mValue = mValue;
		this.isMuted = isMuted;	
		this.description = description;

	}

	
	/**
	 * Get the muted int value.
	 * @return The muted value: 1=muted, 0=unmuted.
	 */
	int getMValue() {
		
		return mValue;
		
	}
	
	
	/** 
	 * Get the muted boolean value.
	 * @return true if muted, false otherwise.
	 */
	boolean getBooleanValue() {
		
		return isMuted;
		
	}
	

	/** 
	 * Get the description of the muted state.
	 * @return The description of the muted state.
	 */
	public String toString() {
		
		return description;
		
	}

}
