package webverifier;

import gov.nist.p25.issi.traceverifier.Verifier;

import java.io.File;
import java.io.IOException;
import java.io.BufferedWriter;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.FileReader;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.mail.Session;
import javax.mail.Message;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.lang.RandomStringUtils;

/**
 * The servlet that issues the token for web based trace verification.
 * 
 * @author npin
 * 
 */
public class TokenIssuer extends HttpServlet {
	private static int LENGTH = 10;

	private static int sequence = 1;

	private static String SUBJECT = "Please Submit your DIETS Trace for verification!";

	private static String WEBSITE = "http://www-x.antd.nist.gov/proj25/";

	private static String SIGNATURE = "Regards,\n\nAdvanced Network Technologies Division\n"
			+ "National Institute of Standards and Technology\n"
			+ "100 Bureau Drive, Stop 8920\nGaithersburg, MD 20899-8920";

	private HashMap<String, String> tokenList = new HashMap<String, String>();

	private Process verifier = null;

	private String error = "";

	private String smtpServer;

	private String issuerEmailAddress;

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		doPost(request, response);
	}
	
	@Override
	public void init() throws ServletException {
		this.smtpServer = getInitParameter("mail.server");
		this.issuerEmailAddress = getInitParameter(
		"mail.fromaddress");
		if ( smtpServer == null || issuerEmailAddress == null ) throw new ServletException("Configuration error -- \n" +
				"check mail.server and mail.fromaddress init parameters are configured in web.xml");

	}

	/**
	 * We are going to perform the same operations for POST requests as for GET
	 * methods, so this method just sends the request to the doGet method.
	 */

	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		String message = "";
	
		// process trace upload request
		try {

			if (ServletFileUpload.isMultipartContent(request)) {
				// test conflict here, one submission each time
				if ((verifier != null) && (verifier.exitValue() != 0)) {
					error = "Another submission is being processed, plz resubmit your trace file later";
					response.sendError(
							HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
							this.error);
					return;
				}

				try {
					if (!this.getUploadFile(request)) {
						response.sendError(
								HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
								this.error);
					} else {
						// construct response page
						message = "<html><HEAD><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">"
								+ "<TITLE>Trace Uploaded Successfully</TITLE></HEAD><BODY>"
								+ "<p><strong style=\"color:green;\">"
								+ "Uploading done! Trace test result will be sent to your mailbox later.</strong></p>"
								+ "<p><a href=\"http://www-x.antd.nist.gov/proj25/\">Back to the DIETS home page</a>"
								+ "</p></BODY></html>";
						response.setStatus(HttpServletResponse.SC_OK);
						response.getWriter().print(message);

						// send verification result table
						this.verifyTraces();

					}
				} catch (Exception ex) {
					response.sendError(
							HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
							"File upload error: " + ex.getMessage());
				}
				return;
			}

			// process token request
			// NOTE the content-type must be "application/x-www-form-urlencoded"
			String email = request.getParameter("email");
			if ((email == null) || (email.equals(""))) {
				response.sendError(
						HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
						"Error! Invalid email address: " + email);
				return;
			}

			String template = "";
			String token = "";

			// generate a random string
			token = RandomStringUtils.randomAlphanumeric(LENGTH);
			this.tokenList.put(email, token);

			System.out.println("email: " + email + "\ntoken: " + token);

			String requestUri = request.getRequestURL().toString();

			URL uri = new URL(requestUri);
			String host = uri.getHost();
			int port = uri.getPort();
			String traceUploadUrl = "http://" + host + ":" + port
					+ "/diets/submitTrace.html";
			// create a new mail
			template = "* This mail has been sent to you because your email address is used to require a token in our website at "
					+ WEBSITE
					+ "\n* Here is your valid token:\n\n"
					+ token
					+ "\n\nHere is the link to submit your trace file:\n"
					+ traceUploadUrl + "\n\n" + SIGNATURE;
			// send mail
			this.sendMail(email, template, "text/plain");

			// return status with ajax
			response.setContentType("application/xml");
			response.setHeader("Cache-Control", "no-cache");

			// return banner
			response.setStatus(HttpServletResponse.SC_OK);
			message = "<html><HEAD><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">"
					+ "<TITLE>Token Request Processed Successfully</TITLE></HEAD><BODY>"
					+ "<p><strong style=\"color:green;\">"
					+ "Success! A token has been sent to your mailbox.</strong></p>"
					+ "<p><a href=\"http://www-x.antd.nist.gov/proj25/\">Back to the DIETS home page</a>"
					+ "</p></BODY></html>";
			response.getWriter().print(message);

		} catch (Exception ex) {
			response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
					"Server Internal Error! " + ex);

		}
	}

	private void sendMail(String rcpt, Object body, String type)
			throws Exception {
		try {
			Properties props = System.getProperties();

			// -- Attaching to default Session, or we could start a new one --

			Session session = Session.getDefaultInstance(props, null);
	
			props.put("mail.smtp.host", smtpServer);
			// -- Create a new message --
			Message msg = new MimeMessage(session);
			// -- Set the FROM and TO fields --
			msg.setFrom(new InternetAddress(issuerEmailAddress));
			msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(
					rcpt, false));
			// -- CC to the issuer for notification --
			msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(
					issuerEmailAddress, false));
			// -- Set the subject and body text --
			msg.setSubject(SUBJECT);
			msg.setContent(body, type);
			// msg.setText(body);
			msg.setSentDate(new Date());
			// -- Send the message --
			Transport.send(msg);
			System.out.println("Message sent OK.");
		} catch (Exception ex) {
			System.out.println("\nSend mail exception: " + ex);
			throw ex;
		}
	}

	public static boolean deleteDir(File dir) {
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++) {
				boolean success = deleteDir(new File(dir, children[i]));
				if (!success) {
					return false;
				}
			}
		}

		// The directory is now empty so delete it
		return dir.delete();
	}

	private void cleanCache() {
		boolean bool = false;
		File unzippedCache = new File(getServletContext().getRealPath("/")
				+ "unzipped");
		System.out
				.println("unzipped cache: " + unzippedCache.getAbsolutePath());
		if (unzippedCache.exists()) {
			bool = TokenIssuer.deleteDir(unzippedCache);
			System.out.println("Unzipped cache deleted= " + bool);
		}

		File resultCache = new File(getServletContext().getRealPath("/")
				+ "result.html");
		if (resultCache.exists()) {
			bool = resultCache.delete();
			System.out.println("result page deleted= " + bool);
		}
	}

	private void verifyTraces() throws Exception {
		try {
			// launch verifier
			File rootDir = new File(getServletContext().getRealPath("/"));

			System.out.println("Root directory = " + rootDir);
			String classPath = new ParseDotClasspath().parseDotClassPath()
					+ File.pathSeparator + "./diets-1.0.rc1.jar";
			String className = Verifier.class.getCanonicalName();

			// java path MUST be specified for correct calling
			String commands[] = {
					System.getenv().get("JAVA_HOME") + "/bin/java",
					"-classpath",
					classPath,
					className,
					"-traces",
					rootDir.getAbsolutePath() + "/temp/trace_" + sequence
							+ "/testrun.zip" };
			System.out.println(commands[0] + " " + commands[1] + " "
					+ commands[2] + " " + commands[3] + " " + commands[4] + " "
					+ commands[5]);

			// remove unzipped cache
			this.cleanCache();

			// Note that we cahnge to the location of the diets webapp before we
			// exec.

			verifier = Runtime.getRuntime().exec(commands, null, rootDir);
			verifier.getOutputStream().close();
			verifier.getErrorStream().close(); 		
			verifier.waitFor();

			if (verifier.exitValue() == 0) {
				File resultPage = new File(getServletContext().getRealPath("/")
						+ "result.html");
				File rcpts = new File(getServletContext().getRealPath("/")
						+ "temp/trace_" + sequence++ + "/email_targets");
				BufferedReader input = null;
				if (resultPage.exists()) {
					input = new BufferedReader(new FileReader(resultPage));
				} else {
					input = new BufferedReader(new FileReader(
							new File(getServletContext().getRealPath("/")
									+ "error.html")));
				}

				String page = "";
				String rcpt = "";
				String value = "";

				while ((value = input.readLine()) != null) {
					page += value;
				}
				input.close();

				input = new BufferedReader(new FileReader(rcpts));

				while ((rcpt = input.readLine()) != null) {
					System.out.println("email: " + rcpt);
					this.sendMail(rcpt, page, "text/html");
				}
				input.close();
			} else {
				System.out.println("\nverifier terminates code: "
						+ verifier.exitValue());
			}
		} catch (Exception ex) {
			System.out.println("\nVerify trace exception: " + ex.getMessage());
			throw ex;
		}
	}

	private boolean getUploadFile(HttpServletRequest request) throws Exception {
		boolean hasToken = false;
		/*
		 * DiskFileItemFactory diskFileItemFactory = new DiskFileItemFactory();
		 * diskFileItemFactory.setSizeThreshold(40960);
		 * 
		 * File repositoryPath = new File("temp");
		 * diskFileItemFactory.setRepository(repositoryPath);
		 * 
		 * ServletFileUpload servletFileUpload = new
		 * ServletFileUpload(diskFileItemFactory);
		 * servletFileUpload.setSizeMax(81920);
		 */
		ServletFileUpload servletFileUpload = new ServletFileUpload(
				new DiskFileItemFactory());

		List<FileItem> fileItemsList = servletFileUpload.parseRequest(request);

		Iterator<FileItem> it = fileItemsList.iterator();
		String[] targets = null;
		String value = "";
		// verify token first
		while (it.hasNext()) {
			FileItem fileItem = (FileItem) it.next();
			if (fileItem.isFormField()) {
				value = fileItem.getString();
				if (fileItem.getFieldName().equals("token")) {
					// test if token is valid
					if (this.tokenList.containsValue(value)
							|| (value.equals("test"))) {
						hasToken = true;
						// break;
					} else {
						this.error = "Invalid token";
						return false;
					}
				} else if (fileItem.getFieldName().equals("targets")) {
					targets = value.split(",");
				}
			}
		}

		if (!hasToken) {
			this.error = "No token contained";
			return false;
		}

		// get upload file if the token is valid
		it = fileItemsList.iterator();
		while (it.hasNext()) {
			FileItem fileItem = (FileItem) it.next();
			if (!fileItem.isFormField()) {
				// unzip fileItem
				File save = new File(getServletContext().getRealPath("/")
						+ "temp/trace_" + sequence + "/testrun.zip");
				if (save.getParentFile().exists()) {
					TokenIssuer.deleteDir(save.getParentFile());
				}
				save.getParentFile().mkdirs();
				fileItem.write(save);
				// save email addresses
				if ((targets != null) && targets.length > 0) {
					File emails = new File(save.getParentFile(),
							"email_targets");
					BufferedWriter output = new BufferedWriter(new FileWriter(
							emails));
					for (int i = 0; i < targets.length; i++) {
						output.append(targets[i].trim() + "\n");
					}
					output.close();
				}

				return true;
			}
		}

		this.error = "File parsing error";
		return false;
	}
}
