/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class implements an P25 block exception.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.5 $, $Date: 2007/03/29 23:20:33 $
 * @since 1.5
 */
public class P25BlockException extends Exception {

	/***************************************************************************
	 * Constants
	 **************************************************************************/

	/** Constant required by Java5?. */
	public static final long serialVersionUID = 0;

	/** Constant that identifies an incomplete P25 payload. */
	protected final static String INCOMPLETE = "P25 packet is not complete.";

	/** Constant that identifies an incomplete P25 payload. */
	protected final static String INCONSISTENT_BLOCK_COUNT = "Number of block headers in control octect do not match number of block headers in P25 payload.";

	/** Constant that identifies an out-of-range numeric parameter. */
	public final static String OUT_OF_RANGE = "Parameter value out of range.";

	/** Constant that identifies an incorrect length for a byte array. */
	public final static String INCORRECT_BYTE_LENGTH = "Length of byte array is incorrect.";

	/** The logger for this class. */
	private static Logger logger = Logger.getLogger(P25BlockException.class);

	static {
		PropertyConfigurator.configure("log4j.properties");
		// You can add more appenders here if you wish.
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	/**
	 * Construct a P25 block exception.
	 * 
	 * @param exceptionType
	 *            the type of P25 block exception.
	 */
	public P25BlockException(String exceptionType) {
		super(exceptionType);
	}

	/**
	 * Construct a P25 block exception.
	 * 
	 * @param error
	 *            the error message for this exception
	 * @param wrappedException
	 *            the exception object
	 */
	public P25BlockException(String error, Exception wrappedException) {

		super(error, wrappedException);

	}
}
