/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

/**
 * Enumeration of the packet types. This enumerated type is passed in as an
 * argument to the methods of ISSIPacketType
 * 
 * @author mranga@nist.gov
 * @version $Revision: 1.15 $, $Date: 2007/07/24 03:20:38 $
 * @since 1.5
 */
// TODO (steveq)  Not clear that this class is really needed.  All
// of this information is already available in ISSIPacketType.  Recommendation:
// remove this class and modify ISSIPacketType to handle all packet types.
public enum PacketType {

	PTT_TRANSMIT_REQUEST(ISSIPacketType.PTT_TRANSMIT_REQUEST,
			"PTT TRANSMIT REQUEST"),

	PTT_TRANSMIT_GRANT(ISSIPacketType.PTT_TRANSMIT_GRANT, "PTT TRANSMIT GRANT"),

	PTT_TRANSMIT_PROGRESS(ISSIPacketType.PTT_TRANSMIT_PROGRESS,
			"PTT TRANSMIT PROGRESS"),

	PTT_TRANSMIT_END(ISSIPacketType.PTT_TRANSMIT_END, "PTT TRANSMIT END"),

	PTT_TRANSMIT_START(ISSIPacketType.PTT_TRANSMIT_START, "PTT TRANSMIT START"),

	PTT_TRANSMIT_MUTE(ISSIPacketType.PTT_TRANSMIT_MUTE, "PTT TRANSMIT MUTE"),

	PTT_TRANSMIT_UNMUTE(ISSIPacketType.PTT_TRANSMIT_UNMUTE,
			"PTT TRANSMIT UNMUTE"),

	PTT_TRANSMIT_WAIT(ISSIPacketType.PTT_TRANSMIT_WAIT, "PTT TRANSMIT WAIT"),

	PTT_TRANSMIT_DENY(ISSIPacketType.PTT_TRANSMIT_DENY, "PTT TRANSMIT DENY"),

	HEARTBEAT(ISSIPacketType.HEARTBEAT, "PTT HEARTBEAT"),

	HEARTBEAT_QUERY(ISSIPacketType.HEARTBEAT_QUERY, "PTT HEARTBEAT QUERY"),

	// Used only for local indication of connection-related heartbeats
	HEARTBEAT_CONNECTION(ISSIPacketType.HEARTBEAT_CONNECTION,
			"PTT HEARTBEAT CONNECTION"),

	// Used only for local indication of mute transmission-related heartbeats
	HEARTBEAT_MUTE_TRANSMISSION(ISSIPacketType.HEARTBEAT_MUTE_TRANSMISSION,
			"PTT HEARTBEAT MUTE TRANSMISSION"),

	// Used only for local indication of mute transmission-related heartbeats
	HEARTBEAT_UNMUTE_TRANSMISSION(ISSIPacketType.HEARTBEAT_UNMUTE_TRANSMISSION,
			"PTT HEARTBEAT UNMUTE TRANSMISSION");

	/** For quick conversion of type to enumerated type */
	private static PacketType[] typeArray = {

	PacketType.PTT_TRANSMIT_REQUEST, PacketType.PTT_TRANSMIT_GRANT,
			PacketType.PTT_TRANSMIT_PROGRESS, PacketType.PTT_TRANSMIT_END,
			PacketType.PTT_TRANSMIT_START, PacketType.PTT_TRANSMIT_MUTE,
			PacketType.PTT_TRANSMIT_UNMUTE, PacketType.PTT_TRANSMIT_WAIT,
			PacketType.PTT_TRANSMIT_DENY, PacketType.HEARTBEAT,
			PacketType.HEARTBEAT_QUERY, PacketType.HEARTBEAT_CONNECTION,
			PacketType.HEARTBEAT_MUTE_TRANSMISSION,
			PacketType.HEARTBEAT_UNMUTE_TRANSMISSION

	};

	// Do a little static check to make sure that the array is
	// correctly initialized.
	static {

		for (int i = 0; i < typeArray.length; i++) {

			assert (typeArray[i].intValue() == i);

		}

	}

	/**
	 * The type that is stored in the field of this the ISSI packet type
	 * structure as a byte.
	 */
	public int type;

	/**
	 * A description of the type.
	 */
	String typeName;

	/**
	 * Construct a packet type.
	 * 
	 * @param type
	 *            the packet type as it appears in ISSIPacketType
	 * @param typeName
	 *            the descriptive name for the packet type.
	 */
	PacketType(int type, String typeName) {

		this.type = type;
		this.typeName = typeName;

	}

	/**
	 * Get the int representation of the packet type.
	 * 
	 * @return int corresponding to this packet type
	 */
	public int intValue() {

		return type;

	}

	/**
	 * Descriptive name for this packet type.
	 */
	public String toString() {

		return typeName;

	}

	/**
	 * Convert from int form to the Enumerated type.
	 * 
	 * @param actualType
	 * @return corresponding enumerated type.
	 */
	public static PacketType getInstance(int actualType) {

		if (actualType > typeArray.length)
			throw new IllegalArgumentException("out of range arg");

		return typeArray[actualType];

	}

	public static int getValueFromString(String value) {
		for (PacketType pt : PacketType.typeArray) {
			if ( pt.typeName.equals(value)) return pt.type;
		}
		throw new IllegalArgumentException("Illegal packet type description " + value);
	}

}