/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests P25 block headers.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/05/07 03:23:21 $
 * @since 1.5
 */

public class Test_BlockHeaderTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating block headers.
	 */
	public void testControlOctet() {


		// Create block header 1 with empty constructor
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating control octet 1");

		ControlOctet controlOctet1 = new ControlOctet();
		controlOctet1.setS(ByteUtil.getMaxIntValueForNumBits(1));
		controlOctet1.setC(ByteUtil.getMaxIntValueForNumBits(1));
		controlOctet1.setBHC(ByteUtil.getMaxIntValueForNumBits(6));
			
			
		// Generate byte representation from block header 1
		byte[] controlOctetBytes = controlOctet1.getBytes();

			
		// Create block header 2 with block header 1 byte representation
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating control octet 2");

			
		ControlOctet controlOctet2 = new ControlOctet(controlOctetBytes[0]);

		// Test block header equivalence
		assertTrue(controlOctet1.getS() == controlOctet2.getS());
		assertTrue(controlOctet1.getC() == controlOctet2.getC());
		assertTrue(controlOctet1.getBHC() == controlOctet2.getBHC());

		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Control octet 1 = control octet 2");

		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
