/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;
import junit.framework.TestCase;

/**
 * This class tests P25 control octets.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/07/24 03:20:38 $
 * @since 1.5
 */
public class Test_ControlOctetTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	protected static org.apache.log4j.Level LOGGER_LEVEL = 
		org.apache.log4j.Level.DEBUG;

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	/**
	 * Contruct a control octet JUnit test.
	 */
	public Test_ControlOctetTest() {
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating block headers.
	 */
	public void testBlockHeader() {

		try {
			// Create block header 1 with empty constructor
			if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
				logger.info("Creating block header 1");

			BlockHeader blockHeader1 = new BlockHeader();
			blockHeader1.E = ByteUtil.getMaxIntValueForNumBits(1);
			blockHeader1.BT = 1;
			blockHeader1.blockLength =  ByteUtil.getMaxIntValueForNumBits(10);
			blockHeader1.timeStampOffset = ByteUtil.getMaxIntValueForNumBits(14);
			
			
			// Generate byte representation from block header 1
			byte[] blockHeaderBytes = blockHeader1.getBytes();

			
			// Create block header 2 with block header 1 byte representation
			if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
				logger.info("Creating block header 2");

			
			BlockHeader blockHeader2 = new BlockHeader(blockHeaderBytes);

			// Test block header equivalence
			assertTrue(blockHeader1.getE() == blockHeader2.getE());
			assertTrue(blockHeader1.getBT() == blockHeader2.getBT());
			assertTrue(blockHeader1.getTimeStampOffset() == blockHeader2.getTimeStampOffset());
			assertTrue(blockHeader1.getBlockLength() == blockHeader2.getBlockLength());

			if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
				logger.info("Block header 1 = block header 2");
			
		} catch (P25BlockException pbe) {

			pbe.printStackTrace();
			fail("unexpected exception ");

		}
		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
