/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests P25 block headers.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/05/07 03:23:21 $
 * @since 1.5
 */

public class Test_IMBEVoiceBlockTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating IMBE voice block.
	 */
	public void testIMBEVoiceBlock() {


		// Create block header 1 with empty constructor
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating IMBE voice block 1");

		IMBEVoiceBlock imbeVoiceBlock1 = new IMBEVoiceBlock();
		imbeVoiceBlock1.setFT(ByteUtil.getMaxIntValueForNumBits(8));
		imbeVoiceBlock1.setU0(ByteUtil.getMaxIntValueForNumBits(12));
		imbeVoiceBlock1.setU1(ByteUtil.getMaxIntValueForNumBits(12));
		imbeVoiceBlock1.setU2(ByteUtil.getMaxIntValueForNumBits(12));
		imbeVoiceBlock1.setU3(ByteUtil.getMaxIntValueForNumBits(12));	
		imbeVoiceBlock1.setU4(ByteUtil.getMaxIntValueForNumBits(11));
		imbeVoiceBlock1.setU5(ByteUtil.getMaxIntValueForNumBits(11));
		imbeVoiceBlock1.setU6(ByteUtil.getMaxIntValueForNumBits(11));	
		imbeVoiceBlock1.setU7(ByteUtil.getMaxIntValueForNumBits(7));	
		imbeVoiceBlock1.setEt(ByteUtil.getMaxIntValueForNumBits(3));
		imbeVoiceBlock1.setEr(ByteUtil.getMaxIntValueForNumBits(3));
		imbeVoiceBlock1.setM(ByteUtil.getMaxIntValueForNumBits(1));
		imbeVoiceBlock1.setL(ByteUtil.getMaxIntValueForNumBits(1));
		imbeVoiceBlock1.setE4(ByteUtil.getMaxIntValueForNumBits(1));
		imbeVoiceBlock1.setE1(ByteUtil.getMaxIntValueForNumBits(3));
		imbeVoiceBlock1.setSF(ByteUtil.getMaxIntValueForNumBits(2));
		imbeVoiceBlock1.setRsvd(ByteUtil.getMaxIntValueForNumBits(2));
		byte[] additionalFrameData1 = new byte[3];
		imbeVoiceBlock1.setAdditionalFrameData(additionalFrameData1);
		
		// Generate byte representation from block header 1
		byte[] imbeVoiceBlockBytes = imbeVoiceBlock1.getBytes();

			
		// Create block header 2 with block header 1 byte representation
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating control octet 2");

			
		IMBEVoiceBlock imbeVoiceBlock2 = new IMBEVoiceBlock(imbeVoiceBlockBytes);

		// Test block header equivalence
		assertTrue(imbeVoiceBlock1.getFT() == imbeVoiceBlock2.getFT());
		assertTrue(imbeVoiceBlock1.getU0() == imbeVoiceBlock2.getU0());
		assertTrue(imbeVoiceBlock1.getU1() == imbeVoiceBlock2.getU1());
		assertTrue(imbeVoiceBlock1.getU2() == imbeVoiceBlock2.getU2());
		assertTrue(imbeVoiceBlock1.getU3() == imbeVoiceBlock2.getU3());
		assertTrue(imbeVoiceBlock1.getU4() == imbeVoiceBlock2.getU4());
		assertTrue(imbeVoiceBlock1.getU5() == imbeVoiceBlock2.getU5());
		assertTrue(imbeVoiceBlock1.getU6() == imbeVoiceBlock2.getU6());
		assertTrue(imbeVoiceBlock1.getU7() == imbeVoiceBlock2.getU7());
		assertTrue(imbeVoiceBlock1.getEt() == imbeVoiceBlock2.getEt());
		assertTrue(imbeVoiceBlock1.getEr() == imbeVoiceBlock2.getEr());
		assertTrue(imbeVoiceBlock1.getM() == imbeVoiceBlock2.getM());
		assertTrue(imbeVoiceBlock1.getL() == imbeVoiceBlock2.getL());
		assertTrue(imbeVoiceBlock1.getE4() == imbeVoiceBlock2.getE4());
		assertTrue(imbeVoiceBlock1.getE1() == imbeVoiceBlock2.getE1());
		assertTrue(imbeVoiceBlock1.getSF() == imbeVoiceBlock2.getSF());
		assertTrue(imbeVoiceBlock1.getRsvd() == imbeVoiceBlock2.getRsvd());
		assertTrue(imbeVoiceBlock1.getAdditionalFrameData().length == 
			imbeVoiceBlock2.getAdditionalFrameData().length);

		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("IMBE voice block 1 = IMBE voice block 2");

		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
