/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests P25 block headers.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/05/07 03:23:21 $
 * @since 1.5
 */
public class Test_ISSIHeaderWordTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating block headers.
	 */
	public void testISSIHeaderWord() {


		// Create block header 1 with empty constructor
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating ISSI Header Word 1");

		ISSIHeaderWord issiHeaderWord1 = new ISSIHeaderWord();
		byte[] messageIndicatorBytes1 = new byte[9];
		issiHeaderWord1.setMessageIndicator(messageIndicatorBytes1);
		issiHeaderWord1.setAlgId(ByteUtil.getMaxIntValueForNumBits(8));
		issiHeaderWord1.setKeyId(ByteUtil.getMaxIntValueForNumBits(16));
		issiHeaderWord1.setMFID(ByteUtil.getMaxIntValueForNumBits(8));
		issiHeaderWord1.setGroupId(ByteUtil.getMaxIntValueForNumBits(16));
		issiHeaderWord1.setNID(ByteUtil.getMaxIntValueForNumBits(16));
		issiHeaderWord1.setSF(ByteUtil.getMaxIntValueForNumBits(2));	
		issiHeaderWord1.setRsvd(ByteUtil.getMaxIntValueForNumBits(6));
		
		// Generate byte representation from block header 1
		byte[] issiHeaderWordBytes = issiHeaderWord1.getBytes();

			
		// Create block header 2 with block header 1 byte representation
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating ISSI Header Word 2");

			
		ISSIHeaderWord issiHeaderWord2 = new ISSIHeaderWord(issiHeaderWordBytes);

		// Test ISSI header word equivalence
		assertTrue(issiHeaderWord1.getMessageIndicator().length == 
			issiHeaderWord2.getMessageIndicator().length);
		assertTrue(issiHeaderWord1.getAlgId() == issiHeaderWord2.getAlgId());
		assertTrue(issiHeaderWord1.getKeyId() == issiHeaderWord2.getKeyId());
		assertTrue(issiHeaderWord1.getMFID() == issiHeaderWord2.getMFID());
		assertTrue(issiHeaderWord1.getGroupId() == issiHeaderWord2.getGroupId());
		assertTrue(issiHeaderWord1.getNID() == issiHeaderWord2.getNID());
		assertTrue(issiHeaderWord1.getSF() == issiHeaderWord2.getSF());
		assertTrue(issiHeaderWord1.getRsvd() == issiHeaderWord2.getRsvd());
		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("ISSI Header Word 1 = ISSI Header Word 2");

		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
