/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests P25 block headers.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.3 $, $Date: 2007/07/25 19:40:06 $
 * @since 1.5
 */
public class Test_ISSIPacketTypeTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating ISSI Packet Type.
	 */
	public void testISSIPacketType() {


		// Create block header 1 with empty constructor
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating ISSI Packet Type 1");

		ISSIPacketType issiPacketType1 = new ISSIPacketType();
		issiPacketType1.setM(ByteUtil.getMaxIntValueForNumBits(1));
		issiPacketType1.setPT(ByteUtil.getMaxIntValueForNumBits(7));
		issiPacketType1.setSO(128);		// Emergency call
		issiPacketType1.setTranssmissionSequenceNumber(ByteUtil.getMaxIntValueForNumBits(7));
		issiPacketType1.setL(ByteUtil.getMaxIntValueForNumBits(1));
		issiPacketType1.setInterval(ByteUtil.getMaxIntValueForNumBits(8));
		
		// Generate byte representation from block header 1
		byte[] issiPacketTypeBytes = issiPacketType1.getBytes();

			
		// Create block header 2 with block header 1 byte representation
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating ISSI Packet Type 2");

			
		ISSIPacketType issiPacketType2 = new ISSIPacketType(issiPacketTypeBytes);

		// Test ISSI packet type equivalence
		assertTrue(issiPacketType1.getM() == issiPacketType2.getM());
		assertTrue(issiPacketType1.getPT() == issiPacketType2.getPT());
		assertTrue(issiPacketType1.getServiceOptions() == issiPacketType2.getServiceOptions());
		assertTrue(issiPacketType1.getTransmissionSequenceNumber() == issiPacketType2.getTransmissionSequenceNumber());
		assertTrue(issiPacketType1.getL() == issiPacketType2.getL());
		assertTrue(issiPacketType1.getInterval() == issiPacketType2.getInterval());
		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("ISSI Packet Type 1 = ISSI Packet Type 2");

		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
