/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests P25 block headers.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/05/07 03:23:21 $
 * @since 1.5
 */
public class Test_ManufacturerSpecificTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Test creating ISSI Packet Type.
	 */
	public void test() {


		// Create block header 1 with empty constructor
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating Mfr Specific Block 1");

		ManufacturerSpecific mfrSpecific1 = new ManufacturerSpecific();
		mfrSpecific1.setMFID(ByteUtil.getMaxIntValueForNumBits(8));
		mfrSpecific1.setLength(ByteUtil.getMaxIntValueForNumBits(8));
		byte[] variableLengthData = new byte[ByteUtil.getMaxIntValueForNumBits(8)];
		mfrSpecific1.setManufacturerData(variableLengthData);
		
		// Generate byte representation from block header 1
		byte[] mfrSpecificBytes = mfrSpecific1.getBytes();
			
		// Create block header 2 with block header 1 byte representation
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Creating Mfr Specific Block 2");

			
		ManufacturerSpecific mfrSpecific2 = 
			new ManufacturerSpecific(mfrSpecificBytes);

		// Test ISSI packet type equivalence
		assertTrue(mfrSpecific1.getMFID() == mfrSpecific2.getMFID());
		assertTrue(mfrSpecific1.getLength() == mfrSpecific2.getLength());
		assertTrue(mfrSpecific1.getManufacturerData().length == 
			mfrSpecific2.getManufacturerData().length);
		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.info("Mfr Specific Block 1 = Mfr Specific Block 2");

		
		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
