/* This software was developed by employees of the National Institute of
 * Standards and Technology (NIST), an agency of the Federal Government.
 * Pursuant to title 15 United States Code Section 105, works of NIST
 * employees are not subject to copyright protection in the United States
 * and are considered to be in the public domain.  As a result, a formal
 * license is not needed to use the software.
 * 
 * The ISSI Emulator software is provided by NIST as a service and is expressly
 * provided "AS IS".  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT
 * AND DATA ACCURACY.  NIST does not warrant or make any representations
 * regarding the use of the software or the results thereof including, but
 * not limited to, the correctness, accuracy, reliability or usefulness of
 * the software.
 * 
 * Permission to use this software is contingent upon your acceptance
 * of the terms of this agreement
 * .
 */
package gov.nist.p25.issi.p25payload;

import gov.nist.p25.issi.utils.ByteUtil;
import junit.framework.TestCase;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.SimpleLayout;

/**
 * This class tests setting and getting a PTT control word object.
 * 
 * @author steveq@nist.gov
 * @version $Revision: 1.2 $, $Date: 2007/05/07 03:23:21 $
 * @since 1.5
 */
public class Test_PTTControlWordTest extends TestCase {

	/***************************************************************************
	 * Variables
	 **************************************************************************/

	/** The logger for this class. */
	protected static org.apache.log4j.Level LOGGER_LEVEL = org.apache.log4j.Level.DEBUG;

	/** The logger for this class. */
	private static Logger logger = Logger
			.getLogger("gov.nist.p25.issi.p25payload");

	static {
		PropertyConfigurator.configure("log4j.properties");
		// You can add more appenders here if you wish.
		logger.addAppender(new ConsoleAppender(new SimpleLayout()));
	}

	/***************************************************************************
	 * Constructors
	 **************************************************************************/

	/**
	 * Construct a PTT control word test.
	 */
	public Test_PTTControlWordTest() {
	}

	/***************************************************************************
	 * Methods
	 **************************************************************************/

	/**
	 * Set up this JUnit test.
	 */
	public void setUp() {
	}

	/**
	 * Test packing and unpacking a PTT control word block.
	 */
	public void testPackUnpack() {

		/*******************************************************************
		 * Create PTTControlWord #1
		 ******************************************************************/
		PTTControlWord pttControlWord = new PTTControlWord();
		pttControlWord.setWacnId(ByteUtil.getMaxIntValueForNumBits(20));
		pttControlWord.setSystemId(ByteUtil.getMaxIntValueForNumBits(12));
		pttControlWord.setUnitId(ByteUtil.getMaxIntValueForNumBits(24));

		int pl = 1;

		int myTP = (TransmitPriorityType.NORMAL.intValue() + (pl << 4));
		pttControlWord.setTP(myTP);

		byte[] encodedPacket = pttControlWord.getBytes();


		// Test unpack
		PTTControlWord newPTTControlWord = new PTTControlWord(encodedPacket);

		/*******************************************************************
		 * Create PTTControlWord #2
		 ******************************************************************/
		
		assertTrue(newPTTControlWord.getWacnId() == ByteUtil.getMaxIntValueForNumBits(20));
		assertTrue(newPTTControlWord.getSystemId() == ByteUtil.getMaxIntValueForNumBits(12));
		assertTrue(newPTTControlWord.getUnitId() == ByteUtil.getMaxIntValueForNumBits(24));

		int tpInt = newPTTControlWord.getTP();

		assertTrue(newPTTControlWord.getTP() == myTP);

		// Try getPT
		int ptInt = newPTTControlWord.getPT();
		if (logger.isEnabledFor(org.apache.log4j.Level.DEBUG))
			logger.debug("PT = " + ptInt + ": "
					+ ByteUtil.writeBytes(ByteUtil.intToBytes(ptInt)));

		// Try getPL
		int plInt = newPTTControlWord.getPL();

		if (logger.isEnabledFor(org.apache.log4j.Level.DEBUG))
			logger.debug("PL = " + plInt + ": "
					+ ByteUtil.writeBytes(ByteUtil.intToBytes(plInt)));

		// Now change values of PT and PL
		ptInt = TransmitPriorityType.PREEMPTIVE_PRIORITY.intValue();
		plInt = 9; // some random decimal value

		newPTTControlWord.setPTAndPL(ptInt, plInt);

		assertTrue(newPTTControlWord.getPL() == plInt);
		assertTrue(newPTTControlWord.getPT() == ptInt);

		if (logger.isEnabledFor(org.apache.log4j.Level.INFO))
			logger.debug("DONE: All assertions passed.");

	}

}
